type gnutls_cipher_algorithm_t
type gnutls_kx_algorithm_t
type gnutls_mac_algorithm_t
type gnutls_compression_method_t
type gnutls_pk_algorithm_t
type gnutls_sign_algorithm_t
type gnutls_certificate_type_t
type gnutls_digest_algorithm_t = gnutls_mac_algorithm_t
type gnutls_session_t
type gnutls_dh_params_t
type gnutls_priority_t
type gnutls_cipher_hd_t
type gnutls_x509_privkey_t
type gnutls_x509_crl_t
type gnutls_x509_crt_t
type gnutls_certificate_credentials_t
type gnutls_anon_server_credentials_t
type gnutls_anon_client_credentials_t
type gnutls_srp_server_credentials_t
type gnutls_srp_client_credentials_t
type gnutls_psk_server_credentials_t
type gnutls_psk_client_credentials_t
type gnutls_pubkey_t
type gnutls_privkey_t
type gnutls_params_type_t =
  [ `Rsa_export
  | `Dh
  | `Ecdh
  ]
type gnutls_credentials_type_t =
  [ `Certificate
  | `Anon
  | `Srp
  | `Psk
  | `Ia
  ]
type gnutls_alert_level_t =
  [ `Warning
  | `Fatal
  ]
type gnutls_alert_description_t =
  [ `Close_notify
  | `Unexpected_message
  | `Bad_record_mac
  | `Decryption_failed
  | `Record_overflow
  | `Decompression_failure
  | `Handshake_failure
  | `Ssl3_no_certificate
  | `Bad_certificate
  | `Unsupported_certificate
  | `Certificate_revoked
  | `Certificate_expired
  | `Certificate_unknown
  | `Illegal_parameter
  | `Unknown_ca
  | `Access_denied
  | `Decode_error
  | `Decrypt_error
  | `Export_restriction
  | `Protocol_version
  | `Insufficient_security
  | `Internal_error
  | `User_canceled
  | `No_renegotiation
  | `Unsupported_extension
  | `Certificate_unobtainable
  | `Unrecognized_name
  | `Unknown_psk_identity
  | `No_application_protocol
  | `Inner_application_failure
  | `Inner_application_verification
  ]
type gnutls_handshake_description_t =
  [ `Hello_request
  | `Client_hello
  | `Server_hello
  | `Hello_verify_request
  | `New_session_ticket
  | `Certificate_pkt
  | `Server_key_exchange
  | `Certificate_request
  | `Server_hello_done
  | `Certificate_verify
  | `Client_key_exchange
  | `Finished
  | `Supplemental
  | `Change_cipher_spec
  | `Client_hello_v2
  ]
type gnutls_certificate_status_t_flag =
  [ `Invalid
  | `Revoked
  | `Signer_not_found
  | `Signer_not_ca
  | `Insecure_algorithm
  | `Not_activated
  | `Expired
  | `Signature_failure
  | `Revocation_data_superseded
  | `Unexpected_owner
  | `Revocation_data_issued_in_future
  | `Signer_constraints_failure
  | `Mismatch
  | `Purpose_mismatch
  ]
type gnutls_certificate_status_t = gnutls_certificate_status_t_flag list
type gnutls_certificate_request_t =
  [ `Ignore
  | `Request
  | `Require
  ]
type gnutls_certificate_verify_flags_flag =
  [ `Disable_ca_sign
  | `Allow_x509_v1_ca_crt
  | `Do_not_allow_same
  | `Allow_any_x509_v1_ca_crt
  | `Allow_sign_rsa_md2
  | `Allow_sign_rsa_md5
  | `Disable_time_checks
  | `Disable_trusted_time_checks
  | `Do_not_allow_x509_v1_ca_crt
  | `Disable_crl_checks
  ]
type gnutls_certificate_verify_flags = gnutls_certificate_verify_flags_flag list
type gnutls_close_request_t =
  [ `Rdwr
  | `Wr
  ]
type gnutls_protocol_t =
  [ `Ssl3
  | `Tls1_0
  | `Tls1_1
  | `Tls1_2
  | `Dtls0_9
  | `Dtls1_0
  | `Dtls1_2
  | `Version_unknown
  ]
type gnutls_x509_crt_fmt_t =
  [ `Der
  | `Pem
  ]
type gnutls_certificate_print_formats_t =
  [ `Full
  | `Oneline
  | `Unsigned_full
  | `Compact
  | `Full_numbers
  ]
type gnutls_ecc_curve_t =
  [ `Invalid
  | `Secp224r1
  | `Secp256r1
  | `Secp384r1
  | `Secp521r1
  | `Secp192r1
  ]
type gnutls_sec_param_t =
  [ `Unknown
  | `Weak
  | `Low
  | `Normal
  | `High
  | `Ultra
  | `Insecure
  | `Export
  | `Very_weak
  | `Legacy
  ]
type gnutls_init_flags_flag =
  [ `Server
  | `Client
  | `Datagram
  | `Nonblock
  | `No_extensions
  | `No_replay_protection
  ]
type gnutls_init_flags = gnutls_init_flags_flag list
type gnutls_server_name_type_t =
  [ `Dns
  ]
type gnutls_supplemental_data_format_type_t =
  [ `Unknown
  ]
type gnutls_psk_key_flags =
  [ `Raw
  | `Hex
  ]
type gnutls_x509_subject_alt_name_t =
  [ `Dnsname
  | `Rfc822name
  | `Uri
  | `Ipaddress
  | `Othername
  | `Dn
  | `Othername_xmpp
  ]
type gnutls_privkey_type_t =
  [ `X509
  | `Openpgp
  | `Pkcs11
  | `Ext
  ]
type key_usage_flag =
  [ `Digital_signature
  | `Non_repudiation
  | `Key_encipherment
  | `Data_encipherment
  | `Key_agreement
  | `Key_cert_sign
  | `Crl_sign
  | `Encipher_only
  | `Decipher_only
  ]
type key_usage = key_usage_flag list
type gnutls_channel_binding_t =
  [ `Tls_unique
  ]
type gnutls_info_access_what_t =
  [ `Accessmethod_oid
  | `Accesslocation_generalname_type
  | `Uri
  | `Ocsp_uri
  | `Caissuers_uri
  | `Unknown
  ]
type gnutls_certificate_import_flags_flag =
  [ `Fail_if_unsorted
  | `Import_fail_if_exceed
  | `Sort
  ]
type gnutls_certificate_import_flags = gnutls_certificate_import_flags_flag list
type error_code =
  [ `Success
  | `Unknown_compression_algorithm
  | `Unknown_cipher_type
  | `Large_packet
  | `Unsupported_version_packet
  | `Unexpected_packet_length
  | `Invalid_session
  | `Fatal_alert_received
  | `Unexpected_packet
  | `Warning_alert_received
  | `Error_in_finished_packet
  | `Unexpected_handshake_packet
  | `Unknown_cipher_suite
  | `Unwanted_algorithm
  | `Mpi_scan_failed
  | `Decryption_failed
  | `Memory_error
  | `Decompression_failed
  | `Compression_failed
  | `Again
  | `Expired
  | `Db_error
  | `Srp_pwd_error
  | `Insufficient_credentials
  | `Hash_failed
  | `Base64_decoding_error
  | `Mpi_print_failed
  | `Rehandshake
  | `Got_application_data
  | `Record_limit_reached
  | `Encryption_failed
  | `Pk_encryption_failed
  | `Pk_decryption_failed
  | `Pk_sign_failed
  | `X509_unsupported_critical_extension
  | `Key_usage_violation
  | `No_certificate_found
  | `Invalid_request
  | `Short_memory_buffer
  | `Interrupted
  | `Push_error
  | `Pull_error
  | `Received_illegal_parameter
  | `Requested_data_not_available
  | `Pkcs1_wrong_pad
  | `Received_illegal_extension
  | `Internal_error
  | `Dh_prime_unacceptable
  | `File_error
  | `Too_many_empty_packets
  | `Unknown_pk_algorithm
  | `Too_many_handshake_packets
  | `No_temporary_rsa_params
  | `No_compression_algorithms
  | `No_cipher_suites
  | `Openpgp_getkey_failed
  | `Pk_sig_verify_failed
  | `Illegal_srp_username
  | `Srp_pwd_parsing_error
  | `No_temporary_dh_params
  | `Asn1_element_not_found
  | `Asn1_identifier_not_found
  | `Asn1_der_error
  | `Asn1_value_not_found
  | `Asn1_generic_error
  | `Asn1_value_not_valid
  | `Asn1_tag_error
  | `Asn1_tag_implicit
  | `Asn1_type_any_error
  | `Asn1_syntax_error
  | `Asn1_der_overflow
  | `Openpgp_uid_revoked
  | `Certificate_error
  | `Certificate_key_mismatch
  | `Unsupported_certificate_type
  | `X509_unknown_san
  | `Openpgp_fingerprint_unsupported
  | `X509_unsupported_attribute
  | `Unknown_hash_algorithm
  | `Unknown_pkcs_content_type
  | `Unknown_pkcs_bag_type
  | `Invalid_password
  | `Mac_verify_failed
  | `Constraint_error
  | `Warning_ia_iphf_received
  | `Warning_ia_fphf_received
  | `Ia_verify_failed
  | `Unknown_algorithm
  | `Unsupported_signature_algorithm
  | `Safe_renegotiation_failed
  | `Unsafe_renegotiation_denied
  | `Unknown_srp_username
  | `Premature_termination
  | `Base64_encoding_error
  | `Incompatible_crypto_library
  | `Incompatible_libtasn1_library
  | `Openpgp_keyring_error
  | `X509_unsupported_oid
  | `Random_failed
  | `Base64_unexpected_header_error
  | `Openpgp_subkey_error
  | `Crypto_already_registered
  | `Handshake_too_large
  | `Cryptodev_ioctl_error
  | `Cryptodev_device_error
  | `Channel_binding_not_available
  | `Bad_cookie
  | `Openpgp_preferred_key_error
  | `Incompat_dsa_key_with_tls_protocol
  | `Heartbeat_pong_received
  | `Heartbeat_ping_received
  | `Pkcs11_error
  | `Pkcs11_load_error
  | `Parsing_error
  | `Pkcs11_pin_error
  | `Pkcs11_slot_error
  | `Locking_error
  | `Pkcs11_attribute_error
  | `Pkcs11_device_error
  | `Pkcs11_data_error
  | `Pkcs11_unsupported_feature_error
  | `Pkcs11_key_error
  | `Pkcs11_pin_expired
  | `Pkcs11_pin_locked
  | `Pkcs11_session_error
  | `Pkcs11_signature_error
  | `Pkcs11_token_error
  | `Pkcs11_user_error
  | `Crypto_init_failed
  | `Timedout
  | `User_error
  | `Ecc_no_supported_curves
  | `Ecc_unsupported_curve
  | `Pkcs11_requested_object_not_availble
  | `Certificate_list_unsorted
  | `Illegal_parameter
  | `No_priorities_were_set
  | `X509_unsupported_extension
  | `Session_eof
  | `Tpm_error
  | `Tpm_key_password_error
  | `Tpm_srk_password_error
  | `Tpm_session_error
  | `Tpm_key_not_found
  | `Tpm_uninitialized
  | `No_certificate_status
  | `Ocsp_response_error
  | `Random_device_error
  | `Auth_error
  | `No_application_protocol
  | `Sockets_init_error
  | `Key_import_failed
  | `Self_test_error
  | `No_self_test
  | `Lib_in_error_state
  | `Pk_generation_error
  | `Idna_error
  | `Need_fallback
  | `Unimplemented_feature
  ]
type gnutls_pkcs_encrypt_flags_t_flag =
  [ `Plain
  | `Use_pkcs12_3des
  | `Use_pkcs12_arcfour
  | `Use_pkcs12_rc2_40
  | `Use_pbes2_3des
  | `Use_pbes2_des
  | `Use_pbes2_aes_128
  | `Use_pbes2_aes_192
  | `Use_pbes2_aes_256
  | `Null_password
  ]
type gnutls_pkcs_encrypt_flags_t = gnutls_pkcs_encrypt_flags_t_flag list
type empty_flags_flag =
  [ `Dummy
  ]
type empty_flags = empty_flags_flag list
type str_datum = string
type str_datum_p = string
type const_str_datum_p = string
external nettls_init : unit -> unit
 = "net_nettls_init" "net_nettls_init"
external nettls_deinit : unit -> unit
 = "net_nettls_deinit" "net_nettls_deinit"
external gnutls_error_is_fatal : error_code -> bool
 = "net_gnutls_error_is_fatal" "net_gnutls_error_is_fatal"
external gnutls_strerror : error_code -> string
 = "net_gnutls_strerror" "net_gnutls_strerror"
external gnutls_strerror_name : error_code -> string
 = "net_gnutls_strerror_name" "net_gnutls_strerror_name"
external gnutls_check_version : unit -> string
 = "net_gnutls_check_version" "net_gnutls_check_version"
external gnutls_global_set_log_level : int -> unit
 = "net_gnutls_global_set_log_level" "net_gnutls_global_set_log_level"
external gnutls_init : gnutls_init_flags -> gnutls_session_t
 = "net_gnutls_init" "net_gnutls_init"
external gnutls_certificate_server_set_request : gnutls_session_t -> gnutls_certificate_request_t -> unit
 = "net_gnutls_certificate_server_set_request" "net_gnutls_certificate_server_set_request"
external gnutls_certificate_send_x509_rdn_sequence : gnutls_session_t -> bool -> unit
 = "net_gnutls_certificate_send_x509_rdn_sequence" "net_gnutls_certificate_send_x509_rdn_sequence"
external gnutls_certificate_verify_peers2 : gnutls_session_t -> gnutls_certificate_status_t
 = "net_gnutls_certificate_verify_peers2" "net_gnutls_certificate_verify_peers2"
external gnutls_certificate_get_peers : gnutls_session_t -> string array
 = "net_gnutls_certificate_get_peers" "net_gnutls_certificate_get_peers"
external gnutls_record_get_direction : gnutls_session_t -> bool
 = "net_gnutls_record_get_direction" "net_gnutls_record_get_direction"
external gnutls_handshake : gnutls_session_t -> unit
 = "net_gnutls_handshake" "net_gnutls_handshake"
external gnutls_rehandshake : gnutls_session_t -> unit
 = "net_gnutls_rehandshake" "net_gnutls_rehandshake"
external gnutls_record_send : gnutls_session_t -> Netsys_mem.memory -> int -> int
 = "net_gnutls_record_send" "net_gnutls_record_send"
external gnutls_record_recv : gnutls_session_t -> Netsys_mem.memory -> int
 = "net_gnutls_record_recv" "net_gnutls_record_recv"
external gnutls_record_check_pending : gnutls_session_t -> int
 = "net_gnutls_record_check_pending" "net_gnutls_record_check_pending"
external gnutls_bye : gnutls_session_t -> gnutls_close_request_t -> unit
 = "net_gnutls_bye" "net_gnutls_bye"
external gnutls_alert_get : gnutls_session_t -> gnutls_alert_description_t
 = "net_gnutls_alert_get" "net_gnutls_alert_get"
external gnutls_alert_get_name : gnutls_alert_description_t -> string
 = "net_gnutls_alert_get_name" "net_gnutls_alert_get_name"
external gnutls_alert_get_strname : gnutls_alert_description_t -> string
 = "net_gnutls_alert_get_strname" "net_gnutls_alert_get_strname"
external gnutls_alert_send : gnutls_session_t -> gnutls_alert_level_t -> gnutls_alert_description_t -> unit
 = "net_gnutls_alert_send" "net_gnutls_alert_send"
external gnutls_alert_send_appropriate : gnutls_session_t -> int -> unit
 = "net_gnutls_alert_send_appropriate" "net_gnutls_alert_send_appropriate"
external gnutls_error_to_alert : error_code -> gnutls_alert_description_t * gnutls_alert_level_t
 = "net_gnutls_error_to_alert" "net_gnutls_error_to_alert"
external gnutls_priority_init : string -> gnutls_priority_t
 = "net_gnutls_priority_init" "net_gnutls_priority_init"
external gnutls_priority_get_cipher_suite_index : gnutls_priority_t -> int -> int
 = "net_gnutls_priority_get_cipher_suite_index" "net_gnutls_priority_get_cipher_suite_index"
external gnutls_priority_set : gnutls_session_t -> gnutls_priority_t -> unit
 = "net_gnutls_priority_set" "net_gnutls_priority_set"
external gnutls_set_default_priority : gnutls_session_t -> unit
 = "net_gnutls_set_default_priority" "net_gnutls_set_default_priority"
external gnutls_sec_param_to_pk_bits : gnutls_pk_algorithm_t -> gnutls_sec_param_t -> int
 = "net_gnutls_sec_param_to_pk_bits" "net_gnutls_sec_param_to_pk_bits"
external gnutls_pk_bits_to_sec_param : gnutls_pk_algorithm_t -> int -> gnutls_sec_param_t
 = "net_gnutls_pk_bits_to_sec_param" "net_gnutls_pk_bits_to_sec_param"
external gnutls_sec_param_get_name : gnutls_sec_param_t -> string
 = "net_gnutls_sec_param_get_name" "net_gnutls_sec_param_get_name"
external gnutls_session_set_data : gnutls_session_t -> Bytes.t -> unit
 = "net_gnutls_session_set_data" "net_gnutls_session_set_data"
external gnutls_session_get_data : gnutls_session_t -> Bytes.t
 = "net_gnutls_session_get_data" "net_gnutls_session_get_data"
external gnutls_session_get_id : gnutls_session_t -> Bytes.t
 = "net_gnutls_session_get_id" "net_gnutls_session_get_id"
external gnutls_session_is_resumed : gnutls_session_t -> bool
 = "net_gnutls_session_is_resumed" "net_gnutls_session_is_resumed"
external gnutls_db_set_cache_expiration : gnutls_session_t -> int -> unit
 = "net_gnutls_db_set_cache_expiration" "net_gnutls_db_set_cache_expiration"
external gnutls_db_remove_session : gnutls_session_t -> unit
 = "net_gnutls_db_remove_session" "net_gnutls_db_remove_session"
external gnutls_db_check_entry : gnutls_session_t -> string -> int
 = "net_gnutls_db_check_entry" "net_gnutls_db_check_entry"
external gnutls_session_ticket_key_generate : unit -> string
 = "net_gnutls_session_ticket_key_generate" "net_gnutls_session_ticket_key_generate"
external gnutls_session_ticket_enable_client : gnutls_session_t -> unit
 = "net_gnutls_session_ticket_enable_client" "net_gnutls_session_ticket_enable_client"
external gnutls_session_ticket_enable_server : gnutls_session_t -> str_datum_p -> unit
 = "net_gnutls_session_ticket_enable_server" "net_gnutls_session_ticket_enable_server"
external gnutls_prf : gnutls_session_t -> Netsys_mem.memory -> bool -> Netsys_mem.memory -> Netsys_mem.memory -> unit
 = "net_gnutls_prf" "net_gnutls_prf"
external gnutls_prf_raw : gnutls_session_t -> Netsys_mem.memory -> Netsys_mem.memory -> Netsys_mem.memory -> unit
 = "net_gnutls_prf_raw" "net_gnutls_prf_raw"
external gnutls_session_channel_binding : gnutls_session_t -> gnutls_channel_binding_t -> string
 = "net_gnutls_session_channel_binding" "net_gnutls_session_channel_binding"
external gnutls_ecc_curve_get_name : gnutls_ecc_curve_t -> string
 = "net_gnutls_ecc_curve_get_name" "net_gnutls_ecc_curve_get_name"
external gnutls_ecc_curve_get_size : gnutls_ecc_curve_t -> int
 = "net_gnutls_ecc_curve_get_size" "net_gnutls_ecc_curve_get_size"
external gnutls_ecc_curve_get : gnutls_session_t -> gnutls_ecc_curve_t
 = "net_gnutls_ecc_curve_get" "net_gnutls_ecc_curve_get"
external gnutls_cipher_get : gnutls_session_t -> gnutls_cipher_algorithm_t
 = "net_gnutls_cipher_get" "net_gnutls_cipher_get"
external gnutls_kx_get : gnutls_session_t -> gnutls_kx_algorithm_t
 = "net_gnutls_kx_get" "net_gnutls_kx_get"
external gnutls_mac_get : gnutls_session_t -> gnutls_mac_algorithm_t
 = "net_gnutls_mac_get" "net_gnutls_mac_get"
external gnutls_compression_get : gnutls_session_t -> gnutls_compression_method_t
 = "net_gnutls_compression_get" "net_gnutls_compression_get"
external gnutls_certificate_type_get : gnutls_session_t -> gnutls_certificate_type_t
 = "net_gnutls_certificate_type_get" "net_gnutls_certificate_type_get"
external gnutls_cipher_get_key_size : gnutls_cipher_algorithm_t -> int
 = "net_gnutls_cipher_get_key_size" "net_gnutls_cipher_get_key_size"
external gnutls_mac_get_key_size : gnutls_mac_algorithm_t -> int
 = "net_gnutls_mac_get_key_size" "net_gnutls_mac_get_key_size"
external gnutls_cipher_get_name : gnutls_cipher_algorithm_t -> string
 = "net_gnutls_cipher_get_name" "net_gnutls_cipher_get_name"
external gnutls_mac_get_name : gnutls_mac_algorithm_t -> string
 = "net_gnutls_mac_get_name" "net_gnutls_mac_get_name"
external gnutls_compression_get_name : gnutls_compression_method_t -> string
 = "net_gnutls_compression_get_name" "net_gnutls_compression_get_name"
external gnutls_kx_get_name : gnutls_kx_algorithm_t -> string
 = "net_gnutls_kx_get_name" "net_gnutls_kx_get_name"
external gnutls_certificate_type_get_name : gnutls_certificate_type_t -> string
 = "net_gnutls_certificate_type_get_name" "net_gnutls_certificate_type_get_name"
external gnutls_pk_get_name : gnutls_pk_algorithm_t -> string
 = "net_gnutls_pk_get_name" "net_gnutls_pk_get_name"
external gnutls_sign_get_name : gnutls_sign_algorithm_t -> string
 = "net_gnutls_sign_get_name" "net_gnutls_sign_get_name"
external gnutls_mac_get_id : string -> gnutls_mac_algorithm_t
 = "net_gnutls_mac_get_id" "net_gnutls_mac_get_id"
external gnutls_compression_get_id : string -> gnutls_compression_method_t
 = "net_gnutls_compression_get_id" "net_gnutls_compression_get_id"
external gnutls_cipher_get_id : string -> gnutls_cipher_algorithm_t
 = "net_gnutls_cipher_get_id" "net_gnutls_cipher_get_id"
external gnutls_kx_get_id : string -> gnutls_kx_algorithm_t
 = "net_gnutls_kx_get_id" "net_gnutls_kx_get_id"
external gnutls_protocol_get_id : string -> gnutls_protocol_t
 = "net_gnutls_protocol_get_id" "net_gnutls_protocol_get_id"
external gnutls_certificate_type_get_id : string -> gnutls_certificate_type_t
 = "net_gnutls_certificate_type_get_id" "net_gnutls_certificate_type_get_id"
external gnutls_pk_get_id : string -> gnutls_pk_algorithm_t
 = "net_gnutls_pk_get_id" "net_gnutls_pk_get_id"
external gnutls_sign_get_id : string -> gnutls_sign_algorithm_t
 = "net_gnutls_sign_get_id" "net_gnutls_sign_get_id"
external gnutls_cipher_list : unit -> gnutls_cipher_algorithm_t list
 = "net_gnutls_cipher_list" "net_gnutls_cipher_list"
external gnutls_mac_list : unit -> gnutls_mac_algorithm_t list
 = "net_gnutls_mac_list" "net_gnutls_mac_list"
external gnutls_compression_list : unit -> gnutls_compression_method_t list
 = "net_gnutls_compression_list" "net_gnutls_compression_list"
external gnutls_protocol_list : unit -> gnutls_protocol_t list
 = "net_gnutls_protocol_list" "net_gnutls_protocol_list"
external gnutls_certificate_type_list : unit -> gnutls_certificate_type_t list
 = "net_gnutls_certificate_type_list" "net_gnutls_certificate_type_list"
external gnutls_kx_list : unit -> gnutls_kx_algorithm_t list
 = "net_gnutls_kx_list" "net_gnutls_kx_list"
external gnutls_pk_list : unit -> gnutls_pk_algorithm_t list
 = "net_gnutls_pk_list" "net_gnutls_pk_list"
external gnutls_sign_list : unit -> gnutls_sign_algorithm_t list
 = "net_gnutls_sign_list" "net_gnutls_sign_list"
external gnutls_protocol_get_version : gnutls_session_t -> gnutls_protocol_t
 = "net_gnutls_protocol_get_version" "net_gnutls_protocol_get_version"
external gnutls_protocol_get_name : gnutls_protocol_t -> string
 = "net_gnutls_protocol_get_name" "net_gnutls_protocol_get_name"
external gnutls_cipher_suite_get_name : gnutls_kx_algorithm_t -> gnutls_cipher_algorithm_t -> gnutls_mac_algorithm_t -> string
 = "net_gnutls_cipher_suite_get_name" "net_gnutls_cipher_suite_get_name"
external net_have_crypto : unit -> bool
 = "net_net_have_crypto" "net_net_have_crypto"
external gnutls_cipher_init : gnutls_cipher_algorithm_t -> str_datum_p -> str_datum_p -> gnutls_cipher_hd_t
 = "net_gnutls_cipher_init" "net_gnutls_cipher_init"
external gnutls_cipher_encrypt2 : gnutls_cipher_hd_t -> Netsys_mem.memory -> Netsys_mem.memory -> unit
 = "net_gnutls_cipher_encrypt2" "net_gnutls_cipher_encrypt2"
external gnutls_cipher_decrypt2 : gnutls_cipher_hd_t -> Netsys_mem.memory -> Netsys_mem.memory -> unit
 = "net_gnutls_cipher_decrypt2" "net_gnutls_cipher_decrypt2"
external gnutls_cipher_add_auth : gnutls_cipher_hd_t -> Bytes.t -> unit
 = "net_gnutls_cipher_add_auth" "net_gnutls_cipher_add_auth"
external gnutls_cipher_tag : gnutls_cipher_hd_t -> Bytes.t -> unit
 = "net_gnutls_cipher_tag" "net_gnutls_cipher_tag"
external gnutls_handshake_set_private_extensions : gnutls_session_t -> bool -> unit
 = "net_gnutls_handshake_set_private_extensions" "net_gnutls_handshake_set_private_extensions"
external gnutls_handshake_get_last_out : gnutls_session_t -> gnutls_handshake_description_t
 = "net_gnutls_handshake_get_last_out" "net_gnutls_handshake_get_last_out"
external gnutls_handshake_get_last_in : gnutls_session_t -> gnutls_handshake_description_t
 = "net_gnutls_handshake_get_last_in" "net_gnutls_handshake_get_last_in"
external gnutls_handshake_set_max_packet_length : gnutls_session_t -> int -> unit
 = "net_gnutls_handshake_set_max_packet_length" "net_gnutls_handshake_set_max_packet_length"
external gnutls_sign_algorithm_get_requested : gnutls_session_t -> int -> gnutls_sign_algorithm_t
 = "net_gnutls_sign_algorithm_get_requested" "net_gnutls_sign_algorithm_get_requested"
external gnutls_session_enable_compatibility_mode : gnutls_session_t -> unit
 = "net_gnutls_session_enable_compatibility_mode" "net_gnutls_session_enable_compatibility_mode"
external gnutls_record_disable_padding : gnutls_session_t -> unit
 = "net_gnutls_record_disable_padding" "net_gnutls_record_disable_padding"
external gnutls_record_get_max_size : gnutls_session_t -> int
 = "net_gnutls_record_get_max_size" "net_gnutls_record_get_max_size"
external gnutls_record_set_max_size : gnutls_session_t -> int -> unit
 = "net_gnutls_record_set_max_size" "net_gnutls_record_set_max_size"
external gnutls_server_name_set : gnutls_session_t -> gnutls_server_name_type_t -> Bytes.t -> unit
 = "net_gnutls_server_name_set" "net_gnutls_server_name_set"
external gnutls_server_name_get : gnutls_session_t -> int -> Bytes.t * gnutls_server_name_type_t
 = "net_gnutls_server_name_get" "net_gnutls_server_name_get"
external gnutls_safe_renegotiation_status : gnutls_session_t -> bool
 = "net_gnutls_safe_renegotiation_status" "net_gnutls_safe_renegotiation_status"
external gnutls_supplemental_get_name : gnutls_supplemental_data_format_type_t -> string
 = "net_gnutls_supplemental_get_name" "net_gnutls_supplemental_get_name"
external gnutls_credentials_clear : gnutls_session_t -> unit
 = "net_gnutls_credentials_clear" "net_gnutls_credentials_clear"
external gnutls_key_generate : int -> string
 = "net_gnutls_key_generate" "net_gnutls_key_generate"
external gnutls_fingerprint : gnutls_digest_algorithm_t -> str_datum_p -> Bytes.t
 = "net_gnutls_fingerprint" "net_gnutls_fingerprint"
external gnutls_certificate_allocate_credentials : unit -> gnutls_certificate_credentials_t
 = "net_gnutls_certificate_allocate_credentials" "net_gnutls_certificate_allocate_credentials"
external gnutls_certificate_set_x509_system_trust : gnutls_certificate_credentials_t -> unit
 = "net_gnutls_certificate_set_x509_system_trust" "net_gnutls_certificate_set_x509_system_trust"
external gnutls_certificate_set_x509_trust_file : gnutls_certificate_credentials_t -> string -> gnutls_x509_crt_fmt_t -> unit
 = "net_gnutls_certificate_set_x509_trust_file" "net_gnutls_certificate_set_x509_trust_file"
external gnutls_certificate_set_x509_trust_mem : gnutls_certificate_credentials_t -> str_datum_p -> gnutls_x509_crt_fmt_t -> unit
 = "net_gnutls_certificate_set_x509_trust_mem" "net_gnutls_certificate_set_x509_trust_mem"
external gnutls_certificate_set_x509_crl_file : gnutls_certificate_credentials_t -> string -> gnutls_x509_crt_fmt_t -> unit
 = "net_gnutls_certificate_set_x509_crl_file" "net_gnutls_certificate_set_x509_crl_file"
external gnutls_certificate_set_x509_crl_mem : gnutls_certificate_credentials_t -> str_datum_p -> gnutls_x509_crt_fmt_t -> unit
 = "net_gnutls_certificate_set_x509_crl_mem" "net_gnutls_certificate_set_x509_crl_mem"
external gnutls_certificate_set_x509_key_file : gnutls_certificate_credentials_t -> string -> string -> gnutls_x509_crt_fmt_t -> unit
 = "net_gnutls_certificate_set_x509_key_file" "net_gnutls_certificate_set_x509_key_file"
external gnutls_certificate_set_x509_key_mem : gnutls_certificate_credentials_t -> str_datum_p -> str_datum_p -> gnutls_x509_crt_fmt_t -> unit
 = "net_gnutls_certificate_set_x509_key_mem" "net_gnutls_certificate_set_x509_key_mem"
external gnutls_certificate_set_x509_simple_pkcs12_file : gnutls_certificate_credentials_t -> string -> gnutls_x509_crt_fmt_t -> string -> unit
 = "net_gnutls_certificate_set_x509_simple_pkcs12_file" "net_gnutls_certificate_set_x509_simple_pkcs12_file"
external gnutls_certificate_set_x509_simple_pkcs12_mem : gnutls_certificate_credentials_t -> str_datum_p -> gnutls_x509_crt_fmt_t -> string -> unit
 = "net_gnutls_certificate_set_x509_simple_pkcs12_mem" "net_gnutls_certificate_set_x509_simple_pkcs12_mem"
external gnutls_certificate_set_x509_key : gnutls_certificate_credentials_t -> gnutls_x509_crt_t array -> gnutls_x509_privkey_t -> unit
 = "net_gnutls_certificate_set_x509_key" "net_gnutls_certificate_set_x509_key"
external gnutls_certificate_set_x509_trust : gnutls_certificate_credentials_t -> gnutls_x509_crt_t array -> int
 = "net_gnutls_certificate_set_x509_trust" "net_gnutls_certificate_set_x509_trust"
external gnutls_certificate_set_x509_crl : gnutls_certificate_credentials_t -> gnutls_x509_crl_t array -> int
 = "net_gnutls_certificate_set_x509_crl" "net_gnutls_certificate_set_x509_crl"
external gnutls_anon_allocate_server_credentials : unit -> gnutls_anon_server_credentials_t
 = "net_gnutls_anon_allocate_server_credentials" "net_gnutls_anon_allocate_server_credentials"
external gnutls_anon_allocate_client_credentials : unit -> gnutls_anon_client_credentials_t
 = "net_gnutls_anon_allocate_client_credentials" "net_gnutls_anon_allocate_client_credentials"
external gnutls_dh_params_init : unit -> gnutls_dh_params_t
 = "net_gnutls_dh_params_init" "net_gnutls_dh_params_init"
external gnutls_dh_params_import_raw : gnutls_dh_params_t -> str_datum_p -> str_datum_p -> unit
 = "net_gnutls_dh_params_import_raw" "net_gnutls_dh_params_import_raw"
external gnutls_dh_params_import_pkcs3 : gnutls_dh_params_t -> str_datum_p -> gnutls_x509_crt_fmt_t -> unit
 = "net_gnutls_dh_params_import_pkcs3" "net_gnutls_dh_params_import_pkcs3"
external gnutls_dh_params_generate2 : gnutls_dh_params_t -> int -> unit
 = "net_gnutls_dh_params_generate2" "net_gnutls_dh_params_generate2"
external gnutls_dh_params_cpy : gnutls_dh_params_t -> gnutls_dh_params_t -> unit
 = "net_gnutls_dh_params_cpy" "net_gnutls_dh_params_cpy"
external gnutls_certificate_set_dh_params : gnutls_certificate_credentials_t -> gnutls_dh_params_t -> unit
 = "net_gnutls_certificate_set_dh_params" "net_gnutls_certificate_set_dh_params"
external gnutls_anon_set_server_dh_params : gnutls_anon_server_credentials_t -> gnutls_dh_params_t -> unit
 = "net_gnutls_anon_set_server_dh_params" "net_gnutls_anon_set_server_dh_params"
external gnutls_psk_set_server_dh_params : gnutls_psk_server_credentials_t -> gnutls_dh_params_t -> unit
 = "net_gnutls_psk_set_server_dh_params" "net_gnutls_psk_set_server_dh_params"
external gnutls_dh_set_prime_bits : gnutls_session_t -> int -> unit
 = "net_gnutls_dh_set_prime_bits" "net_gnutls_dh_set_prime_bits"
external gnutls_dh_get_secret_bits : gnutls_session_t -> int
 = "net_gnutls_dh_get_secret_bits" "net_gnutls_dh_get_secret_bits"
external gnutls_dh_get_peers_public_bits : gnutls_session_t -> int
 = "net_gnutls_dh_get_peers_public_bits" "net_gnutls_dh_get_peers_public_bits"
external gnutls_dh_get_prime_bits : gnutls_session_t -> int
 = "net_gnutls_dh_get_prime_bits" "net_gnutls_dh_get_prime_bits"
external gnutls_dh_get_group : gnutls_session_t -> string * string
 = "net_gnutls_dh_get_group" "net_gnutls_dh_get_group"
external gnutls_dh_get_pubkey : gnutls_session_t -> string
 = "net_gnutls_dh_get_pubkey" "net_gnutls_dh_get_pubkey"
external gnutls_auth_get_type : gnutls_session_t -> gnutls_credentials_type_t
 = "net_gnutls_auth_get_type" "net_gnutls_auth_get_type"
external gnutls_auth_server_get_type : gnutls_session_t -> gnutls_credentials_type_t
 = "net_gnutls_auth_server_get_type" "net_gnutls_auth_server_get_type"
external gnutls_auth_client_get_type : gnutls_session_t -> gnutls_credentials_type_t
 = "net_gnutls_auth_client_get_type" "net_gnutls_auth_client_get_type"
external gnutls_certificate_get_ours : gnutls_session_t -> const_str_datum_p
 = "net_gnutls_certificate_get_ours" "net_gnutls_certificate_get_ours"
external gnutls_certificate_activation_time_peers : gnutls_session_t -> float
 = "net_gnutls_certificate_activation_time_peers" "net_gnutls_certificate_activation_time_peers"
external gnutls_certificate_expiration_time_peers : gnutls_session_t -> float
 = "net_gnutls_certificate_expiration_time_peers" "net_gnutls_certificate_expiration_time_peers"
external gnutls_certificate_client_get_request_status : gnutls_session_t -> bool
 = "net_gnutls_certificate_client_get_request_status" "net_gnutls_certificate_client_get_request_status"
external gnutls_certificate_get_issuer : gnutls_certificate_credentials_t -> gnutls_x509_crt_t -> empty_flags -> gnutls_x509_crt_t
 = "net_gnutls_certificate_get_issuer" "net_gnutls_certificate_get_issuer"
external gnutls_certificate_set_verify_flags : gnutls_certificate_credentials_t -> gnutls_certificate_verify_flags -> unit
 = "net_gnutls_certificate_set_verify_flags" "net_gnutls_certificate_set_verify_flags"
external gnutls_certificate_set_verify_limits : gnutls_certificate_credentials_t -> int -> int -> unit
 = "net_gnutls_certificate_set_verify_limits" "net_gnutls_certificate_set_verify_limits"
external gnutls_srp_allocate_client_credentials : unit -> gnutls_srp_client_credentials_t
 = "net_gnutls_srp_allocate_client_credentials" "net_gnutls_srp_allocate_client_credentials"
external gnutls_srp_set_client_credentials : gnutls_srp_client_credentials_t -> string -> string -> unit
 = "net_gnutls_srp_set_client_credentials" "net_gnutls_srp_set_client_credentials"
external gnutls_srp_allocate_server_credentials : unit -> gnutls_srp_server_credentials_t
 = "net_gnutls_srp_allocate_server_credentials" "net_gnutls_srp_allocate_server_credentials"
external gnutls_srp_set_server_credentials_file : gnutls_srp_server_credentials_t -> string -> string -> unit
 = "net_gnutls_srp_set_server_credentials_file" "net_gnutls_srp_set_server_credentials_file"
external gnutls_srp_server_get_username : gnutls_session_t -> string
 = "net_gnutls_srp_server_get_username" "net_gnutls_srp_server_get_username"
external gnutls_srp_set_prime_bits : gnutls_session_t -> int -> unit
 = "net_gnutls_srp_set_prime_bits" "net_gnutls_srp_set_prime_bits"
external gnutls_srp_verifier : string -> string -> str_datum_p -> str_datum_p -> str_datum_p -> string
 = "net_gnutls_srp_verifier" "net_gnutls_srp_verifier"
external gnutls_psk_allocate_client_credentials : unit -> gnutls_psk_client_credentials_t
 = "net_gnutls_psk_allocate_client_credentials" "net_gnutls_psk_allocate_client_credentials"
external gnutls_psk_set_client_credentials : gnutls_psk_client_credentials_t -> string -> str_datum_p -> gnutls_psk_key_flags -> unit
 = "net_gnutls_psk_set_client_credentials" "net_gnutls_psk_set_client_credentials"
external gnutls_psk_allocate_server_credentials : unit -> gnutls_psk_server_credentials_t
 = "net_gnutls_psk_allocate_server_credentials" "net_gnutls_psk_allocate_server_credentials"
external gnutls_psk_set_server_credentials_file : gnutls_psk_server_credentials_t -> string -> unit
 = "net_gnutls_psk_set_server_credentials_file" "net_gnutls_psk_set_server_credentials_file"
external gnutls_psk_set_server_credentials_hint : gnutls_psk_server_credentials_t -> string -> unit
 = "net_gnutls_psk_set_server_credentials_hint" "net_gnutls_psk_set_server_credentials_hint"
external gnutls_psk_server_get_username : gnutls_session_t -> string
 = "net_gnutls_psk_server_get_username" "net_gnutls_psk_server_get_username"
external gnutls_psk_client_get_hint : gnutls_session_t -> string
 = "net_gnutls_psk_client_get_hint" "net_gnutls_psk_client_get_hint"
external gnutls_x509_crt_init : unit -> gnutls_x509_crt_t
 = "net_gnutls_x509_crt_init" "net_gnutls_x509_crt_init"
external gnutls_x509_crt_import : gnutls_x509_crt_t -> const_str_datum_p -> gnutls_x509_crt_fmt_t -> unit
 = "net_gnutls_x509_crt_import" "net_gnutls_x509_crt_import"
external gnutls_x509_crt_export : gnutls_x509_crt_t -> gnutls_x509_crt_fmt_t -> Bytes.t
 = "net_gnutls_x509_crt_export" "net_gnutls_x509_crt_export"
external gnutls_x509_crt_get_issuer_dn : gnutls_x509_crt_t -> Bytes.t
 = "net_gnutls_x509_crt_get_issuer_dn" "net_gnutls_x509_crt_get_issuer_dn"
external gnutls_x509_crt_get_issuer_dn_oid : gnutls_x509_crt_t -> int -> Bytes.t
 = "net_gnutls_x509_crt_get_issuer_dn_oid" "net_gnutls_x509_crt_get_issuer_dn_oid"
external gnutls_x509_crt_get_dn : gnutls_x509_crt_t -> Bytes.t
 = "net_gnutls_x509_crt_get_dn" "net_gnutls_x509_crt_get_dn"
external gnutls_x509_crt_get_dn_oid : gnutls_x509_crt_t -> int -> Bytes.t
 = "net_gnutls_x509_crt_get_dn_oid" "net_gnutls_x509_crt_get_dn_oid"
external gnutls_x509_crt_check_hostname : gnutls_x509_crt_t -> string -> bool
 = "net_gnutls_x509_crt_check_hostname" "net_gnutls_x509_crt_check_hostname"
external gnutls_x509_crt_get_signature_algorithm : gnutls_x509_crt_t -> gnutls_sign_algorithm_t
 = "net_gnutls_x509_crt_get_signature_algorithm" "net_gnutls_x509_crt_get_signature_algorithm"
external gnutls_x509_crt_get_signature : gnutls_x509_crt_t -> Bytes.t
 = "net_gnutls_x509_crt_get_signature" "net_gnutls_x509_crt_get_signature"
external gnutls_x509_crt_get_version : gnutls_x509_crt_t -> int
 = "net_gnutls_x509_crt_get_version" "net_gnutls_x509_crt_get_version"
external gnutls_x509_crt_get_key_id : gnutls_x509_crt_t -> empty_flags -> Bytes.t
 = "net_gnutls_x509_crt_get_key_id" "net_gnutls_x509_crt_get_key_id"
external gnutls_x509_crt_get_authority_key_id : gnutls_x509_crt_t -> Bytes.t * bool
 = "net_gnutls_x509_crt_get_authority_key_id" "net_gnutls_x509_crt_get_authority_key_id"
external gnutls_x509_crt_get_subject_key_id : gnutls_x509_crt_t -> Bytes.t * bool
 = "net_gnutls_x509_crt_get_subject_key_id" "net_gnutls_x509_crt_get_subject_key_id"
external gnutls_x509_crt_get_subject_unique_id : gnutls_x509_crt_t -> Bytes.t
 = "net_gnutls_x509_crt_get_subject_unique_id" "net_gnutls_x509_crt_get_subject_unique_id"
external gnutls_x509_crt_get_issuer_unique_id : gnutls_x509_crt_t -> Bytes.t
 = "net_gnutls_x509_crt_get_issuer_unique_id" "net_gnutls_x509_crt_get_issuer_unique_id"
external gnutls_x509_crt_get_authority_info_access : gnutls_x509_crt_t -> int -> gnutls_info_access_what_t -> string * bool
 = "net_gnutls_x509_crt_get_authority_info_access" "net_gnutls_x509_crt_get_authority_info_access"
external gnutls_x509_crt_get_activation_time : gnutls_x509_crt_t -> float
 = "net_gnutls_x509_crt_get_activation_time" "net_gnutls_x509_crt_get_activation_time"
external gnutls_x509_crt_get_expiration_time : gnutls_x509_crt_t -> float
 = "net_gnutls_x509_crt_get_expiration_time" "net_gnutls_x509_crt_get_expiration_time"
external gnutls_x509_crt_get_serial : gnutls_x509_crt_t -> Bytes.t
 = "net_gnutls_x509_crt_get_serial" "net_gnutls_x509_crt_get_serial"
external gnutls_x509_crt_get_pk_algorithm : gnutls_x509_crt_t -> gnutls_pk_algorithm_t * int
 = "net_gnutls_x509_crt_get_pk_algorithm" "net_gnutls_x509_crt_get_pk_algorithm"
external gnutls_x509_crt_get_pk_rsa_raw : gnutls_x509_crt_t -> string * string
 = "net_gnutls_x509_crt_get_pk_rsa_raw" "net_gnutls_x509_crt_get_pk_rsa_raw"
external gnutls_x509_crt_get_pk_dsa_raw : gnutls_x509_crt_t -> string * string * string * string
 = "net_gnutls_x509_crt_get_pk_dsa_raw" "net_gnutls_x509_crt_get_pk_dsa_raw"
external gnutls_x509_crt_get_subject_alt_name : gnutls_x509_crt_t -> int -> gnutls_x509_subject_alt_name_t * Bytes.t * bool
 = "net_gnutls_x509_crt_get_subject_alt_name" "net_gnutls_x509_crt_get_subject_alt_name"
external gnutls_x509_crt_get_subject_alt_othername_oid : gnutls_x509_crt_t -> int -> gnutls_x509_subject_alt_name_t * Bytes.t
 = "net_gnutls_x509_crt_get_subject_alt_othername_oid" "net_gnutls_x509_crt_get_subject_alt_othername_oid"
external gnutls_x509_crt_get_issuer_alt_name : gnutls_x509_crt_t -> int -> gnutls_x509_subject_alt_name_t * Bytes.t * bool
 = "net_gnutls_x509_crt_get_issuer_alt_name" "net_gnutls_x509_crt_get_issuer_alt_name"
external gnutls_x509_crt_get_issuer_alt_othername_oid : gnutls_x509_crt_t -> int -> gnutls_x509_subject_alt_name_t * Bytes.t
 = "net_gnutls_x509_crt_get_issuer_alt_othername_oid" "net_gnutls_x509_crt_get_issuer_alt_othername_oid"
external gnutls_x509_crt_get_ca_status : gnutls_x509_crt_t -> bool
 = "net_gnutls_x509_crt_get_ca_status" "net_gnutls_x509_crt_get_ca_status"
external gnutls_x509_crt_get_basic_constraints : gnutls_x509_crt_t -> bool * bool * int
 = "net_gnutls_x509_crt_get_basic_constraints" "net_gnutls_x509_crt_get_basic_constraints"
external gnutls_x509_crt_get_key_usage : gnutls_x509_crt_t -> key_usage * bool
 = "net_gnutls_x509_crt_get_key_usage" "net_gnutls_x509_crt_get_key_usage"
external gnutls_x509_crt_get_extension_oid : gnutls_x509_crt_t -> int -> Bytes.t
 = "net_gnutls_x509_crt_get_extension_oid" "net_gnutls_x509_crt_get_extension_oid"
external gnutls_x509_crt_get_extension_info : gnutls_x509_crt_t -> int -> Bytes.t * bool
 = "net_gnutls_x509_crt_get_extension_info" "net_gnutls_x509_crt_get_extension_info"
external gnutls_x509_crt_get_extension_data : gnutls_x509_crt_t -> int -> Bytes.t
 = "net_gnutls_x509_crt_get_extension_data" "net_gnutls_x509_crt_get_extension_data"
external gnutls_x509_crt_check_issuer : gnutls_x509_crt_t -> gnutls_x509_crt_t -> bool
 = "net_gnutls_x509_crt_check_issuer" "net_gnutls_x509_crt_check_issuer"
external gnutls_x509_crt_list_verify : gnutls_x509_crt_t array -> gnutls_x509_crt_t array -> gnutls_x509_crl_t array -> gnutls_certificate_verify_flags -> gnutls_certificate_status_t
 = "net_gnutls_x509_crt_list_verify" "net_gnutls_x509_crt_list_verify"
external gnutls_x509_crt_verify : gnutls_x509_crt_t -> gnutls_x509_crt_t array -> gnutls_certificate_verify_flags -> gnutls_certificate_status_t
 = "net_gnutls_x509_crt_verify" "net_gnutls_x509_crt_verify"
external gnutls_x509_crl_verify : gnutls_x509_crl_t -> gnutls_x509_crt_t array -> gnutls_certificate_verify_flags -> gnutls_certificate_status_t
 = "net_gnutls_x509_crl_verify" "net_gnutls_x509_crl_verify"
external gnutls_x509_crt_check_revocation : gnutls_x509_crt_t -> gnutls_x509_crl_t array -> bool
 = "net_gnutls_x509_crt_check_revocation" "net_gnutls_x509_crt_check_revocation"
external gnutls_x509_crt_get_fingerprint : gnutls_x509_crt_t -> gnutls_digest_algorithm_t -> Bytes.t
 = "net_gnutls_x509_crt_get_fingerprint" "net_gnutls_x509_crt_get_fingerprint"
external gnutls_x509_crt_get_key_purpose_oid : gnutls_x509_crt_t -> int -> Bytes.t * bool
 = "net_gnutls_x509_crt_get_key_purpose_oid" "net_gnutls_x509_crt_get_key_purpose_oid"
external gnutls_x509_privkey_init : unit -> gnutls_x509_privkey_t
 = "net_gnutls_x509_privkey_init" "net_gnutls_x509_privkey_init"
external gnutls_x509_privkey_import : gnutls_x509_privkey_t -> const_str_datum_p -> gnutls_x509_crt_fmt_t -> unit
 = "net_gnutls_x509_privkey_import" "net_gnutls_x509_privkey_import"
external gnutls_x509_privkey_import_pkcs8 : gnutls_x509_privkey_t -> const_str_datum_p -> gnutls_x509_crt_fmt_t -> string -> gnutls_pkcs_encrypt_flags_t -> unit
 = "net_gnutls_x509_privkey_import_pkcs8" "net_gnutls_x509_privkey_import_pkcs8"
external gnutls_x509_crl_init : unit -> gnutls_x509_crl_t
 = "net_gnutls_x509_crl_init" "net_gnutls_x509_crl_init"
external gnutls_x509_crl_import : gnutls_x509_crl_t -> const_str_datum_p -> gnutls_x509_crt_fmt_t -> unit
 = "net_gnutls_x509_crl_import" "net_gnutls_x509_crl_import"
external net_have_pubkey : unit -> bool
 = "net_net_have_pubkey" "net_net_have_pubkey"
external gnutls_pubkey_init : unit -> gnutls_pubkey_t
 = "net_gnutls_pubkey_init" "net_gnutls_pubkey_init"
external gnutls_privkey_init : unit -> gnutls_privkey_t
 = "net_gnutls_privkey_init" "net_gnutls_privkey_init"
external gnutls_pubkey_import : gnutls_pubkey_t -> const_str_datum_p -> gnutls_x509_crt_fmt_t -> unit
 = "net_gnutls_pubkey_import" "net_gnutls_pubkey_import"
external gnutls_pubkey_import_url : gnutls_pubkey_t -> string -> int -> unit
 = "net_gnutls_pubkey_import_url" "net_gnutls_pubkey_import_url"
external gnutls_pubkey_import_privkey : gnutls_pubkey_t -> gnutls_privkey_t -> int -> int -> unit
 = "net_gnutls_pubkey_import_privkey" "net_gnutls_pubkey_import_privkey"
external gnutls_privkey_import_x509 : gnutls_privkey_t -> gnutls_x509_privkey_t -> int -> unit
 = "net_gnutls_privkey_import_x509" "net_gnutls_privkey_import_x509"
external gnutls_pubkey_encrypt_data : gnutls_pubkey_t -> int -> const_str_datum_p -> string
 = "net_gnutls_pubkey_encrypt_data" "net_gnutls_pubkey_encrypt_data"
external gnutls_privkey_decrypt_data : gnutls_privkey_t -> int -> const_str_datum_p -> string
 = "net_gnutls_privkey_decrypt_data" "net_gnutls_privkey_decrypt_data"
external gnutls_privkey_sign_data : gnutls_privkey_t -> gnutls_digest_algorithm_t -> int -> const_str_datum_p -> string
 = "net_gnutls_privkey_sign_data" "net_gnutls_privkey_sign_data"
external gnutls_pubkey_verify_data2 : gnutls_pubkey_t -> gnutls_sign_algorithm_t -> int -> const_str_datum_p -> const_str_datum_p -> unit
 = "net_gnutls_pubkey_verify_data2" "net_gnutls_pubkey_verify_data2"
let b_error_of_name name =
  match name with
  | "GNUTLS_E_SUCCESS" -> `Success
  | "GNUTLS_E_UNKNOWN_COMPRESSION_ALGORITHM" -> `Unknown_compression_algorithm
  | "GNUTLS_E_UNKNOWN_CIPHER_TYPE" -> `Unknown_cipher_type
  | "GNUTLS_E_LARGE_PACKET" -> `Large_packet
  | "GNUTLS_E_UNSUPPORTED_VERSION_PACKET" -> `Unsupported_version_packet
  | "GNUTLS_E_UNEXPECTED_PACKET_LENGTH" -> `Unexpected_packet_length
  | "GNUTLS_E_INVALID_SESSION" -> `Invalid_session
  | "GNUTLS_E_FATAL_ALERT_RECEIVED" -> `Fatal_alert_received
  | "GNUTLS_E_UNEXPECTED_PACKET" -> `Unexpected_packet
  | "GNUTLS_E_WARNING_ALERT_RECEIVED" -> `Warning_alert_received
  | "GNUTLS_E_ERROR_IN_FINISHED_PACKET" -> `Error_in_finished_packet
  | "GNUTLS_E_UNEXPECTED_HANDSHAKE_PACKET" -> `Unexpected_handshake_packet
  | "GNUTLS_E_UNKNOWN_CIPHER_SUITE" -> `Unknown_cipher_suite
  | "GNUTLS_E_UNWANTED_ALGORITHM" -> `Unwanted_algorithm
  | "GNUTLS_E_MPI_SCAN_FAILED" -> `Mpi_scan_failed
  | "GNUTLS_E_DECRYPTION_FAILED" -> `Decryption_failed
  | "GNUTLS_E_MEMORY_ERROR" -> `Memory_error
  | "GNUTLS_E_DECOMPRESSION_FAILED" -> `Decompression_failed
  | "GNUTLS_E_COMPRESSION_FAILED" -> `Compression_failed
  | "GNUTLS_E_AGAIN" -> `Again
  | "GNUTLS_E_EXPIRED" -> `Expired
  | "GNUTLS_E_DB_ERROR" -> `Db_error
  | "GNUTLS_E_SRP_PWD_ERROR" -> `Srp_pwd_error
  | "GNUTLS_E_INSUFFICIENT_CREDENTIALS" -> `Insufficient_credentials
  | "GNUTLS_E_HASH_FAILED" -> `Hash_failed
  | "GNUTLS_E_BASE64_DECODING_ERROR" -> `Base64_decoding_error
  | "GNUTLS_E_MPI_PRINT_FAILED" -> `Mpi_print_failed
  | "GNUTLS_E_REHANDSHAKE" -> `Rehandshake
  | "GNUTLS_E_GOT_APPLICATION_DATA" -> `Got_application_data
  | "GNUTLS_E_RECORD_LIMIT_REACHED" -> `Record_limit_reached
  | "GNUTLS_E_ENCRYPTION_FAILED" -> `Encryption_failed
  | "GNUTLS_E_PK_ENCRYPTION_FAILED" -> `Pk_encryption_failed
  | "GNUTLS_E_PK_DECRYPTION_FAILED" -> `Pk_decryption_failed
  | "GNUTLS_E_PK_SIGN_FAILED" -> `Pk_sign_failed
  | "GNUTLS_E_X509_UNSUPPORTED_CRITICAL_EXTENSION" -> `X509_unsupported_critical_extension
  | "GNUTLS_E_KEY_USAGE_VIOLATION" -> `Key_usage_violation
  | "GNUTLS_E_NO_CERTIFICATE_FOUND" -> `No_certificate_found
  | "GNUTLS_E_INVALID_REQUEST" -> `Invalid_request
  | "GNUTLS_E_SHORT_MEMORY_BUFFER" -> `Short_memory_buffer
  | "GNUTLS_E_INTERRUPTED" -> `Interrupted
  | "GNUTLS_E_PUSH_ERROR" -> `Push_error
  | "GNUTLS_E_PULL_ERROR" -> `Pull_error
  | "GNUTLS_E_RECEIVED_ILLEGAL_PARAMETER" -> `Received_illegal_parameter
  | "GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE" -> `Requested_data_not_available
  | "GNUTLS_E_PKCS1_WRONG_PAD" -> `Pkcs1_wrong_pad
  | "GNUTLS_E_RECEIVED_ILLEGAL_EXTENSION" -> `Received_illegal_extension
  | "GNUTLS_E_INTERNAL_ERROR" -> `Internal_error
  | "GNUTLS_E_DH_PRIME_UNACCEPTABLE" -> `Dh_prime_unacceptable
  | "GNUTLS_E_FILE_ERROR" -> `File_error
  | "GNUTLS_E_TOO_MANY_EMPTY_PACKETS" -> `Too_many_empty_packets
  | "GNUTLS_E_UNKNOWN_PK_ALGORITHM" -> `Unknown_pk_algorithm
  | "GNUTLS_E_TOO_MANY_HANDSHAKE_PACKETS" -> `Too_many_handshake_packets
  | "GNUTLS_E_NO_TEMPORARY_RSA_PARAMS" -> `No_temporary_rsa_params
  | "GNUTLS_E_NO_COMPRESSION_ALGORITHMS" -> `No_compression_algorithms
  | "GNUTLS_E_NO_CIPHER_SUITES" -> `No_cipher_suites
  | "GNUTLS_E_OPENPGP_GETKEY_FAILED" -> `Openpgp_getkey_failed
  | "GNUTLS_E_PK_SIG_VERIFY_FAILED" -> `Pk_sig_verify_failed
  | "GNUTLS_E_ILLEGAL_SRP_USERNAME" -> `Illegal_srp_username
  | "GNUTLS_E_SRP_PWD_PARSING_ERROR" -> `Srp_pwd_parsing_error
  | "GNUTLS_E_NO_TEMPORARY_DH_PARAMS" -> `No_temporary_dh_params
  | "GNUTLS_E_ASN1_ELEMENT_NOT_FOUND" -> `Asn1_element_not_found
  | "GNUTLS_E_ASN1_IDENTIFIER_NOT_FOUND" -> `Asn1_identifier_not_found
  | "GNUTLS_E_ASN1_DER_ERROR" -> `Asn1_der_error
  | "GNUTLS_E_ASN1_VALUE_NOT_FOUND" -> `Asn1_value_not_found
  | "GNUTLS_E_ASN1_GENERIC_ERROR" -> `Asn1_generic_error
  | "GNUTLS_E_ASN1_VALUE_NOT_VALID" -> `Asn1_value_not_valid
  | "GNUTLS_E_ASN1_TAG_ERROR" -> `Asn1_tag_error
  | "GNUTLS_E_ASN1_TAG_IMPLICIT" -> `Asn1_tag_implicit
  | "GNUTLS_E_ASN1_TYPE_ANY_ERROR" -> `Asn1_type_any_error
  | "GNUTLS_E_ASN1_SYNTAX_ERROR" -> `Asn1_syntax_error
  | "GNUTLS_E_ASN1_DER_OVERFLOW" -> `Asn1_der_overflow
  | "GNUTLS_E_OPENPGP_UID_REVOKED" -> `Openpgp_uid_revoked
  | "GNUTLS_E_CERTIFICATE_ERROR" -> `Certificate_error
  | "GNUTLS_E_CERTIFICATE_KEY_MISMATCH" -> `Certificate_key_mismatch
  | "GNUTLS_E_UNSUPPORTED_CERTIFICATE_TYPE" -> `Unsupported_certificate_type
  | "GNUTLS_E_X509_UNKNOWN_SAN" -> `X509_unknown_san
  | "GNUTLS_E_OPENPGP_FINGERPRINT_UNSUPPORTED" -> `Openpgp_fingerprint_unsupported
  | "GNUTLS_E_X509_UNSUPPORTED_ATTRIBUTE" -> `X509_unsupported_attribute
  | "GNUTLS_E_UNKNOWN_HASH_ALGORITHM" -> `Unknown_hash_algorithm
  | "GNUTLS_E_UNKNOWN_PKCS_CONTENT_TYPE" -> `Unknown_pkcs_content_type
  | "GNUTLS_E_UNKNOWN_PKCS_BAG_TYPE" -> `Unknown_pkcs_bag_type
  | "GNUTLS_E_INVALID_PASSWORD" -> `Invalid_password
  | "GNUTLS_E_MAC_VERIFY_FAILED" -> `Mac_verify_failed
  | "GNUTLS_E_CONSTRAINT_ERROR" -> `Constraint_error
  | "GNUTLS_E_WARNING_IA_IPHF_RECEIVED" -> `Warning_ia_iphf_received
  | "GNUTLS_E_WARNING_IA_FPHF_RECEIVED" -> `Warning_ia_fphf_received
  | "GNUTLS_E_IA_VERIFY_FAILED" -> `Ia_verify_failed
  | "GNUTLS_E_UNKNOWN_ALGORITHM" -> `Unknown_algorithm
  | "GNUTLS_E_UNSUPPORTED_SIGNATURE_ALGORITHM" -> `Unsupported_signature_algorithm
  | "GNUTLS_E_SAFE_RENEGOTIATION_FAILED" -> `Safe_renegotiation_failed
  | "GNUTLS_E_UNSAFE_RENEGOTIATION_DENIED" -> `Unsafe_renegotiation_denied
  | "GNUTLS_E_UNKNOWN_SRP_USERNAME" -> `Unknown_srp_username
  | "GNUTLS_E_PREMATURE_TERMINATION" -> `Premature_termination
  | "GNUTLS_E_BASE64_ENCODING_ERROR" -> `Base64_encoding_error
  | "GNUTLS_E_INCOMPATIBLE_CRYPTO_LIBRARY" -> `Incompatible_crypto_library
  | "GNUTLS_E_INCOMPATIBLE_LIBTASN1_LIBRARY" -> `Incompatible_libtasn1_library
  | "GNUTLS_E_OPENPGP_KEYRING_ERROR" -> `Openpgp_keyring_error
  | "GNUTLS_E_X509_UNSUPPORTED_OID" -> `X509_unsupported_oid
  | "GNUTLS_E_RANDOM_FAILED" -> `Random_failed
  | "GNUTLS_E_BASE64_UNEXPECTED_HEADER_ERROR" -> `Base64_unexpected_header_error
  | "GNUTLS_E_OPENPGP_SUBKEY_ERROR" -> `Openpgp_subkey_error
  | "GNUTLS_E_CRYPTO_ALREADY_REGISTERED" -> `Crypto_already_registered
  | "GNUTLS_E_HANDSHAKE_TOO_LARGE" -> `Handshake_too_large
  | "GNUTLS_E_CRYPTODEV_IOCTL_ERROR" -> `Cryptodev_ioctl_error
  | "GNUTLS_E_CRYPTODEV_DEVICE_ERROR" -> `Cryptodev_device_error
  | "GNUTLS_E_CHANNEL_BINDING_NOT_AVAILABLE" -> `Channel_binding_not_available
  | "GNUTLS_E_BAD_COOKIE" -> `Bad_cookie
  | "GNUTLS_E_OPENPGP_PREFERRED_KEY_ERROR" -> `Openpgp_preferred_key_error
  | "GNUTLS_E_INCOMPAT_DSA_KEY_WITH_TLS_PROTOCOL" -> `Incompat_dsa_key_with_tls_protocol
  | "GNUTLS_E_HEARTBEAT_PONG_RECEIVED" -> `Heartbeat_pong_received
  | "GNUTLS_E_HEARTBEAT_PING_RECEIVED" -> `Heartbeat_ping_received
  | "GNUTLS_E_PKCS11_ERROR" -> `Pkcs11_error
  | "GNUTLS_E_PKCS11_LOAD_ERROR" -> `Pkcs11_load_error
  | "GNUTLS_E_PARSING_ERROR" -> `Parsing_error
  | "GNUTLS_E_PKCS11_PIN_ERROR" -> `Pkcs11_pin_error
  | "GNUTLS_E_PKCS11_SLOT_ERROR" -> `Pkcs11_slot_error
  | "GNUTLS_E_LOCKING_ERROR" -> `Locking_error
  | "GNUTLS_E_PKCS11_ATTRIBUTE_ERROR" -> `Pkcs11_attribute_error
  | "GNUTLS_E_PKCS11_DEVICE_ERROR" -> `Pkcs11_device_error
  | "GNUTLS_E_PKCS11_DATA_ERROR" -> `Pkcs11_data_error
  | "GNUTLS_E_PKCS11_UNSUPPORTED_FEATURE_ERROR" -> `Pkcs11_unsupported_feature_error
  | "GNUTLS_E_PKCS11_KEY_ERROR" -> `Pkcs11_key_error
  | "GNUTLS_E_PKCS11_PIN_EXPIRED" -> `Pkcs11_pin_expired
  | "GNUTLS_E_PKCS11_PIN_LOCKED" -> `Pkcs11_pin_locked
  | "GNUTLS_E_PKCS11_SESSION_ERROR" -> `Pkcs11_session_error
  | "GNUTLS_E_PKCS11_SIGNATURE_ERROR" -> `Pkcs11_signature_error
  | "GNUTLS_E_PKCS11_TOKEN_ERROR" -> `Pkcs11_token_error
  | "GNUTLS_E_PKCS11_USER_ERROR" -> `Pkcs11_user_error
  | "GNUTLS_E_CRYPTO_INIT_FAILED" -> `Crypto_init_failed
  | "GNUTLS_E_TIMEDOUT" -> `Timedout
  | "GNUTLS_E_USER_ERROR" -> `User_error
  | "GNUTLS_E_ECC_NO_SUPPORTED_CURVES" -> `Ecc_no_supported_curves
  | "GNUTLS_E_ECC_UNSUPPORTED_CURVE" -> `Ecc_unsupported_curve
  | "GNUTLS_E_PKCS11_REQUESTED_OBJECT_NOT_AVAILBLE" -> `Pkcs11_requested_object_not_availble
  | "GNUTLS_E_CERTIFICATE_LIST_UNSORTED" -> `Certificate_list_unsorted
  | "GNUTLS_E_ILLEGAL_PARAMETER" -> `Illegal_parameter
  | "GNUTLS_E_NO_PRIORITIES_WERE_SET" -> `No_priorities_were_set
  | "GNUTLS_E_X509_UNSUPPORTED_EXTENSION" -> `X509_unsupported_extension
  | "GNUTLS_E_SESSION_EOF" -> `Session_eof
  | "GNUTLS_E_TPM_ERROR" -> `Tpm_error
  | "GNUTLS_E_TPM_KEY_PASSWORD_ERROR" -> `Tpm_key_password_error
  | "GNUTLS_E_TPM_SRK_PASSWORD_ERROR" -> `Tpm_srk_password_error
  | "GNUTLS_E_TPM_SESSION_ERROR" -> `Tpm_session_error
  | "GNUTLS_E_TPM_KEY_NOT_FOUND" -> `Tpm_key_not_found
  | "GNUTLS_E_TPM_UNINITIALIZED" -> `Tpm_uninitialized
  | "GNUTLS_E_NO_CERTIFICATE_STATUS" -> `No_certificate_status
  | "GNUTLS_E_OCSP_RESPONSE_ERROR" -> `Ocsp_response_error
  | "GNUTLS_E_RANDOM_DEVICE_ERROR" -> `Random_device_error
  | "GNUTLS_E_AUTH_ERROR" -> `Auth_error
  | "GNUTLS_E_NO_APPLICATION_PROTOCOL" -> `No_application_protocol
  | "GNUTLS_E_SOCKETS_INIT_ERROR" -> `Sockets_init_error
  | "GNUTLS_E_KEY_IMPORT_FAILED" -> `Key_import_failed
  | "GNUTLS_E_SELF_TEST_ERROR" -> `Self_test_error
  | "GNUTLS_E_NO_SELF_TEST" -> `No_self_test
  | "GNUTLS_E_LIB_IN_ERROR_STATE" -> `Lib_in_error_state
  | "GNUTLS_E_PK_GENERATION_ERROR" -> `Pk_generation_error
  | "GNUTLS_E_IDNA_ERROR" -> `Idna_error
  | "GNUTLS_E_NEED_FALLBACK" -> `Need_fallback
  | "GNUTLS_E_UNIMPLEMENTED_FEATURE" -> `Unimplemented_feature
  | any -> failwith("b_error_of_name: unknown error code" ^ any)

(* This file is included into nettls_gnutls_bindings.ml *)

exception Null_pointer
exception Error of error_code
exception Short_memory_buffer of int

type memory = 
    (char,Bigarray.int8_unsigned_elt,Bigarray.c_layout) Bigarray.Array1.t

type gnutls_credentials =
    [ `Certificate of gnutls_certificate_credentials_t
    | `Srp_client of gnutls_srp_client_credentials_t
    | `Srp_server of gnutls_srp_server_credentials_t
    | `Psk_client of gnutls_psk_client_credentials_t
    | `Psk_server of gnutls_psk_server_credentials_t
    | `Anon_client of gnutls_anon_client_credentials_t
    | `Anon_server of gnutls_anon_server_credentials_t
    ]

external gnutls_credentials_set : gnutls_session_t -> gnutls_credentials -> unit
  = "net_gnutls_credentials_set" "net_gnutls_credentials_set"

type 'a unix_code =
  | ESUCCESS of 'a
  | EINTR
  | EAGAIN
  | EMSGSIZE
  | EPERM

external net_b_set_pull_callback : 
  gnutls_session_t -> (memory -> int unix_code) -> unit
  = "net_b_set_pull_callback" "net_b_set_pull_callback"

external net_b_set_push_callback : 
  gnutls_session_t -> (memory -> int -> int unix_code) -> unit
  = "net_b_set_push_callback" "net_b_set_push_callback"

external net_b_set_pull_timeout_callback : 
  gnutls_session_t -> (int -> bool unix_code) -> unit
  = "net_b_set_pull_timeout_callback" "net_b_set_pull_timeout_callback"

external net_b_set_verify_callback : 
  gnutls_session_t -> (unit -> bool) -> unit
  = "net_b_set_verify_callback" "net_b_set_verify_callback"


let protect f arg =
  try
    ESUCCESS(f arg)
  with
    | Unix.Unix_error(Unix.EINTR, _, _) ->
         EINTR
    | Unix.Unix_error((Unix.EAGAIN | Unix.EWOULDBLOCK), _, _) ->
         EAGAIN
    | Unix.Unix_error(Unix.EMSGSIZE, _, _) ->
         EMSGSIZE
    | Unix.Unix_error(e, _, _) ->
         EPERM
    | e ->
         Netlog.logf `Crit "Exception in Nettls_gnutls_bindings: %s"
                     (Netexn.to_string e);
         EPERM


let b_set_pull_callback s f =
  net_b_set_pull_callback s (protect f)


let b_set_push_callback s f =
  net_b_set_push_callback s (fun buf size -> protect (f buf) size)


let b_set_pull_timeout_callback s f =
  net_b_set_pull_timeout_callback s (protect f)


let b_set_verify_callback s f =
  net_b_set_verify_callback s (fun () -> f s)


external b_set_db_callbacks :
  gnutls_session_t ->
  (string -> string -> unit) ->
  (string -> unit) ->
  (string -> string) ->
  unit
  = "net_b_set_db_callbacks" "net_b_set_db_callbacks"


let set_fd s fd =
  let recv mem =
    Netsys_mem.mem_recv fd mem 0 (Bigarray.Array1.dim mem) [] in
  let send mem size =
    Netsys_mem.mem_send fd mem 0 size [] in
  let timeout ms =
    Netsys_posix.poll_single fd true false false (0.001 *. float ms) in
  b_set_pull_callback s recv;
  b_set_push_callback s send;
  b_set_pull_timeout_callback s timeout;
  ()

let string_of_verification_status_flag =
  function
    | `Invalid -> "INVALID"
    | `Revoked -> "REVOKED"
    | `Signer_not_found -> "SIGNER_NOT_FOUND"
    | `Signer_not_ca -> "SIGNER_NOT_CA"
    | `Insecure_algorithm -> "INSECURE_ALGORITHM"
    | `Not_activated -> "NOT_ACTIVATED"
    | `Expired -> "EXPIRED"
    | `Signature_failure -> "SIGNATURE_FAILURE"
    | `Revocation_data_superseded -> "REVOCATION_DATA_SUPERSEDED"
    | `Unexpected_owner -> "UNEXPECTED_OWNER"
    | `Revocation_data_issued_in_future -> "REVOCATION_DATA_ISSUED_IN_FUTURE"
    | `Signer_constraints_failure -> "SIGNER_CONSTRAINTS_FAILURE"
    | `Mismatch -> "MISMATCH"
    | `Purpose_mismatch -> "PURPOSE_MISMATCH"

external gnutls_x509_crt_list_import : string -> gnutls_x509_crt_fmt_t ->
                                  gnutls_certificate_import_flags ->
                                  gnutls_x509_crt_t array
  = "net_gnutls_x509_crt_list_import" "net_gnutls_x509_crt_list_import"

external gnutls_x509_crl_list_import : string -> gnutls_x509_crt_fmt_t ->
                                  gnutls_certificate_import_flags ->
                                  gnutls_x509_crl_t array
  = "net_gnutls_x509_crl_list_import" "net_gnutls_x509_crl_list_import"


let () =
  Callback.register_exception
    "Nettls_gnutls_bindings.Null_pointer"
    Null_pointer;
  Callback.register_exception
    "Nettls_gnutls_bindings.Error"
    (Error `Success);
  Callback.register_exception
    "Nettls_gnutls_bindings.Short_memory_buffer"
    (Short_memory_buffer 0)


let () =
  Netexn.register_printer
    (Error `Success)
    (function
      | Error code ->
           Printf.sprintf
             "Nettls_gnutls_bindings.Error(%s)" (gnutls_strerror_name code)
      | _ ->
           assert false
    )
