#define H_Certificate -9084681
#define H_Srp_client -964794055
#define H_Srp_server 711322033
#define H_Anon_client -610719234
#define H_Anon_server 1065396854
#define H_Psk_client 982790146
#define H_Psk_server 511422586
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "caml/mlvalues.h"
#include "caml/alloc.h"
#include "caml/memory.h"
#include "caml/misc.h"
#include "caml/custom.h"
#include "caml/fail.h"
#include "caml/unixsupport.h"
#include "caml/callback.h"
#include "caml/bigarray.h"
#include "caml/threads.h"

static unsigned int uint_val(value v);
static value protected_copy_string(const char *s);

/* This file is included into nettls_gnutls_bindings_stubs.c */

#include "./config.h"

#include <gnutls/gnutls.h>
#include <gnutls/openpgp.h>
#include <gnutls/x509.h>

#ifdef HAVE_GNUTLS_CRYPTO_H
#include <gnutls/crypto.h>
#endif

#ifdef HAVE_GNUTLS_ABSTRACT_H
#include <gnutls/abstract.h>
#endif

#include <errno.h>

#pragma GCC diagnostic ignored "-Wunused-function"

static int initialized = 0;

void nettls_init(void) {
    int code;
    if (!initialized) {
        code = gnutls_global_init();
        if (code != GNUTLS_E_SUCCESS) {
            fprintf(stderr, "Nettls_gnutls: cannot initialize: %s\n",
                    gnutls_strerror(code));
        }
        else
            initialized = 1;
    }
}


void nettls_deinit(void) {
    if (initialized) {
        gnutls_global_deinit();
        initialized = 0;
    }    
}


typedef int error_code;
typedef unsigned int gnutls_init_flags;
typedef unsigned int key_usage;
typedef const char * const_charp;
typedef gnutls_datum_t str_datum;
typedef gnutls_datum_t * str_datum_p;
typedef const gnutls_datum_t * const_str_datum_p;

#define DUMMY 0

typedef unsigned int empty_flags;
   
#define wrap_const_str_datum_p wrap_str_datum_p
#define unwrap_const_str_datum_p unwrap_str_datum_p

static value wrap_error_code(error_code x);
static value wrap_gnutls_x509_crt_t(gnutls_x509_crt_t x);
static value wrap_gnutls_x509_crl_t(gnutls_x509_crl_t x);

static gnutls_session_t 
           unwrap_gnutls_session_t(value v);
static void
           attach_gnutls_session_t(value v, value aux);
static gnutls_certificate_credentials_t 
           unwrap_gnutls_certificate_credentials_t(value v);
static void
           attach_gnutls_certificate_credentials_t(value v, value aux);
static gnutls_srp_client_credentials_t 
           unwrap_gnutls_srp_client_credentials_t(value v);
static gnutls_srp_server_credentials_t 
           unwrap_gnutls_srp_server_credentials_t(value v);
static gnutls_psk_client_credentials_t 
           unwrap_gnutls_psk_client_credentials_t(value v);
static gnutls_psk_server_credentials_t 
           unwrap_gnutls_psk_server_credentials_t(value v);
static gnutls_anon_client_credentials_t 
           unwrap_gnutls_anon_client_credentials_t(value v);
static gnutls_anon_server_credentials_t 
           unwrap_gnutls_anon_server_credentials_t(value v);
static gnutls_x509_crt_fmt_t 
           unwrap_gnutls_x509_crt_fmt_t(value v);
static gnutls_certificate_import_flags 
           unwrap_gnutls_certificate_import_flags(value v);

#define raise_null_pointer net_gnutls_null_pointer

static void net_gnutls_null_pointer(void) {
    caml_raise_constant(*caml_named_value
                          ("Nettls_gnutls_bindings.Null_pointer"));
}


static const char * unwrap_charp(value dummy) { return NULL; }
   
static value wrap_str_datum(const gnutls_datum_t d) {
    value s;
    s = caml_alloc_string(d.size);
    memcpy(String_val(s), d.data, d.size);
    return s;
}

static value wrap_str_datum_p(const gnutls_datum_t *d) {
    if (d==NULL) net_gnutls_null_pointer();
    return wrap_str_datum(*d);
}
   
static gnutls_datum_t unwrap_str_datum(value v) {
    gnutls_datum_t d;
    d.size = caml_string_length(v);
    d.data = stat_alloc(d.size);
    memcpy(d.data, String_val(v), d.size);
    return d;
}
   
static gnutls_datum_t * unwrap_str_datum_p(value v) {
    gnutls_datum_t *d;
    d = stat_alloc(sizeof(gnutls_datum_t));
    *d = unwrap_str_datum(v);
    return d;
}
   
static void free_str_datum(gnutls_datum_t d) {
    if (d.data != NULL) { stat_free(d.data); d.data = NULL; };
}

static void free_str_datum_p(gnutls_datum_t *d) {
    if (d != NULL) {
        if (d->data != NULL) stat_free(d->data);
        stat_free(d);
    }
}
   
static void net_gnutls_error_check(int error_code) {
    if (error_code < 0) {
        caml_raise_with_arg(*caml_named_value("Nettls_gnutls_bindings.Error"),
                            wrap_error_code(error_code));
    }
}


static void net_gnutls_size_check(int error_code, size_t s) {
    if (error_code == GNUTLS_E_SHORT_MEMORY_BUFFER) {
        caml_raise_with_arg(*caml_named_value
                              ("Nettls_gnutls_bindings.Short_memory_buffer"),
                            Val_long(s));
    }
}


/*
static void net_gnutls_transport_set_int(gnutls_session_t s,long fd) {
    gnutls_transport_set_ptr(s, (gnutls_transport_ptr_t) fd);
}
*/


struct b_session_callbacks_st {
    gnutls_session_t session;
    value pull_fun;            /* memory -> int unix_code */
    value pull_timeout_fun;    /* int -> int unix_code */
    value push_fun;            /* memory -> int -> int unix_code */
    value verify_fun;          /* unit -> bool */
    /* value params_fun; */
    value db_retrieve_fun;     /* string -> string */
    value db_store_fun;        /* string -> string -> unit */
    value db_remove_fun;       /* string -> unit */
};

typedef struct b_session_callbacks_st *b_session_callbacks_t;


static int get_transport_errno(value r) {
    switch (Int_val(r)) {
    case 0:
        return EINTR;
    case 1:
        return EAGAIN;
    case 2:
        return EMSGSIZE;
    case 3:
        return EPERM;
    default:
        return EPERM;
    }    
}


static ssize_t push_callback(gnutls_transport_ptr_t cb_ptr, const void *data,
                     size_t size) {
    b_session_callbacks_t cb;
    int flags;
    ssize_t n;
    CAMLparam0();
    CAMLlocal2(ba,r);

    cb = (b_session_callbacks_t) cb_ptr;
    if (Is_block(cb->push_fun)) {
        flags = CAML_BA_UINT8 | CAML_BA_C_LAYOUT | CAML_BA_EXTERNAL;
        ba = caml_ba_alloc_dims(flags, 1, (void *) data, (intnat) size);
        r = caml_callback2_exn(cb->push_fun, ba, Val_long(size));
        if (Is_exception_result(r)) {
            r = Extract_exception(r);
            gnutls_transport_set_errno(cb->session, EPERM);
            n = -1;
        }
        else {
            /* r is an [int unix_code] */
            if (Is_block(r)) {
                n = Long_val(Field(r, 0));
                if (n<0) {
                    gnutls_transport_set_errno(cb->session, EPERM);
                    n = -1;
                }
            }
            else {
                gnutls_transport_set_errno(cb->session, get_transport_errno(r));
                n = -1;
            }
        }
    } else {
        gnutls_transport_set_errno(cb->session, EPERM);
        n = -1;
    };
    CAMLreturn(n);
}


static ssize_t pull_callback(gnutls_transport_ptr_t cb_ptr, void *data,
                     size_t size) {
    b_session_callbacks_t cb;
    int flags;
    ssize_t n;
    CAMLparam0();
    CAMLlocal2(ba,r);

    cb = (b_session_callbacks_t) cb_ptr;
    if (Is_block(cb->pull_fun)) {
        flags = CAML_BA_UINT8 | CAML_BA_C_LAYOUT | CAML_BA_EXTERNAL;
        ba = caml_ba_alloc_dims(flags, 1, data, (intnat) size);
        r = caml_callback_exn(cb->pull_fun, ba);
        if (Is_exception_result(r)) {
            r = Extract_exception(r);
            gnutls_transport_set_errno(cb->session, EPERM);
            n = -1;
        }
        else {
            /* r is an [int unix_code] */
            if (Is_block(r)) {
                n = Long_val(Field(r, 0));
                if (n<0) {
                    gnutls_transport_set_errno(cb->session, EPERM);
                    n = -1;
                }
            }
            else {
                gnutls_transport_set_errno(cb->session, get_transport_errno(r));
                n = -1;
            }
        }
    } else {
        gnutls_transport_set_errno(cb->session, EPERM);
        n = -1;
    };
    CAMLreturn(n);
}


static int pull_timeout_callback(gnutls_transport_ptr_t cb_ptr,
                                     unsigned int ms) {
    b_session_callbacks_t cb;
    int n;
    CAMLparam0();
    CAMLlocal1(r);

    cb = (b_session_callbacks_t) cb_ptr;
    if (Is_block(cb->pull_timeout_fun)) {
        r = caml_callback_exn(cb->pull_timeout_fun, Val_int(ms));
        if (Is_exception_result(r)) {
            r = Extract_exception(r);
            gnutls_transport_set_errno(cb->session, EPERM);
            n = -1;
        }
        else {
            /* r is an [int unix_code] */
            if (Is_block(r)) {
                n = Long_val(Field(r, 0));
                if (n<0) {
                    gnutls_transport_set_errno(cb->session, EPERM);
                    n = -1;
                }
            }
            else {
                gnutls_transport_set_errno(cb->session, get_transport_errno(r));
                n = -1;
            }
        }
    } else {
        gnutls_transport_set_errno(cb->session, EPERM);
        n = -1;
    };
    CAMLreturn(n);
}


static int verify_callback(gnutls_session_t s) {
    b_session_callbacks_t cb;
    int n;
    CAMLparam0();
    CAMLlocal1(r);

    cb = (b_session_callbacks_t) gnutls_session_get_ptr(s);
    if (Is_block(cb->verify_fun)) {
        r = caml_callback_exn(cb->verify_fun, Val_unit);
        if (Is_exception_result(r)) {
            r = Extract_exception(r);
            n = 1;
        }
        else
            n = Bool_val(r) ? 0 : 1;
    }
    else 
        n=0;
    CAMLreturn(n);
}


static int db_store_callback(void *ptr, gnutls_datum_t key, 
                             gnutls_datum_t data) {
    b_session_callbacks_t cb;
    int n;
    CAMLparam0();
    CAMLlocal3(r, keyv, datav);

    cb = (b_session_callbacks_t) ptr;
    if (Is_block(cb->db_store_fun)) {
        keyv = wrap_str_datum(key);
        datav = wrap_str_datum(data);
        r = caml_callback2_exn(cb->db_store_fun, keyv, datav);
        if (Is_exception_result(r)) {
            r = Extract_exception(r);
            n = 1;
        }
        else
            n = 0;
    }
    else
        n = 1;
    CAMLreturn(n);
}


static int db_remove_callback(void *ptr, gnutls_datum_t key) {
    b_session_callbacks_t cb;
    int n;
    CAMLparam0();
    CAMLlocal2(r, keyv);

    cb = (b_session_callbacks_t) ptr;
    if (Is_block(cb->db_remove_fun)) {
        keyv = wrap_str_datum(key);
        r = caml_callback_exn(cb->db_remove_fun, keyv);
        if (Is_exception_result(r)) {
            r = Extract_exception(r);
            n = 1;
        }
        else
            n = 0;
    }
    else
        n = 1;
    CAMLreturn(n);
}


static gnutls_datum_t db_retrieve_callback(void *ptr, gnutls_datum_t key) {
    b_session_callbacks_t cb;
    gnutls_datum_t r;
    CAMLparam0();
    CAMLlocal2(keyv, datav);

    r.data = NULL;
    r.size = 0;
    cb = (b_session_callbacks_t) ptr;
    if (Is_block(cb->db_retrieve_fun)) {
        keyv = wrap_str_datum(key);
        datav = caml_callback_exn(cb->db_retrieve_fun, keyv);
        if (Is_exception_result(datav)) {
            datav = Extract_exception(datav);
        }
        else {
            r.size = caml_string_length(datav);
            r.data = gnutls_malloc(r.size);
            memcpy(r.data, String_val(datav), r.size);
        }
    };
    CAMLreturnT(gnutls_datum_t, r);
}


static void attach_session_callbacks (gnutls_session_t s) {
    b_session_callbacks_t cb;

    cb = (b_session_callbacks_t) 
            stat_alloc(sizeof(struct b_session_callbacks_st));
    cb->session = s;
    cb->pull_fun = Val_int(0);
    cb->pull_timeout_fun = Val_int(0);
    cb->push_fun = Val_int(0);
    cb->verify_fun = Val_int(0);
    /* cb->params_fun = Val_int(0); */
    cb->db_retrieve_fun = Val_int(0);
    cb->db_store_fun = Val_int(0);
    cb->db_remove_fun = Val_int(0);

    caml_register_generational_global_root(&(cb->pull_fun));
    caml_register_generational_global_root(&(cb->pull_timeout_fun));
    caml_register_generational_global_root(&(cb->push_fun));
    caml_register_generational_global_root(&(cb->verify_fun));
    /* caml_register_generational_global_root(&(cb->params_fun));*/
    caml_register_generational_global_root(&(cb->db_retrieve_fun));
    caml_register_generational_global_root(&(cb->db_store_fun));
    caml_register_generational_global_root(&(cb->db_remove_fun));

    gnutls_session_set_ptr(s, cb);
    gnutls_transport_set_ptr(s, cb);
    gnutls_db_set_ptr(s, cb);
    gnutls_transport_set_push_function(s, &push_callback);
    gnutls_transport_set_pull_function(s, &pull_callback);
#ifdef HAVE_FUN_gnutls_transport_set_pull_timeout_function
    gnutls_transport_set_pull_timeout_function(s, &pull_timeout_callback);
#endif
    /*
    gnutls_db_set_retrieve_function: see net_b_set_db_callbacks
    gnutls_db_set_remove_function: see net_b_set_db_callbacks
    gnutls_db_set_store_function: see net_b_set_db_callbacks
    */
    /* verify_callback: this is set in net_gnutls_credentials_set for
       the certificate once it is connected with the session. (The
       callback is the same for all sessions, so this is no problem.)
    */
}


static void b_free_session(gnutls_session_t s) {
    b_session_callbacks_t cb;

    cb = gnutls_session_get_ptr(s);
    caml_remove_generational_global_root(&(cb->pull_fun));
    caml_remove_generational_global_root(&(cb->pull_timeout_fun));
    caml_remove_generational_global_root(&(cb->push_fun));
    caml_remove_generational_global_root(&(cb->verify_fun));
    /* caml_remove_generational_global_root(&(cb->params_fun)); */
    caml_remove_generational_global_root(&(cb->db_retrieve_fun));
    caml_remove_generational_global_root(&(cb->db_store_fun));
    caml_remove_generational_global_root(&(cb->db_remove_fun));

    stat_free(cb);
    gnutls_deinit(s);
}


CAMLprim value net_b_set_pull_callback(value sv, value fun) {
    gnutls_session_t s;
    b_session_callbacks_t cb;
    nettls_init();
    s = unwrap_gnutls_session_t(sv);
    cb = gnutls_session_get_ptr(s);
    caml_modify_generational_global_root(&(cb->pull_fun), fun);
    return Val_unit;
}


CAMLprim value net_b_set_pull_timeout_callback(value sv, value fun) {
#ifdef HAVE_FUN_gnutls_transport_set_pull_timeout_function
    gnutls_session_t s;
    b_session_callbacks_t cb;
    nettls_init();
    s = unwrap_gnutls_session_t(sv);
    cb = gnutls_session_get_ptr(s);
    caml_modify_generational_global_root(&(cb->pull_timeout_fun), fun);
    return Val_unit;
#else
    invalid_argument("b_set_pull_timeout_callback");
#endif
}


CAMLprim value net_b_set_push_callback(value sv, value fun) {
    gnutls_session_t s;
    b_session_callbacks_t cb;
    nettls_init();
    s = unwrap_gnutls_session_t(sv);
    cb = gnutls_session_get_ptr(s);
    caml_modify_generational_global_root(&(cb->push_fun), fun);
    return Val_unit;
}


CAMLprim value net_b_set_verify_callback(value sv, value fun) {
#ifdef HAVE_FUN_gnutls_certificate_set_verify_function
    gnutls_session_t s;
    b_session_callbacks_t cb;
    nettls_init();
    s = unwrap_gnutls_session_t(sv);
    cb = gnutls_session_get_ptr(s);
    caml_modify_generational_global_root(&(cb->verify_fun), fun);
    return Val_unit;
#else
    invalid_argument("b_set_verify_callback");
#endif
}


CAMLprim value net_b_set_db_callbacks(value sv,
                                      value store_fun,
                                      value remove_fun,
                                      value retrieve_fun) {
    gnutls_session_t s;
    b_session_callbacks_t cb;
    nettls_init();
    s = unwrap_gnutls_session_t(sv);
    cb = gnutls_session_get_ptr(s);
    gnutls_db_set_retrieve_function(s, &db_retrieve_callback);
    gnutls_db_set_remove_function(s, &db_remove_callback);
    gnutls_db_set_store_function(s, &db_store_callback);
    caml_modify_generational_global_root(&(cb->db_store_fun), store_fun);
    caml_modify_generational_global_root(&(cb->db_remove_fun), remove_fun);
    caml_modify_generational_global_root(&(cb->db_retrieve_fun), retrieve_fun);
    return Val_unit;
}



CAMLprim value net_gnutls_credentials_set(value sess, value creds) {
    gnutls_session_t s;
    int error_code;
    CAMLparam2(sess,creds);
    nettls_init();
    s = unwrap_gnutls_session_t(sess);
    switch (Long_val(Field(creds,0))) {
    case H_Certificate: {
        gnutls_certificate_credentials_t cert;
        cert = unwrap_gnutls_certificate_credentials_t(Field(creds,1));
        error_code = 
            gnutls_credentials_set(s, GNUTLS_CRD_CERTIFICATE, cert);
#ifdef HAVE_FUN_gnutls_certificate_set_verify_function
        if (error_code == 0) 
            gnutls_certificate_set_verify_function(cert, &verify_callback);
#endif
        break;
        }
    case H_Srp_client:
        error_code = 
            gnutls_credentials_set(s,
                                   GNUTLS_CRD_SRP,
                                   unwrap_gnutls_srp_client_credentials_t 
                                     (Field(creds,1))
                                   );
        break;
    case H_Srp_server:
        error_code = 
            gnutls_credentials_set(s,
                                   GNUTLS_CRD_SRP,
                                   unwrap_gnutls_srp_server_credentials_t 
                                     (Field(creds,1))
                                   );
        break;
    case H_Anon_client:
        error_code = 
            gnutls_credentials_set(s,
                                   GNUTLS_CRD_ANON,
                                   unwrap_gnutls_anon_client_credentials_t 
                                     (Field(creds,1))
                                   );
        break;
    case H_Anon_server:
        error_code = 
            gnutls_credentials_set(s,
                                   GNUTLS_CRD_ANON,
                                   unwrap_gnutls_anon_server_credentials_t 
                                     (Field(creds,1))
                                   );
        break;
    case H_Psk_client:
        error_code = 
            gnutls_credentials_set(s,
                                   GNUTLS_CRD_PSK,
                                   unwrap_gnutls_psk_client_credentials_t 
                                     (Field(creds,1))
                                   );
        break;
    case H_Psk_server:
        error_code = 
            gnutls_credentials_set(s,
                                   GNUTLS_CRD_CERTIFICATE,
                                   unwrap_gnutls_psk_server_credentials_t 
                                     (Field(creds,1))
                                   );
        break;
    default:
        failwith("net_gnutls_credentials_set");
    };
    net_gnutls_error_check(error_code);
    attach_gnutls_session_t(sess, creds);
    CAMLreturn(Val_unit);
}


CAMLprim value net_gnutls_x509_crt_list_import(value datav, value formatv, 
                                               value flagsv) {
    gnutls_datum_t data;
    gnutls_x509_crt_fmt_t format;
    unsigned int flags;
    gnutls_x509_crt_t cert1;
    gnutls_x509_crt_t *certs;
    unsigned int n;
    int code, k, alloc_certs;
    CAMLparam3(datav, formatv, flagsv);
    CAMLlocal2(array, crt);

    nettls_init();
    data = unwrap_str_datum(datav);
    format = unwrap_gnutls_x509_crt_fmt_t(formatv);
    flags = unwrap_gnutls_certificate_import_flags(flagsv);

    certs = &cert1;
    n = 1;
    alloc_certs = 0;
    code = gnutls_x509_crt_list_import(certs, &n, &data, format, 
                    flags | GNUTLS_X509_CRT_LIST_IMPORT_FAIL_IF_EXCEED);
    if (code == GNUTLS_E_SHORT_MEMORY_BUFFER) {
        certs = (gnutls_x509_crt_t *) stat_alloc(n * sizeof(void *));
        alloc_certs = 1;
        code = gnutls_x509_crt_list_import(certs, &n, &data, format, 
                                           flags);
    };
    if (code >= 0) {
        array = caml_alloc(code, 0);
        for (k = 0; k < code; k++) {
            crt = wrap_gnutls_x509_crt_t(certs[k]);
            Store_field(array, k, crt);
        };
    };
    if (alloc_certs)
        stat_free(certs);
    net_gnutls_error_check(code);
    CAMLreturn(array);
}


CAMLprim value net_gnutls_x509_crl_list_import(value datav, value formatv, 
                                               value flagsv) {
#ifdef HAVE_FUN_net_gnutls_x509_crl_list_import
    gnutls_datum_t data;
    gnutls_x509_crt_fmt_t format;
    unsigned int flags;
    gnutls_x509_crl_t cert1;
    gnutls_x509_crl_t *certs;
    unsigned int n;
    int code, k, alloc_certs;
    CAMLparam3(datav, formatv, flagsv);
    CAMLlocal2(array, crt);

    nettls_init();
    data = unwrap_str_datum(datav);
    format = unwrap_gnutls_x509_crt_fmt_t(formatv);
    flags = unwrap_gnutls_certificate_import_flags(flagsv);

    certs = &cert1;
    n = 1;
    alloc_certs = 0;
    code = gnutls_x509_crl_list_import(certs, &n, &data, format, 
                    flags | GNUTLS_X509_CRT_LIST_IMPORT_FAIL_IF_EXCEED);
    if (code == GNUTLS_E_SHORT_MEMORY_BUFFER) {
        certs = (gnutls_x509_crl_t *) stat_alloc(n * sizeof(void *));
        alloc_certs = 1;
        code = gnutls_x509_crl_list_import(certs, &n, &data, format, 
                                           flags);
    };
    if (code >= 0) {
        array = caml_alloc(code, 0);
        for (k = 0; k < code; k++) {
            crt = wrap_gnutls_x509_crl_t(certs[k]);
            Store_field(array, k, crt);
        };
    };
    if (alloc_certs)
        stat_free(certs);
    net_gnutls_error_check(code);
    CAMLreturn(array);
#else
    invalid_argument("gnutls_x509_crl_list_import");
#endif
}

static int net_have_crypto(void) {
#ifdef HAVE_FUN_gnutls_cipher_encrypt2
    return 1;
#else
    return 0;
#endif
}


#ifndef HAVE_GNUTLS_ABSTRACT_H
typedef void *gnutls_pubkey_t;
typedef void *gnutls_privkey_t;
static void gnutls_pubkey_deinit(gnutls_pubkey_t key) {}
static void gnutls_privkey_deinit(gnutls_privkey_t key) {}
#endif


static int net_have_pubkey(void) {
#ifdef HAVE_FUN_gnutls_pubkey_encrypt_data
    return 1;
#else
    return 0;
#endif
}
static unsigned int uint_val(value v) {
    if (Int_val(v) < 0) invalid_argument("negative integer");
    return (unsigned int) Int_val(v);
}

static value protected_copy_string(const char *s) {
    if (s==NULL) raise_null_pointer();
    return caml_copy_string(s);
}

/************** gnutls_cipher_algorithm_t *************/

struct enumstruct_gnutls_cipher_algorithm_t { gnutls_cipher_algorithm_t value; long oid; };

#define enumstructptr_gnutls_cipher_algorithm_t_val(v) ((struct enumstruct_gnutls_cipher_algorithm_t *) (Data_custom_val(v)))
#define enum_gnutls_cipher_algorithm_t_unwrap(v) (enumstructptr_gnutls_cipher_algorithm_t_val(v)->value)
long enum_gnutls_cipher_algorithm_t_oid = 0;

static int enum_gnutls_cipher_algorithm_t_compare(value v1, value v2) {
  struct enumstruct_gnutls_cipher_algorithm_t *p1;
  struct enumstruct_gnutls_cipher_algorithm_t *p2;
  p1 = enumstructptr_gnutls_cipher_algorithm_t_val(v1);
  p2 = enumstructptr_gnutls_cipher_algorithm_t_val(v2);
  return p1->oid - p2->oid;
}

static struct custom_operations enum_gnutls_cipher_algorithm_t_ops = {
  "",
  custom_finalize_default,
  enum_gnutls_cipher_algorithm_t_compare,
  custom_hash_default,
  custom_serialize_default,
  custom_deserialize_default
};

static gnutls_cipher_algorithm_t unwrap_gnutls_cipher_algorithm_t(value v) {
  return enum_gnutls_cipher_algorithm_t_unwrap(v);
}

static value wrap_gnutls_cipher_algorithm_t(gnutls_cipher_algorithm_t x) {
  value v;
  v = caml_alloc_custom(&enum_gnutls_cipher_algorithm_t_ops, sizeof(struct enumstruct_gnutls_cipher_algorithm_t), 0, 1);
  enumstructptr_gnutls_cipher_algorithm_t_val(v)->value = x;
  enumstructptr_gnutls_cipher_algorithm_t_val(v)->oid = enum_gnutls_cipher_algorithm_t_oid++;
  return v;
}

/************** gnutls_kx_algorithm_t *************/

struct enumstruct_gnutls_kx_algorithm_t { gnutls_kx_algorithm_t value; long oid; };

#define enumstructptr_gnutls_kx_algorithm_t_val(v) ((struct enumstruct_gnutls_kx_algorithm_t *) (Data_custom_val(v)))
#define enum_gnutls_kx_algorithm_t_unwrap(v) (enumstructptr_gnutls_kx_algorithm_t_val(v)->value)
long enum_gnutls_kx_algorithm_t_oid = 0;

static int enum_gnutls_kx_algorithm_t_compare(value v1, value v2) {
  struct enumstruct_gnutls_kx_algorithm_t *p1;
  struct enumstruct_gnutls_kx_algorithm_t *p2;
  p1 = enumstructptr_gnutls_kx_algorithm_t_val(v1);
  p2 = enumstructptr_gnutls_kx_algorithm_t_val(v2);
  return p1->oid - p2->oid;
}

static struct custom_operations enum_gnutls_kx_algorithm_t_ops = {
  "",
  custom_finalize_default,
  enum_gnutls_kx_algorithm_t_compare,
  custom_hash_default,
  custom_serialize_default,
  custom_deserialize_default
};

static gnutls_kx_algorithm_t unwrap_gnutls_kx_algorithm_t(value v) {
  return enum_gnutls_kx_algorithm_t_unwrap(v);
}

static value wrap_gnutls_kx_algorithm_t(gnutls_kx_algorithm_t x) {
  value v;
  v = caml_alloc_custom(&enum_gnutls_kx_algorithm_t_ops, sizeof(struct enumstruct_gnutls_kx_algorithm_t), 0, 1);
  enumstructptr_gnutls_kx_algorithm_t_val(v)->value = x;
  enumstructptr_gnutls_kx_algorithm_t_val(v)->oid = enum_gnutls_kx_algorithm_t_oid++;
  return v;
}

/************** gnutls_mac_algorithm_t *************/

struct enumstruct_gnutls_mac_algorithm_t { gnutls_mac_algorithm_t value; long oid; };

#define enumstructptr_gnutls_mac_algorithm_t_val(v) ((struct enumstruct_gnutls_mac_algorithm_t *) (Data_custom_val(v)))
#define enum_gnutls_mac_algorithm_t_unwrap(v) (enumstructptr_gnutls_mac_algorithm_t_val(v)->value)
long enum_gnutls_mac_algorithm_t_oid = 0;

static int enum_gnutls_mac_algorithm_t_compare(value v1, value v2) {
  struct enumstruct_gnutls_mac_algorithm_t *p1;
  struct enumstruct_gnutls_mac_algorithm_t *p2;
  p1 = enumstructptr_gnutls_mac_algorithm_t_val(v1);
  p2 = enumstructptr_gnutls_mac_algorithm_t_val(v2);
  return p1->oid - p2->oid;
}

static struct custom_operations enum_gnutls_mac_algorithm_t_ops = {
  "",
  custom_finalize_default,
  enum_gnutls_mac_algorithm_t_compare,
  custom_hash_default,
  custom_serialize_default,
  custom_deserialize_default
};

static gnutls_mac_algorithm_t unwrap_gnutls_mac_algorithm_t(value v) {
  return enum_gnutls_mac_algorithm_t_unwrap(v);
}

static value wrap_gnutls_mac_algorithm_t(gnutls_mac_algorithm_t x) {
  value v;
  v = caml_alloc_custom(&enum_gnutls_mac_algorithm_t_ops, sizeof(struct enumstruct_gnutls_mac_algorithm_t), 0, 1);
  enumstructptr_gnutls_mac_algorithm_t_val(v)->value = x;
  enumstructptr_gnutls_mac_algorithm_t_val(v)->oid = enum_gnutls_mac_algorithm_t_oid++;
  return v;
}

/************** gnutls_compression_method_t *************/

struct enumstruct_gnutls_compression_method_t { gnutls_compression_method_t value; long oid; };

#define enumstructptr_gnutls_compression_method_t_val(v) ((struct enumstruct_gnutls_compression_method_t *) (Data_custom_val(v)))
#define enum_gnutls_compression_method_t_unwrap(v) (enumstructptr_gnutls_compression_method_t_val(v)->value)
long enum_gnutls_compression_method_t_oid = 0;

static int enum_gnutls_compression_method_t_compare(value v1, value v2) {
  struct enumstruct_gnutls_compression_method_t *p1;
  struct enumstruct_gnutls_compression_method_t *p2;
  p1 = enumstructptr_gnutls_compression_method_t_val(v1);
  p2 = enumstructptr_gnutls_compression_method_t_val(v2);
  return p1->oid - p2->oid;
}

static struct custom_operations enum_gnutls_compression_method_t_ops = {
  "",
  custom_finalize_default,
  enum_gnutls_compression_method_t_compare,
  custom_hash_default,
  custom_serialize_default,
  custom_deserialize_default
};

static gnutls_compression_method_t unwrap_gnutls_compression_method_t(value v) {
  return enum_gnutls_compression_method_t_unwrap(v);
}

static value wrap_gnutls_compression_method_t(gnutls_compression_method_t x) {
  value v;
  v = caml_alloc_custom(&enum_gnutls_compression_method_t_ops, sizeof(struct enumstruct_gnutls_compression_method_t), 0, 1);
  enumstructptr_gnutls_compression_method_t_val(v)->value = x;
  enumstructptr_gnutls_compression_method_t_val(v)->oid = enum_gnutls_compression_method_t_oid++;
  return v;
}

/************** gnutls_pk_algorithm_t *************/

struct enumstruct_gnutls_pk_algorithm_t { gnutls_pk_algorithm_t value; long oid; };

#define enumstructptr_gnutls_pk_algorithm_t_val(v) ((struct enumstruct_gnutls_pk_algorithm_t *) (Data_custom_val(v)))
#define enum_gnutls_pk_algorithm_t_unwrap(v) (enumstructptr_gnutls_pk_algorithm_t_val(v)->value)
long enum_gnutls_pk_algorithm_t_oid = 0;

static int enum_gnutls_pk_algorithm_t_compare(value v1, value v2) {
  struct enumstruct_gnutls_pk_algorithm_t *p1;
  struct enumstruct_gnutls_pk_algorithm_t *p2;
  p1 = enumstructptr_gnutls_pk_algorithm_t_val(v1);
  p2 = enumstructptr_gnutls_pk_algorithm_t_val(v2);
  return p1->oid - p2->oid;
}

static struct custom_operations enum_gnutls_pk_algorithm_t_ops = {
  "",
  custom_finalize_default,
  enum_gnutls_pk_algorithm_t_compare,
  custom_hash_default,
  custom_serialize_default,
  custom_deserialize_default
};

static gnutls_pk_algorithm_t unwrap_gnutls_pk_algorithm_t(value v) {
  return enum_gnutls_pk_algorithm_t_unwrap(v);
}

static value wrap_gnutls_pk_algorithm_t(gnutls_pk_algorithm_t x) {
  value v;
  v = caml_alloc_custom(&enum_gnutls_pk_algorithm_t_ops, sizeof(struct enumstruct_gnutls_pk_algorithm_t), 0, 1);
  enumstructptr_gnutls_pk_algorithm_t_val(v)->value = x;
  enumstructptr_gnutls_pk_algorithm_t_val(v)->oid = enum_gnutls_pk_algorithm_t_oid++;
  return v;
}

/************** gnutls_sign_algorithm_t *************/

struct enumstruct_gnutls_sign_algorithm_t { gnutls_sign_algorithm_t value; long oid; };

#define enumstructptr_gnutls_sign_algorithm_t_val(v) ((struct enumstruct_gnutls_sign_algorithm_t *) (Data_custom_val(v)))
#define enum_gnutls_sign_algorithm_t_unwrap(v) (enumstructptr_gnutls_sign_algorithm_t_val(v)->value)
long enum_gnutls_sign_algorithm_t_oid = 0;

static int enum_gnutls_sign_algorithm_t_compare(value v1, value v2) {
  struct enumstruct_gnutls_sign_algorithm_t *p1;
  struct enumstruct_gnutls_sign_algorithm_t *p2;
  p1 = enumstructptr_gnutls_sign_algorithm_t_val(v1);
  p2 = enumstructptr_gnutls_sign_algorithm_t_val(v2);
  return p1->oid - p2->oid;
}

static struct custom_operations enum_gnutls_sign_algorithm_t_ops = {
  "",
  custom_finalize_default,
  enum_gnutls_sign_algorithm_t_compare,
  custom_hash_default,
  custom_serialize_default,
  custom_deserialize_default
};

static gnutls_sign_algorithm_t unwrap_gnutls_sign_algorithm_t(value v) {
  return enum_gnutls_sign_algorithm_t_unwrap(v);
}

static value wrap_gnutls_sign_algorithm_t(gnutls_sign_algorithm_t x) {
  value v;
  v = caml_alloc_custom(&enum_gnutls_sign_algorithm_t_ops, sizeof(struct enumstruct_gnutls_sign_algorithm_t), 0, 1);
  enumstructptr_gnutls_sign_algorithm_t_val(v)->value = x;
  enumstructptr_gnutls_sign_algorithm_t_val(v)->oid = enum_gnutls_sign_algorithm_t_oid++;
  return v;
}

/************** gnutls_certificate_type_t *************/

struct enumstruct_gnutls_certificate_type_t { gnutls_certificate_type_t value; long oid; };

#define enumstructptr_gnutls_certificate_type_t_val(v) ((struct enumstruct_gnutls_certificate_type_t *) (Data_custom_val(v)))
#define enum_gnutls_certificate_type_t_unwrap(v) (enumstructptr_gnutls_certificate_type_t_val(v)->value)
long enum_gnutls_certificate_type_t_oid = 0;

static int enum_gnutls_certificate_type_t_compare(value v1, value v2) {
  struct enumstruct_gnutls_certificate_type_t *p1;
  struct enumstruct_gnutls_certificate_type_t *p2;
  p1 = enumstructptr_gnutls_certificate_type_t_val(v1);
  p2 = enumstructptr_gnutls_certificate_type_t_val(v2);
  return p1->oid - p2->oid;
}

static struct custom_operations enum_gnutls_certificate_type_t_ops = {
  "",
  custom_finalize_default,
  enum_gnutls_certificate_type_t_compare,
  custom_hash_default,
  custom_serialize_default,
  custom_deserialize_default
};

static gnutls_certificate_type_t unwrap_gnutls_certificate_type_t(value v) {
  return enum_gnutls_certificate_type_t_unwrap(v);
}

static value wrap_gnutls_certificate_type_t(gnutls_certificate_type_t x) {
  value v;
  v = caml_alloc_custom(&enum_gnutls_certificate_type_t_ops, sizeof(struct enumstruct_gnutls_certificate_type_t), 0, 1);
  enumstructptr_gnutls_certificate_type_t_val(v)->value = x;
  enumstructptr_gnutls_certificate_type_t_val(v)->oid = enum_gnutls_certificate_type_t_oid++;
  return v;
}

/************** gnutls_digest_algorithm_t *************/

static value wrap_gnutls_digest_algorithm_t(gnutls_digest_algorithm_t x) {
  gnutls_mac_algorithm_t y;
  y = (gnutls_mac_algorithm_t) x;
  return wrap_gnutls_mac_algorithm_t(y);
}

static gnutls_digest_algorithm_t unwrap_gnutls_digest_algorithm_t(value v) {
  gnutls_mac_algorithm_t y;
  y = unwrap_gnutls_mac_algorithm_t(v);
  return (gnutls_digest_algorithm_t) y;
}

/************** gnutls_session_t *************/

struct absstruct_gnutls_session_t { gnutls_session_t value; long tag; long oid; };

#define absstructptr_gnutls_session_t_val(v) ((struct absstruct_gnutls_session_t *) (Data_custom_val(v)))
#define abs_gnutls_session_t_unwrap(v) (absstructptr_gnutls_session_t_val(v)->value)
#define abs_gnutls_session_t_tag(v) (absstructptr_gnutls_session_t_val(v)->tag)
long abs_gnutls_session_t_oid = 0;

static int abs_gnutls_session_t_compare(value v1, value v2) {
  struct absstruct_gnutls_session_t *p1;
  struct absstruct_gnutls_session_t *p2;
  p1 = absstructptr_gnutls_session_t_val(v1);
  p2 = absstructptr_gnutls_session_t_val(v2);
  return p1->oid - p2->oid;
}

static void abs_gnutls_session_t_finalize(value v1) {
  struct absstruct_gnutls_session_t *p1;
  p1 = absstructptr_gnutls_session_t_val(v1);
  b_free_session(p1->value);
}

static struct custom_operations abs_gnutls_session_t_ops = {
  "",
  abs_gnutls_session_t_finalize,
  abs_gnutls_session_t_compare,
  custom_hash_default,
  custom_serialize_default,
  custom_deserialize_default
};

static gnutls_session_t unwrap_gnutls_session_t(value v) {
  gnutls_session_t r;
  r = abs_gnutls_session_t_unwrap(Field(v,0));
  if (r == NULL) raise_null_pointer();
  return r;
}

static long tag_gnutls_session_t(value v) {
  return abs_gnutls_session_t_tag(Field(v,0));
}

static value twrap_gnutls_session_t(long tag, gnutls_session_t x) {
  CAMLparam0();
  CAMLlocal2(v,r);
  if (x == NULL) failwith("wrap_gnutls_session_t: NULL pointer");
  v = caml_alloc_custom(&abs_gnutls_session_t_ops, sizeof(struct absstruct_gnutls_session_t), 0, 1);
  absstructptr_gnutls_session_t_val(v)->tag = tag;
  absstructptr_gnutls_session_t_val(v)->value = x;
  absstructptr_gnutls_session_t_val(v)->oid = abs_gnutls_session_t_oid++;
  r = caml_alloc(2,0);
  Field(r,0) = v;
  Field(r,1) = Val_int(0);
  CAMLreturn(r);
}

static value wrap_gnutls_session_t(gnutls_session_t x) {
  return twrap_gnutls_session_t(0, x);
}

static void attach_gnutls_session_t(value v, value aux) {
  CAMLparam2(v,aux);
  CAMLlocal1(h);
  h = caml_alloc(2,0);
  Field(h,0) = aux;
  Field(h,1) = Field(v,1);
  Store_field(v,1,h);
  CAMLreturn0;
}

/************** gnutls_dh_params_t *************/

struct absstruct_gnutls_dh_params_t { gnutls_dh_params_t value; long tag; long oid; };

#define absstructptr_gnutls_dh_params_t_val(v) ((struct absstruct_gnutls_dh_params_t *) (Data_custom_val(v)))
#define abs_gnutls_dh_params_t_unwrap(v) (absstructptr_gnutls_dh_params_t_val(v)->value)
#define abs_gnutls_dh_params_t_tag(v) (absstructptr_gnutls_dh_params_t_val(v)->tag)
long abs_gnutls_dh_params_t_oid = 0;

static int abs_gnutls_dh_params_t_compare(value v1, value v2) {
  struct absstruct_gnutls_dh_params_t *p1;
  struct absstruct_gnutls_dh_params_t *p2;
  p1 = absstructptr_gnutls_dh_params_t_val(v1);
  p2 = absstructptr_gnutls_dh_params_t_val(v2);
  return p1->oid - p2->oid;
}

static void abs_gnutls_dh_params_t_finalize(value v1) {
  struct absstruct_gnutls_dh_params_t *p1;
  p1 = absstructptr_gnutls_dh_params_t_val(v1);
  gnutls_dh_params_deinit(p1->value);
}

static struct custom_operations abs_gnutls_dh_params_t_ops = {
  "",
  abs_gnutls_dh_params_t_finalize,
  abs_gnutls_dh_params_t_compare,
  custom_hash_default,
  custom_serialize_default,
  custom_deserialize_default
};

static gnutls_dh_params_t unwrap_gnutls_dh_params_t(value v) {
  gnutls_dh_params_t r;
  r = abs_gnutls_dh_params_t_unwrap(Field(v,0));
  if (r == NULL) raise_null_pointer();
  return r;
}

static long tag_gnutls_dh_params_t(value v) {
  return abs_gnutls_dh_params_t_tag(Field(v,0));
}

static value twrap_gnutls_dh_params_t(long tag, gnutls_dh_params_t x) {
  CAMLparam0();
  CAMLlocal2(v,r);
  if (x == NULL) failwith("wrap_gnutls_dh_params_t: NULL pointer");
  v = caml_alloc_custom(&abs_gnutls_dh_params_t_ops, sizeof(struct absstruct_gnutls_dh_params_t), 0, 1);
  absstructptr_gnutls_dh_params_t_val(v)->tag = tag;
  absstructptr_gnutls_dh_params_t_val(v)->value = x;
  absstructptr_gnutls_dh_params_t_val(v)->oid = abs_gnutls_dh_params_t_oid++;
  r = caml_alloc(2,0);
  Field(r,0) = v;
  Field(r,1) = Val_int(0);
  CAMLreturn(r);
}

static value wrap_gnutls_dh_params_t(gnutls_dh_params_t x) {
  return twrap_gnutls_dh_params_t(0, x);
}

static void attach_gnutls_dh_params_t(value v, value aux) {
  CAMLparam2(v,aux);
  CAMLlocal1(h);
  h = caml_alloc(2,0);
  Field(h,0) = aux;
  Field(h,1) = Field(v,1);
  Store_field(v,1,h);
  CAMLreturn0;
}

/************** gnutls_priority_t *************/

struct absstruct_gnutls_priority_t { gnutls_priority_t value; long tag; long oid; };

#define absstructptr_gnutls_priority_t_val(v) ((struct absstruct_gnutls_priority_t *) (Data_custom_val(v)))
#define abs_gnutls_priority_t_unwrap(v) (absstructptr_gnutls_priority_t_val(v)->value)
#define abs_gnutls_priority_t_tag(v) (absstructptr_gnutls_priority_t_val(v)->tag)
long abs_gnutls_priority_t_oid = 0;

static int abs_gnutls_priority_t_compare(value v1, value v2) {
  struct absstruct_gnutls_priority_t *p1;
  struct absstruct_gnutls_priority_t *p2;
  p1 = absstructptr_gnutls_priority_t_val(v1);
  p2 = absstructptr_gnutls_priority_t_val(v2);
  return p1->oid - p2->oid;
}

static void abs_gnutls_priority_t_finalize(value v1) {
  struct absstruct_gnutls_priority_t *p1;
  p1 = absstructptr_gnutls_priority_t_val(v1);
  gnutls_priority_deinit(p1->value);
}

static struct custom_operations abs_gnutls_priority_t_ops = {
  "",
  abs_gnutls_priority_t_finalize,
  abs_gnutls_priority_t_compare,
  custom_hash_default,
  custom_serialize_default,
  custom_deserialize_default
};

static gnutls_priority_t unwrap_gnutls_priority_t(value v) {
  gnutls_priority_t r;
  r = abs_gnutls_priority_t_unwrap(Field(v,0));
  if (r == NULL) raise_null_pointer();
  return r;
}

static long tag_gnutls_priority_t(value v) {
  return abs_gnutls_priority_t_tag(Field(v,0));
}

static value twrap_gnutls_priority_t(long tag, gnutls_priority_t x) {
  CAMLparam0();
  CAMLlocal2(v,r);
  if (x == NULL) failwith("wrap_gnutls_priority_t: NULL pointer");
  v = caml_alloc_custom(&abs_gnutls_priority_t_ops, sizeof(struct absstruct_gnutls_priority_t), 0, 1);
  absstructptr_gnutls_priority_t_val(v)->tag = tag;
  absstructptr_gnutls_priority_t_val(v)->value = x;
  absstructptr_gnutls_priority_t_val(v)->oid = abs_gnutls_priority_t_oid++;
  r = caml_alloc(2,0);
  Field(r,0) = v;
  Field(r,1) = Val_int(0);
  CAMLreturn(r);
}

static value wrap_gnutls_priority_t(gnutls_priority_t x) {
  return twrap_gnutls_priority_t(0, x);
}

static void attach_gnutls_priority_t(value v, value aux) {
  CAMLparam2(v,aux);
  CAMLlocal1(h);
  h = caml_alloc(2,0);
  Field(h,0) = aux;
  Field(h,1) = Field(v,1);
  Store_field(v,1,h);
  CAMLreturn0;
}

/************** gnutls_cipher_hd_t *************/

#ifdef HAVE_TY_gnutls_cipher_hd_t
struct absstruct_gnutls_cipher_hd_t { gnutls_cipher_hd_t value; long tag; long oid; };

#define absstructptr_gnutls_cipher_hd_t_val(v) ((struct absstruct_gnutls_cipher_hd_t *) (Data_custom_val(v)))
#define abs_gnutls_cipher_hd_t_unwrap(v) (absstructptr_gnutls_cipher_hd_t_val(v)->value)
#define abs_gnutls_cipher_hd_t_tag(v) (absstructptr_gnutls_cipher_hd_t_val(v)->tag)
long abs_gnutls_cipher_hd_t_oid = 0;

static int abs_gnutls_cipher_hd_t_compare(value v1, value v2) {
  struct absstruct_gnutls_cipher_hd_t *p1;
  struct absstruct_gnutls_cipher_hd_t *p2;
  p1 = absstructptr_gnutls_cipher_hd_t_val(v1);
  p2 = absstructptr_gnutls_cipher_hd_t_val(v2);
  return p1->oid - p2->oid;
}

static void abs_gnutls_cipher_hd_t_finalize(value v1) {
  struct absstruct_gnutls_cipher_hd_t *p1;
  p1 = absstructptr_gnutls_cipher_hd_t_val(v1);
  gnutls_cipher_deinit(p1->value);
}

static struct custom_operations abs_gnutls_cipher_hd_t_ops = {
  "",
  abs_gnutls_cipher_hd_t_finalize,
  abs_gnutls_cipher_hd_t_compare,
  custom_hash_default,
  custom_serialize_default,
  custom_deserialize_default
};

static gnutls_cipher_hd_t unwrap_gnutls_cipher_hd_t(value v) {
  gnutls_cipher_hd_t r;
  r = abs_gnutls_cipher_hd_t_unwrap(Field(v,0));
  if (r == NULL) raise_null_pointer();
  return r;
}

static long tag_gnutls_cipher_hd_t(value v) {
  return abs_gnutls_cipher_hd_t_tag(Field(v,0));
}

static value twrap_gnutls_cipher_hd_t(long tag, gnutls_cipher_hd_t x) {
  CAMLparam0();
  CAMLlocal2(v,r);
  if (x == NULL) failwith("wrap_gnutls_cipher_hd_t: NULL pointer");
  v = caml_alloc_custom(&abs_gnutls_cipher_hd_t_ops, sizeof(struct absstruct_gnutls_cipher_hd_t), 0, 1);
  absstructptr_gnutls_cipher_hd_t_val(v)->tag = tag;
  absstructptr_gnutls_cipher_hd_t_val(v)->value = x;
  absstructptr_gnutls_cipher_hd_t_val(v)->oid = abs_gnutls_cipher_hd_t_oid++;
  r = caml_alloc(2,0);
  Field(r,0) = v;
  Field(r,1) = Val_int(0);
  CAMLreturn(r);
}

static value wrap_gnutls_cipher_hd_t(gnutls_cipher_hd_t x) {
  return twrap_gnutls_cipher_hd_t(0, x);
}

static void attach_gnutls_cipher_hd_t(value v, value aux) {
  CAMLparam2(v,aux);
  CAMLlocal1(h);
  h = caml_alloc(2,0);
  Field(h,0) = aux;
  Field(h,1) = Field(v,1);
  Store_field(v,1,h);
  CAMLreturn0;
}
#endif

/************** gnutls_x509_privkey_t *************/

struct absstruct_gnutls_x509_privkey_t { gnutls_x509_privkey_t value; long tag; long oid; };

#define absstructptr_gnutls_x509_privkey_t_val(v) ((struct absstruct_gnutls_x509_privkey_t *) (Data_custom_val(v)))
#define abs_gnutls_x509_privkey_t_unwrap(v) (absstructptr_gnutls_x509_privkey_t_val(v)->value)
#define abs_gnutls_x509_privkey_t_tag(v) (absstructptr_gnutls_x509_privkey_t_val(v)->tag)
long abs_gnutls_x509_privkey_t_oid = 0;

static int abs_gnutls_x509_privkey_t_compare(value v1, value v2) {
  struct absstruct_gnutls_x509_privkey_t *p1;
  struct absstruct_gnutls_x509_privkey_t *p2;
  p1 = absstructptr_gnutls_x509_privkey_t_val(v1);
  p2 = absstructptr_gnutls_x509_privkey_t_val(v2);
  return p1->oid - p2->oid;
}

static void abs_gnutls_x509_privkey_t_finalize(value v1) {
  struct absstruct_gnutls_x509_privkey_t *p1;
  p1 = absstructptr_gnutls_x509_privkey_t_val(v1);
  gnutls_x509_privkey_deinit(p1->value);
}

static struct custom_operations abs_gnutls_x509_privkey_t_ops = {
  "",
  abs_gnutls_x509_privkey_t_finalize,
  abs_gnutls_x509_privkey_t_compare,
  custom_hash_default,
  custom_serialize_default,
  custom_deserialize_default
};

static gnutls_x509_privkey_t unwrap_gnutls_x509_privkey_t(value v) {
  gnutls_x509_privkey_t r;
  r = abs_gnutls_x509_privkey_t_unwrap(Field(v,0));
  if (r == NULL) raise_null_pointer();
  return r;
}

static long tag_gnutls_x509_privkey_t(value v) {
  return abs_gnutls_x509_privkey_t_tag(Field(v,0));
}

static value twrap_gnutls_x509_privkey_t(long tag, gnutls_x509_privkey_t x) {
  CAMLparam0();
  CAMLlocal2(v,r);
  if (x == NULL) failwith("wrap_gnutls_x509_privkey_t: NULL pointer");
  v = caml_alloc_custom(&abs_gnutls_x509_privkey_t_ops, sizeof(struct absstruct_gnutls_x509_privkey_t), 0, 1);
  absstructptr_gnutls_x509_privkey_t_val(v)->tag = tag;
  absstructptr_gnutls_x509_privkey_t_val(v)->value = x;
  absstructptr_gnutls_x509_privkey_t_val(v)->oid = abs_gnutls_x509_privkey_t_oid++;
  r = caml_alloc(2,0);
  Field(r,0) = v;
  Field(r,1) = Val_int(0);
  CAMLreturn(r);
}

static value wrap_gnutls_x509_privkey_t(gnutls_x509_privkey_t x) {
  return twrap_gnutls_x509_privkey_t(0, x);
}

static void attach_gnutls_x509_privkey_t(value v, value aux) {
  CAMLparam2(v,aux);
  CAMLlocal1(h);
  h = caml_alloc(2,0);
  Field(h,0) = aux;
  Field(h,1) = Field(v,1);
  Store_field(v,1,h);
  CAMLreturn0;
}

/************** gnutls_x509_crl_t *************/

struct absstruct_gnutls_x509_crl_t { gnutls_x509_crl_t value; long tag; long oid; };

#define absstructptr_gnutls_x509_crl_t_val(v) ((struct absstruct_gnutls_x509_crl_t *) (Data_custom_val(v)))
#define abs_gnutls_x509_crl_t_unwrap(v) (absstructptr_gnutls_x509_crl_t_val(v)->value)
#define abs_gnutls_x509_crl_t_tag(v) (absstructptr_gnutls_x509_crl_t_val(v)->tag)
long abs_gnutls_x509_crl_t_oid = 0;

static int abs_gnutls_x509_crl_t_compare(value v1, value v2) {
  struct absstruct_gnutls_x509_crl_t *p1;
  struct absstruct_gnutls_x509_crl_t *p2;
  p1 = absstructptr_gnutls_x509_crl_t_val(v1);
  p2 = absstructptr_gnutls_x509_crl_t_val(v2);
  return p1->oid - p2->oid;
}

static void abs_gnutls_x509_crl_t_finalize(value v1) {
  struct absstruct_gnutls_x509_crl_t *p1;
  p1 = absstructptr_gnutls_x509_crl_t_val(v1);
  gnutls_x509_crl_deinit(p1->value);
}

static struct custom_operations abs_gnutls_x509_crl_t_ops = {
  "",
  abs_gnutls_x509_crl_t_finalize,
  abs_gnutls_x509_crl_t_compare,
  custom_hash_default,
  custom_serialize_default,
  custom_deserialize_default
};

static gnutls_x509_crl_t unwrap_gnutls_x509_crl_t(value v) {
  gnutls_x509_crl_t r;
  r = abs_gnutls_x509_crl_t_unwrap(Field(v,0));
  if (r == NULL) raise_null_pointer();
  return r;
}

static long tag_gnutls_x509_crl_t(value v) {
  return abs_gnutls_x509_crl_t_tag(Field(v,0));
}

static value twrap_gnutls_x509_crl_t(long tag, gnutls_x509_crl_t x) {
  CAMLparam0();
  CAMLlocal2(v,r);
  if (x == NULL) failwith("wrap_gnutls_x509_crl_t: NULL pointer");
  v = caml_alloc_custom(&abs_gnutls_x509_crl_t_ops, sizeof(struct absstruct_gnutls_x509_crl_t), 0, 1);
  absstructptr_gnutls_x509_crl_t_val(v)->tag = tag;
  absstructptr_gnutls_x509_crl_t_val(v)->value = x;
  absstructptr_gnutls_x509_crl_t_val(v)->oid = abs_gnutls_x509_crl_t_oid++;
  r = caml_alloc(2,0);
  Field(r,0) = v;
  Field(r,1) = Val_int(0);
  CAMLreturn(r);
}

static value wrap_gnutls_x509_crl_t(gnutls_x509_crl_t x) {
  return twrap_gnutls_x509_crl_t(0, x);
}

static void attach_gnutls_x509_crl_t(value v, value aux) {
  CAMLparam2(v,aux);
  CAMLlocal1(h);
  h = caml_alloc(2,0);
  Field(h,0) = aux;
  Field(h,1) = Field(v,1);
  Store_field(v,1,h);
  CAMLreturn0;
}

/************** gnutls_x509_crt_t *************/

struct absstruct_gnutls_x509_crt_t { gnutls_x509_crt_t value; long tag; long oid; };

#define absstructptr_gnutls_x509_crt_t_val(v) ((struct absstruct_gnutls_x509_crt_t *) (Data_custom_val(v)))
#define abs_gnutls_x509_crt_t_unwrap(v) (absstructptr_gnutls_x509_crt_t_val(v)->value)
#define abs_gnutls_x509_crt_t_tag(v) (absstructptr_gnutls_x509_crt_t_val(v)->tag)
long abs_gnutls_x509_crt_t_oid = 0;

static int abs_gnutls_x509_crt_t_compare(value v1, value v2) {
  struct absstruct_gnutls_x509_crt_t *p1;
  struct absstruct_gnutls_x509_crt_t *p2;
  p1 = absstructptr_gnutls_x509_crt_t_val(v1);
  p2 = absstructptr_gnutls_x509_crt_t_val(v2);
  return p1->oid - p2->oid;
}

static void abs_gnutls_x509_crt_t_finalize(value v1) {
  struct absstruct_gnutls_x509_crt_t *p1;
  p1 = absstructptr_gnutls_x509_crt_t_val(v1);
  gnutls_x509_crt_deinit(p1->value);
}

static struct custom_operations abs_gnutls_x509_crt_t_ops = {
  "",
  abs_gnutls_x509_crt_t_finalize,
  abs_gnutls_x509_crt_t_compare,
  custom_hash_default,
  custom_serialize_default,
  custom_deserialize_default
};

static gnutls_x509_crt_t unwrap_gnutls_x509_crt_t(value v) {
  gnutls_x509_crt_t r;
  r = abs_gnutls_x509_crt_t_unwrap(Field(v,0));
  if (r == NULL) raise_null_pointer();
  return r;
}

static long tag_gnutls_x509_crt_t(value v) {
  return abs_gnutls_x509_crt_t_tag(Field(v,0));
}

static value twrap_gnutls_x509_crt_t(long tag, gnutls_x509_crt_t x) {
  CAMLparam0();
  CAMLlocal2(v,r);
  if (x == NULL) failwith("wrap_gnutls_x509_crt_t: NULL pointer");
  v = caml_alloc_custom(&abs_gnutls_x509_crt_t_ops, sizeof(struct absstruct_gnutls_x509_crt_t), 0, 1);
  absstructptr_gnutls_x509_crt_t_val(v)->tag = tag;
  absstructptr_gnutls_x509_crt_t_val(v)->value = x;
  absstructptr_gnutls_x509_crt_t_val(v)->oid = abs_gnutls_x509_crt_t_oid++;
  r = caml_alloc(2,0);
  Field(r,0) = v;
  Field(r,1) = Val_int(0);
  CAMLreturn(r);
}

static value wrap_gnutls_x509_crt_t(gnutls_x509_crt_t x) {
  return twrap_gnutls_x509_crt_t(0, x);
}

static void attach_gnutls_x509_crt_t(value v, value aux) {
  CAMLparam2(v,aux);
  CAMLlocal1(h);
  h = caml_alloc(2,0);
  Field(h,0) = aux;
  Field(h,1) = Field(v,1);
  Store_field(v,1,h);
  CAMLreturn0;
}

/************** gnutls_certificate_credentials_t *************/

struct absstruct_gnutls_certificate_credentials_t { gnutls_certificate_credentials_t value; long tag; long oid; };

#define absstructptr_gnutls_certificate_credentials_t_val(v) ((struct absstruct_gnutls_certificate_credentials_t *) (Data_custom_val(v)))
#define abs_gnutls_certificate_credentials_t_unwrap(v) (absstructptr_gnutls_certificate_credentials_t_val(v)->value)
#define abs_gnutls_certificate_credentials_t_tag(v) (absstructptr_gnutls_certificate_credentials_t_val(v)->tag)
long abs_gnutls_certificate_credentials_t_oid = 0;

static int abs_gnutls_certificate_credentials_t_compare(value v1, value v2) {
  struct absstruct_gnutls_certificate_credentials_t *p1;
  struct absstruct_gnutls_certificate_credentials_t *p2;
  p1 = absstructptr_gnutls_certificate_credentials_t_val(v1);
  p2 = absstructptr_gnutls_certificate_credentials_t_val(v2);
  return p1->oid - p2->oid;
}

static void abs_gnutls_certificate_credentials_t_finalize(value v1) {
  struct absstruct_gnutls_certificate_credentials_t *p1;
  p1 = absstructptr_gnutls_certificate_credentials_t_val(v1);
  gnutls_certificate_free_credentials(p1->value);
}

static struct custom_operations abs_gnutls_certificate_credentials_t_ops = {
  "",
  abs_gnutls_certificate_credentials_t_finalize,
  abs_gnutls_certificate_credentials_t_compare,
  custom_hash_default,
  custom_serialize_default,
  custom_deserialize_default
};

static gnutls_certificate_credentials_t unwrap_gnutls_certificate_credentials_t(value v) {
  gnutls_certificate_credentials_t r;
  r = abs_gnutls_certificate_credentials_t_unwrap(Field(v,0));
  if (r == NULL) raise_null_pointer();
  return r;
}

static long tag_gnutls_certificate_credentials_t(value v) {
  return abs_gnutls_certificate_credentials_t_tag(Field(v,0));
}

static value twrap_gnutls_certificate_credentials_t(long tag, gnutls_certificate_credentials_t x) {
  CAMLparam0();
  CAMLlocal2(v,r);
  if (x == NULL) failwith("wrap_gnutls_certificate_credentials_t: NULL pointer");
  v = caml_alloc_custom(&abs_gnutls_certificate_credentials_t_ops, sizeof(struct absstruct_gnutls_certificate_credentials_t), 0, 1);
  absstructptr_gnutls_certificate_credentials_t_val(v)->tag = tag;
  absstructptr_gnutls_certificate_credentials_t_val(v)->value = x;
  absstructptr_gnutls_certificate_credentials_t_val(v)->oid = abs_gnutls_certificate_credentials_t_oid++;
  r = caml_alloc(2,0);
  Field(r,0) = v;
  Field(r,1) = Val_int(0);
  CAMLreturn(r);
}

static value wrap_gnutls_certificate_credentials_t(gnutls_certificate_credentials_t x) {
  return twrap_gnutls_certificate_credentials_t(0, x);
}

static void attach_gnutls_certificate_credentials_t(value v, value aux) {
  CAMLparam2(v,aux);
  CAMLlocal1(h);
  h = caml_alloc(2,0);
  Field(h,0) = aux;
  Field(h,1) = Field(v,1);
  Store_field(v,1,h);
  CAMLreturn0;
}

/************** gnutls_anon_server_credentials_t *************/

struct absstruct_gnutls_anon_server_credentials_t { gnutls_anon_server_credentials_t value; long tag; long oid; };

#define absstructptr_gnutls_anon_server_credentials_t_val(v) ((struct absstruct_gnutls_anon_server_credentials_t *) (Data_custom_val(v)))
#define abs_gnutls_anon_server_credentials_t_unwrap(v) (absstructptr_gnutls_anon_server_credentials_t_val(v)->value)
#define abs_gnutls_anon_server_credentials_t_tag(v) (absstructptr_gnutls_anon_server_credentials_t_val(v)->tag)
long abs_gnutls_anon_server_credentials_t_oid = 0;

static int abs_gnutls_anon_server_credentials_t_compare(value v1, value v2) {
  struct absstruct_gnutls_anon_server_credentials_t *p1;
  struct absstruct_gnutls_anon_server_credentials_t *p2;
  p1 = absstructptr_gnutls_anon_server_credentials_t_val(v1);
  p2 = absstructptr_gnutls_anon_server_credentials_t_val(v2);
  return p1->oid - p2->oid;
}

static void abs_gnutls_anon_server_credentials_t_finalize(value v1) {
  struct absstruct_gnutls_anon_server_credentials_t *p1;
  p1 = absstructptr_gnutls_anon_server_credentials_t_val(v1);
  gnutls_anon_free_server_credentials(p1->value);
}

static struct custom_operations abs_gnutls_anon_server_credentials_t_ops = {
  "",
  abs_gnutls_anon_server_credentials_t_finalize,
  abs_gnutls_anon_server_credentials_t_compare,
  custom_hash_default,
  custom_serialize_default,
  custom_deserialize_default
};

static gnutls_anon_server_credentials_t unwrap_gnutls_anon_server_credentials_t(value v) {
  gnutls_anon_server_credentials_t r;
  r = abs_gnutls_anon_server_credentials_t_unwrap(Field(v,0));
  if (r == NULL) raise_null_pointer();
  return r;
}

static long tag_gnutls_anon_server_credentials_t(value v) {
  return abs_gnutls_anon_server_credentials_t_tag(Field(v,0));
}

static value twrap_gnutls_anon_server_credentials_t(long tag, gnutls_anon_server_credentials_t x) {
  CAMLparam0();
  CAMLlocal2(v,r);
  if (x == NULL) failwith("wrap_gnutls_anon_server_credentials_t: NULL pointer");
  v = caml_alloc_custom(&abs_gnutls_anon_server_credentials_t_ops, sizeof(struct absstruct_gnutls_anon_server_credentials_t), 0, 1);
  absstructptr_gnutls_anon_server_credentials_t_val(v)->tag = tag;
  absstructptr_gnutls_anon_server_credentials_t_val(v)->value = x;
  absstructptr_gnutls_anon_server_credentials_t_val(v)->oid = abs_gnutls_anon_server_credentials_t_oid++;
  r = caml_alloc(2,0);
  Field(r,0) = v;
  Field(r,1) = Val_int(0);
  CAMLreturn(r);
}

static value wrap_gnutls_anon_server_credentials_t(gnutls_anon_server_credentials_t x) {
  return twrap_gnutls_anon_server_credentials_t(0, x);
}

static void attach_gnutls_anon_server_credentials_t(value v, value aux) {
  CAMLparam2(v,aux);
  CAMLlocal1(h);
  h = caml_alloc(2,0);
  Field(h,0) = aux;
  Field(h,1) = Field(v,1);
  Store_field(v,1,h);
  CAMLreturn0;
}

/************** gnutls_anon_client_credentials_t *************/

struct absstruct_gnutls_anon_client_credentials_t { gnutls_anon_client_credentials_t value; long tag; long oid; };

#define absstructptr_gnutls_anon_client_credentials_t_val(v) ((struct absstruct_gnutls_anon_client_credentials_t *) (Data_custom_val(v)))
#define abs_gnutls_anon_client_credentials_t_unwrap(v) (absstructptr_gnutls_anon_client_credentials_t_val(v)->value)
#define abs_gnutls_anon_client_credentials_t_tag(v) (absstructptr_gnutls_anon_client_credentials_t_val(v)->tag)
long abs_gnutls_anon_client_credentials_t_oid = 0;

static int abs_gnutls_anon_client_credentials_t_compare(value v1, value v2) {
  struct absstruct_gnutls_anon_client_credentials_t *p1;
  struct absstruct_gnutls_anon_client_credentials_t *p2;
  p1 = absstructptr_gnutls_anon_client_credentials_t_val(v1);
  p2 = absstructptr_gnutls_anon_client_credentials_t_val(v2);
  return p1->oid - p2->oid;
}

static void abs_gnutls_anon_client_credentials_t_finalize(value v1) {
  struct absstruct_gnutls_anon_client_credentials_t *p1;
  p1 = absstructptr_gnutls_anon_client_credentials_t_val(v1);
  gnutls_anon_free_client_credentials(p1->value);
}

static struct custom_operations abs_gnutls_anon_client_credentials_t_ops = {
  "",
  abs_gnutls_anon_client_credentials_t_finalize,
  abs_gnutls_anon_client_credentials_t_compare,
  custom_hash_default,
  custom_serialize_default,
  custom_deserialize_default
};

static gnutls_anon_client_credentials_t unwrap_gnutls_anon_client_credentials_t(value v) {
  gnutls_anon_client_credentials_t r;
  r = abs_gnutls_anon_client_credentials_t_unwrap(Field(v,0));
  if (r == NULL) raise_null_pointer();
  return r;
}

static long tag_gnutls_anon_client_credentials_t(value v) {
  return abs_gnutls_anon_client_credentials_t_tag(Field(v,0));
}

static value twrap_gnutls_anon_client_credentials_t(long tag, gnutls_anon_client_credentials_t x) {
  CAMLparam0();
  CAMLlocal2(v,r);
  if (x == NULL) failwith("wrap_gnutls_anon_client_credentials_t: NULL pointer");
  v = caml_alloc_custom(&abs_gnutls_anon_client_credentials_t_ops, sizeof(struct absstruct_gnutls_anon_client_credentials_t), 0, 1);
  absstructptr_gnutls_anon_client_credentials_t_val(v)->tag = tag;
  absstructptr_gnutls_anon_client_credentials_t_val(v)->value = x;
  absstructptr_gnutls_anon_client_credentials_t_val(v)->oid = abs_gnutls_anon_client_credentials_t_oid++;
  r = caml_alloc(2,0);
  Field(r,0) = v;
  Field(r,1) = Val_int(0);
  CAMLreturn(r);
}

static value wrap_gnutls_anon_client_credentials_t(gnutls_anon_client_credentials_t x) {
  return twrap_gnutls_anon_client_credentials_t(0, x);
}

static void attach_gnutls_anon_client_credentials_t(value v, value aux) {
  CAMLparam2(v,aux);
  CAMLlocal1(h);
  h = caml_alloc(2,0);
  Field(h,0) = aux;
  Field(h,1) = Field(v,1);
  Store_field(v,1,h);
  CAMLreturn0;
}

/************** gnutls_srp_server_credentials_t *************/

struct absstruct_gnutls_srp_server_credentials_t { gnutls_srp_server_credentials_t value; long tag; long oid; };

#define absstructptr_gnutls_srp_server_credentials_t_val(v) ((struct absstruct_gnutls_srp_server_credentials_t *) (Data_custom_val(v)))
#define abs_gnutls_srp_server_credentials_t_unwrap(v) (absstructptr_gnutls_srp_server_credentials_t_val(v)->value)
#define abs_gnutls_srp_server_credentials_t_tag(v) (absstructptr_gnutls_srp_server_credentials_t_val(v)->tag)
long abs_gnutls_srp_server_credentials_t_oid = 0;

static int abs_gnutls_srp_server_credentials_t_compare(value v1, value v2) {
  struct absstruct_gnutls_srp_server_credentials_t *p1;
  struct absstruct_gnutls_srp_server_credentials_t *p2;
  p1 = absstructptr_gnutls_srp_server_credentials_t_val(v1);
  p2 = absstructptr_gnutls_srp_server_credentials_t_val(v2);
  return p1->oid - p2->oid;
}

static void abs_gnutls_srp_server_credentials_t_finalize(value v1) {
  struct absstruct_gnutls_srp_server_credentials_t *p1;
  p1 = absstructptr_gnutls_srp_server_credentials_t_val(v1);
  gnutls_srp_free_server_credentials(p1->value);
}

static struct custom_operations abs_gnutls_srp_server_credentials_t_ops = {
  "",
  abs_gnutls_srp_server_credentials_t_finalize,
  abs_gnutls_srp_server_credentials_t_compare,
  custom_hash_default,
  custom_serialize_default,
  custom_deserialize_default
};

static gnutls_srp_server_credentials_t unwrap_gnutls_srp_server_credentials_t(value v) {
  gnutls_srp_server_credentials_t r;
  r = abs_gnutls_srp_server_credentials_t_unwrap(Field(v,0));
  if (r == NULL) raise_null_pointer();
  return r;
}

static long tag_gnutls_srp_server_credentials_t(value v) {
  return abs_gnutls_srp_server_credentials_t_tag(Field(v,0));
}

static value twrap_gnutls_srp_server_credentials_t(long tag, gnutls_srp_server_credentials_t x) {
  CAMLparam0();
  CAMLlocal2(v,r);
  if (x == NULL) failwith("wrap_gnutls_srp_server_credentials_t: NULL pointer");
  v = caml_alloc_custom(&abs_gnutls_srp_server_credentials_t_ops, sizeof(struct absstruct_gnutls_srp_server_credentials_t), 0, 1);
  absstructptr_gnutls_srp_server_credentials_t_val(v)->tag = tag;
  absstructptr_gnutls_srp_server_credentials_t_val(v)->value = x;
  absstructptr_gnutls_srp_server_credentials_t_val(v)->oid = abs_gnutls_srp_server_credentials_t_oid++;
  r = caml_alloc(2,0);
  Field(r,0) = v;
  Field(r,1) = Val_int(0);
  CAMLreturn(r);
}

static value wrap_gnutls_srp_server_credentials_t(gnutls_srp_server_credentials_t x) {
  return twrap_gnutls_srp_server_credentials_t(0, x);
}

static void attach_gnutls_srp_server_credentials_t(value v, value aux) {
  CAMLparam2(v,aux);
  CAMLlocal1(h);
  h = caml_alloc(2,0);
  Field(h,0) = aux;
  Field(h,1) = Field(v,1);
  Store_field(v,1,h);
  CAMLreturn0;
}

/************** gnutls_srp_client_credentials_t *************/

struct absstruct_gnutls_srp_client_credentials_t { gnutls_srp_client_credentials_t value; long tag; long oid; };

#define absstructptr_gnutls_srp_client_credentials_t_val(v) ((struct absstruct_gnutls_srp_client_credentials_t *) (Data_custom_val(v)))
#define abs_gnutls_srp_client_credentials_t_unwrap(v) (absstructptr_gnutls_srp_client_credentials_t_val(v)->value)
#define abs_gnutls_srp_client_credentials_t_tag(v) (absstructptr_gnutls_srp_client_credentials_t_val(v)->tag)
long abs_gnutls_srp_client_credentials_t_oid = 0;

static int abs_gnutls_srp_client_credentials_t_compare(value v1, value v2) {
  struct absstruct_gnutls_srp_client_credentials_t *p1;
  struct absstruct_gnutls_srp_client_credentials_t *p2;
  p1 = absstructptr_gnutls_srp_client_credentials_t_val(v1);
  p2 = absstructptr_gnutls_srp_client_credentials_t_val(v2);
  return p1->oid - p2->oid;
}

static void abs_gnutls_srp_client_credentials_t_finalize(value v1) {
  struct absstruct_gnutls_srp_client_credentials_t *p1;
  p1 = absstructptr_gnutls_srp_client_credentials_t_val(v1);
  gnutls_srp_free_client_credentials(p1->value);
}

static struct custom_operations abs_gnutls_srp_client_credentials_t_ops = {
  "",
  abs_gnutls_srp_client_credentials_t_finalize,
  abs_gnutls_srp_client_credentials_t_compare,
  custom_hash_default,
  custom_serialize_default,
  custom_deserialize_default
};

static gnutls_srp_client_credentials_t unwrap_gnutls_srp_client_credentials_t(value v) {
  gnutls_srp_client_credentials_t r;
  r = abs_gnutls_srp_client_credentials_t_unwrap(Field(v,0));
  if (r == NULL) raise_null_pointer();
  return r;
}

static long tag_gnutls_srp_client_credentials_t(value v) {
  return abs_gnutls_srp_client_credentials_t_tag(Field(v,0));
}

static value twrap_gnutls_srp_client_credentials_t(long tag, gnutls_srp_client_credentials_t x) {
  CAMLparam0();
  CAMLlocal2(v,r);
  if (x == NULL) failwith("wrap_gnutls_srp_client_credentials_t: NULL pointer");
  v = caml_alloc_custom(&abs_gnutls_srp_client_credentials_t_ops, sizeof(struct absstruct_gnutls_srp_client_credentials_t), 0, 1);
  absstructptr_gnutls_srp_client_credentials_t_val(v)->tag = tag;
  absstructptr_gnutls_srp_client_credentials_t_val(v)->value = x;
  absstructptr_gnutls_srp_client_credentials_t_val(v)->oid = abs_gnutls_srp_client_credentials_t_oid++;
  r = caml_alloc(2,0);
  Field(r,0) = v;
  Field(r,1) = Val_int(0);
  CAMLreturn(r);
}

static value wrap_gnutls_srp_client_credentials_t(gnutls_srp_client_credentials_t x) {
  return twrap_gnutls_srp_client_credentials_t(0, x);
}

static void attach_gnutls_srp_client_credentials_t(value v, value aux) {
  CAMLparam2(v,aux);
  CAMLlocal1(h);
  h = caml_alloc(2,0);
  Field(h,0) = aux;
  Field(h,1) = Field(v,1);
  Store_field(v,1,h);
  CAMLreturn0;
}

/************** gnutls_psk_server_credentials_t *************/

struct absstruct_gnutls_psk_server_credentials_t { gnutls_psk_server_credentials_t value; long tag; long oid; };

#define absstructptr_gnutls_psk_server_credentials_t_val(v) ((struct absstruct_gnutls_psk_server_credentials_t *) (Data_custom_val(v)))
#define abs_gnutls_psk_server_credentials_t_unwrap(v) (absstructptr_gnutls_psk_server_credentials_t_val(v)->value)
#define abs_gnutls_psk_server_credentials_t_tag(v) (absstructptr_gnutls_psk_server_credentials_t_val(v)->tag)
long abs_gnutls_psk_server_credentials_t_oid = 0;

static int abs_gnutls_psk_server_credentials_t_compare(value v1, value v2) {
  struct absstruct_gnutls_psk_server_credentials_t *p1;
  struct absstruct_gnutls_psk_server_credentials_t *p2;
  p1 = absstructptr_gnutls_psk_server_credentials_t_val(v1);
  p2 = absstructptr_gnutls_psk_server_credentials_t_val(v2);
  return p1->oid - p2->oid;
}

static void abs_gnutls_psk_server_credentials_t_finalize(value v1) {
  struct absstruct_gnutls_psk_server_credentials_t *p1;
  p1 = absstructptr_gnutls_psk_server_credentials_t_val(v1);
  gnutls_psk_free_server_credentials(p1->value);
}

static struct custom_operations abs_gnutls_psk_server_credentials_t_ops = {
  "",
  abs_gnutls_psk_server_credentials_t_finalize,
  abs_gnutls_psk_server_credentials_t_compare,
  custom_hash_default,
  custom_serialize_default,
  custom_deserialize_default
};

static gnutls_psk_server_credentials_t unwrap_gnutls_psk_server_credentials_t(value v) {
  gnutls_psk_server_credentials_t r;
  r = abs_gnutls_psk_server_credentials_t_unwrap(Field(v,0));
  if (r == NULL) raise_null_pointer();
  return r;
}

static long tag_gnutls_psk_server_credentials_t(value v) {
  return abs_gnutls_psk_server_credentials_t_tag(Field(v,0));
}

static value twrap_gnutls_psk_server_credentials_t(long tag, gnutls_psk_server_credentials_t x) {
  CAMLparam0();
  CAMLlocal2(v,r);
  if (x == NULL) failwith("wrap_gnutls_psk_server_credentials_t: NULL pointer");
  v = caml_alloc_custom(&abs_gnutls_psk_server_credentials_t_ops, sizeof(struct absstruct_gnutls_psk_server_credentials_t), 0, 1);
  absstructptr_gnutls_psk_server_credentials_t_val(v)->tag = tag;
  absstructptr_gnutls_psk_server_credentials_t_val(v)->value = x;
  absstructptr_gnutls_psk_server_credentials_t_val(v)->oid = abs_gnutls_psk_server_credentials_t_oid++;
  r = caml_alloc(2,0);
  Field(r,0) = v;
  Field(r,1) = Val_int(0);
  CAMLreturn(r);
}

static value wrap_gnutls_psk_server_credentials_t(gnutls_psk_server_credentials_t x) {
  return twrap_gnutls_psk_server_credentials_t(0, x);
}

static void attach_gnutls_psk_server_credentials_t(value v, value aux) {
  CAMLparam2(v,aux);
  CAMLlocal1(h);
  h = caml_alloc(2,0);
  Field(h,0) = aux;
  Field(h,1) = Field(v,1);
  Store_field(v,1,h);
  CAMLreturn0;
}

/************** gnutls_psk_client_credentials_t *************/

struct absstruct_gnutls_psk_client_credentials_t { gnutls_psk_client_credentials_t value; long tag; long oid; };

#define absstructptr_gnutls_psk_client_credentials_t_val(v) ((struct absstruct_gnutls_psk_client_credentials_t *) (Data_custom_val(v)))
#define abs_gnutls_psk_client_credentials_t_unwrap(v) (absstructptr_gnutls_psk_client_credentials_t_val(v)->value)
#define abs_gnutls_psk_client_credentials_t_tag(v) (absstructptr_gnutls_psk_client_credentials_t_val(v)->tag)
long abs_gnutls_psk_client_credentials_t_oid = 0;

static int abs_gnutls_psk_client_credentials_t_compare(value v1, value v2) {
  struct absstruct_gnutls_psk_client_credentials_t *p1;
  struct absstruct_gnutls_psk_client_credentials_t *p2;
  p1 = absstructptr_gnutls_psk_client_credentials_t_val(v1);
  p2 = absstructptr_gnutls_psk_client_credentials_t_val(v2);
  return p1->oid - p2->oid;
}

static void abs_gnutls_psk_client_credentials_t_finalize(value v1) {
  struct absstruct_gnutls_psk_client_credentials_t *p1;
  p1 = absstructptr_gnutls_psk_client_credentials_t_val(v1);
  gnutls_psk_free_client_credentials(p1->value);
}

static struct custom_operations abs_gnutls_psk_client_credentials_t_ops = {
  "",
  abs_gnutls_psk_client_credentials_t_finalize,
  abs_gnutls_psk_client_credentials_t_compare,
  custom_hash_default,
  custom_serialize_default,
  custom_deserialize_default
};

static gnutls_psk_client_credentials_t unwrap_gnutls_psk_client_credentials_t(value v) {
  gnutls_psk_client_credentials_t r;
  r = abs_gnutls_psk_client_credentials_t_unwrap(Field(v,0));
  if (r == NULL) raise_null_pointer();
  return r;
}

static long tag_gnutls_psk_client_credentials_t(value v) {
  return abs_gnutls_psk_client_credentials_t_tag(Field(v,0));
}

static value twrap_gnutls_psk_client_credentials_t(long tag, gnutls_psk_client_credentials_t x) {
  CAMLparam0();
  CAMLlocal2(v,r);
  if (x == NULL) failwith("wrap_gnutls_psk_client_credentials_t: NULL pointer");
  v = caml_alloc_custom(&abs_gnutls_psk_client_credentials_t_ops, sizeof(struct absstruct_gnutls_psk_client_credentials_t), 0, 1);
  absstructptr_gnutls_psk_client_credentials_t_val(v)->tag = tag;
  absstructptr_gnutls_psk_client_credentials_t_val(v)->value = x;
  absstructptr_gnutls_psk_client_credentials_t_val(v)->oid = abs_gnutls_psk_client_credentials_t_oid++;
  r = caml_alloc(2,0);
  Field(r,0) = v;
  Field(r,1) = Val_int(0);
  CAMLreturn(r);
}

static value wrap_gnutls_psk_client_credentials_t(gnutls_psk_client_credentials_t x) {
  return twrap_gnutls_psk_client_credentials_t(0, x);
}

static void attach_gnutls_psk_client_credentials_t(value v, value aux) {
  CAMLparam2(v,aux);
  CAMLlocal1(h);
  h = caml_alloc(2,0);
  Field(h,0) = aux;
  Field(h,1) = Field(v,1);
  Store_field(v,1,h);
  CAMLreturn0;
}

/************** gnutls_pubkey_t *************/

struct absstruct_gnutls_pubkey_t { gnutls_pubkey_t value; long tag; long oid; };

#define absstructptr_gnutls_pubkey_t_val(v) ((struct absstruct_gnutls_pubkey_t *) (Data_custom_val(v)))
#define abs_gnutls_pubkey_t_unwrap(v) (absstructptr_gnutls_pubkey_t_val(v)->value)
#define abs_gnutls_pubkey_t_tag(v) (absstructptr_gnutls_pubkey_t_val(v)->tag)
long abs_gnutls_pubkey_t_oid = 0;

static int abs_gnutls_pubkey_t_compare(value v1, value v2) {
  struct absstruct_gnutls_pubkey_t *p1;
  struct absstruct_gnutls_pubkey_t *p2;
  p1 = absstructptr_gnutls_pubkey_t_val(v1);
  p2 = absstructptr_gnutls_pubkey_t_val(v2);
  return p1->oid - p2->oid;
}

static void abs_gnutls_pubkey_t_finalize(value v1) {
  struct absstruct_gnutls_pubkey_t *p1;
  p1 = absstructptr_gnutls_pubkey_t_val(v1);
  gnutls_pubkey_deinit(p1->value);
}

static struct custom_operations abs_gnutls_pubkey_t_ops = {
  "",
  abs_gnutls_pubkey_t_finalize,
  abs_gnutls_pubkey_t_compare,
  custom_hash_default,
  custom_serialize_default,
  custom_deserialize_default
};

static gnutls_pubkey_t unwrap_gnutls_pubkey_t(value v) {
  gnutls_pubkey_t r;
  r = abs_gnutls_pubkey_t_unwrap(Field(v,0));
  if (r == NULL) raise_null_pointer();
  return r;
}

static long tag_gnutls_pubkey_t(value v) {
  return abs_gnutls_pubkey_t_tag(Field(v,0));
}

static value twrap_gnutls_pubkey_t(long tag, gnutls_pubkey_t x) {
  CAMLparam0();
  CAMLlocal2(v,r);
  if (x == NULL) failwith("wrap_gnutls_pubkey_t: NULL pointer");
  v = caml_alloc_custom(&abs_gnutls_pubkey_t_ops, sizeof(struct absstruct_gnutls_pubkey_t), 0, 1);
  absstructptr_gnutls_pubkey_t_val(v)->tag = tag;
  absstructptr_gnutls_pubkey_t_val(v)->value = x;
  absstructptr_gnutls_pubkey_t_val(v)->oid = abs_gnutls_pubkey_t_oid++;
  r = caml_alloc(2,0);
  Field(r,0) = v;
  Field(r,1) = Val_int(0);
  CAMLreturn(r);
}

static value wrap_gnutls_pubkey_t(gnutls_pubkey_t x) {
  return twrap_gnutls_pubkey_t(0, x);
}

static void attach_gnutls_pubkey_t(value v, value aux) {
  CAMLparam2(v,aux);
  CAMLlocal1(h);
  h = caml_alloc(2,0);
  Field(h,0) = aux;
  Field(h,1) = Field(v,1);
  Store_field(v,1,h);
  CAMLreturn0;
}

/************** gnutls_privkey_t *************/

struct absstruct_gnutls_privkey_t { gnutls_privkey_t value; long tag; long oid; };

#define absstructptr_gnutls_privkey_t_val(v) ((struct absstruct_gnutls_privkey_t *) (Data_custom_val(v)))
#define abs_gnutls_privkey_t_unwrap(v) (absstructptr_gnutls_privkey_t_val(v)->value)
#define abs_gnutls_privkey_t_tag(v) (absstructptr_gnutls_privkey_t_val(v)->tag)
long abs_gnutls_privkey_t_oid = 0;

static int abs_gnutls_privkey_t_compare(value v1, value v2) {
  struct absstruct_gnutls_privkey_t *p1;
  struct absstruct_gnutls_privkey_t *p2;
  p1 = absstructptr_gnutls_privkey_t_val(v1);
  p2 = absstructptr_gnutls_privkey_t_val(v2);
  return p1->oid - p2->oid;
}

static void abs_gnutls_privkey_t_finalize(value v1) {
  struct absstruct_gnutls_privkey_t *p1;
  p1 = absstructptr_gnutls_privkey_t_val(v1);
  gnutls_privkey_deinit(p1->value);
}

static struct custom_operations abs_gnutls_privkey_t_ops = {
  "",
  abs_gnutls_privkey_t_finalize,
  abs_gnutls_privkey_t_compare,
  custom_hash_default,
  custom_serialize_default,
  custom_deserialize_default
};

static gnutls_privkey_t unwrap_gnutls_privkey_t(value v) {
  gnutls_privkey_t r;
  r = abs_gnutls_privkey_t_unwrap(Field(v,0));
  if (r == NULL) raise_null_pointer();
  return r;
}

static long tag_gnutls_privkey_t(value v) {
  return abs_gnutls_privkey_t_tag(Field(v,0));
}

static value twrap_gnutls_privkey_t(long tag, gnutls_privkey_t x) {
  CAMLparam0();
  CAMLlocal2(v,r);
  if (x == NULL) failwith("wrap_gnutls_privkey_t: NULL pointer");
  v = caml_alloc_custom(&abs_gnutls_privkey_t_ops, sizeof(struct absstruct_gnutls_privkey_t), 0, 1);
  absstructptr_gnutls_privkey_t_val(v)->tag = tag;
  absstructptr_gnutls_privkey_t_val(v)->value = x;
  absstructptr_gnutls_privkey_t_val(v)->oid = abs_gnutls_privkey_t_oid++;
  r = caml_alloc(2,0);
  Field(r,0) = v;
  Field(r,1) = Val_int(0);
  CAMLreturn(r);
}

static value wrap_gnutls_privkey_t(gnutls_privkey_t x) {
  return twrap_gnutls_privkey_t(0, x);
}

static void attach_gnutls_privkey_t(value v, value aux) {
  CAMLparam2(v,aux);
  CAMLlocal1(h);
  h = caml_alloc(2,0);
  Field(h,0) = aux;
  Field(h,1) = Field(v,1);
  Store_field(v,1,h);
  CAMLreturn0;
}

/************** gnutls_params_type_t *************/

static value wrap_gnutls_params_type_t(gnutls_params_type_t x) {
  switch (x) {
    case GNUTLS_PARAMS_RSA_EXPORT: return Val_long(962490803);
    case GNUTLS_PARAMS_DH: return Val_long(15268);
#ifdef HAVE_ENUM_GNUTLS_PARAMS_ECDH
    case GNUTLS_PARAMS_ECDH: return Val_long(770125698);
#endif
    default: break;
  };
  failwith("wrap_gnutls_params_type_t: unexpected value");
}

static gnutls_params_type_t unwrap_gnutls_params_type_t(value v) {
  switch (Long_val(v)) {
    case 962490803: return GNUTLS_PARAMS_RSA_EXPORT;
    case 15268: return GNUTLS_PARAMS_DH;
#ifdef HAVE_ENUM_GNUTLS_PARAMS_ECDH
    case 770125698: return GNUTLS_PARAMS_ECDH;
#endif
    default: invalid_argument("unwrap_gnutls_params_type_t");
  };
  failwith("unwrap_gnutls_params_type_t: unexpected value");
}

/************** gnutls_credentials_type_t *************/

static value wrap_gnutls_credentials_type_t(gnutls_credentials_type_t x) {
  switch (x) {
    case GNUTLS_CRD_CERTIFICATE: return Val_long(-9084681);
    case GNUTLS_CRD_ANON: return Val_long(726316908);
    case GNUTLS_CRD_SRP: return Val_long(4153041);
    case GNUTLS_CRD_PSK: return Val_long(4004072);
    case GNUTLS_CRD_IA: return Val_long(16376);
    default: break;
  };
  failwith("wrap_gnutls_credentials_type_t: unexpected value");
}

static gnutls_credentials_type_t unwrap_gnutls_credentials_type_t(value v) {
  switch (Long_val(v)) {
    case -9084681: return GNUTLS_CRD_CERTIFICATE;
    case 726316908: return GNUTLS_CRD_ANON;
    case 4153041: return GNUTLS_CRD_SRP;
    case 4004072: return GNUTLS_CRD_PSK;
    case 16376: return GNUTLS_CRD_IA;
    default: invalid_argument("unwrap_gnutls_credentials_type_t");
  };
  failwith("unwrap_gnutls_credentials_type_t: unexpected value");
}

/************** gnutls_alert_level_t *************/

static value wrap_gnutls_alert_level_t(gnutls_alert_level_t x) {
  switch (x) {
    case GNUTLS_AL_WARNING: return Val_long(-685964740);
    case GNUTLS_AL_FATAL: return Val_long(243443684);
    default: break;
  };
  failwith("wrap_gnutls_alert_level_t: unexpected value");
}

static gnutls_alert_level_t unwrap_gnutls_alert_level_t(value v) {
  switch (Long_val(v)) {
    case -685964740: return GNUTLS_AL_WARNING;
    case 243443684: return GNUTLS_AL_FATAL;
    default: invalid_argument("unwrap_gnutls_alert_level_t");
  };
  failwith("unwrap_gnutls_alert_level_t: unexpected value");
}

/************** gnutls_alert_description_t *************/

static value wrap_gnutls_alert_description_t(gnutls_alert_description_t x) {
  switch (x) {
    case GNUTLS_A_CLOSE_NOTIFY: return Val_long(-423491632);
    case GNUTLS_A_UNEXPECTED_MESSAGE: return Val_long(-437026247);
    case GNUTLS_A_BAD_RECORD_MAC: return Val_long(143101499);
    case GNUTLS_A_DECRYPTION_FAILED: return Val_long(220224769);
    case GNUTLS_A_RECORD_OVERFLOW: return Val_long(190878160);
    case GNUTLS_A_DECOMPRESSION_FAILURE: return Val_long(542737456);
    case GNUTLS_A_HANDSHAKE_FAILURE: return Val_long(641382818);
    case GNUTLS_A_SSL3_NO_CERTIFICATE: return Val_long(53716561);
    case GNUTLS_A_BAD_CERTIFICATE: return Val_long(-318159811);
    case GNUTLS_A_UNSUPPORTED_CERTIFICATE: return Val_long(-738302163);
    case GNUTLS_A_CERTIFICATE_REVOKED: return Val_long(-435353386);
    case GNUTLS_A_CERTIFICATE_EXPIRED: return Val_long(-370906851);
    case GNUTLS_A_CERTIFICATE_UNKNOWN: return Val_long(636824834);
    case GNUTLS_A_ILLEGAL_PARAMETER: return Val_long(75791936);
    case GNUTLS_A_UNKNOWN_CA: return Val_long(-233988205);
    case GNUTLS_A_ACCESS_DENIED: return Val_long(621588918);
    case GNUTLS_A_DECODE_ERROR: return Val_long(-791686441);
    case GNUTLS_A_DECRYPT_ERROR: return Val_long(-271850762);
    case GNUTLS_A_EXPORT_RESTRICTION: return Val_long(54702497);
    case GNUTLS_A_PROTOCOL_VERSION: return Val_long(139102993);
    case GNUTLS_A_INSUFFICIENT_SECURITY: return Val_long(-232832612);
    case GNUTLS_A_INTERNAL_ERROR: return Val_long(163682854);
    case GNUTLS_A_USER_CANCELED: return Val_long(294497261);
    case GNUTLS_A_NO_RENEGOTIATION: return Val_long(-346505104);
    case GNUTLS_A_UNSUPPORTED_EXTENSION: return Val_long(118448149);
    case GNUTLS_A_CERTIFICATE_UNOBTAINABLE: return Val_long(1032770656);
    case GNUTLS_A_UNRECOGNIZED_NAME: return Val_long(-741729925);
    case GNUTLS_A_UNKNOWN_PSK_IDENTITY: return Val_long(-823957558);
#ifdef HAVE_ENUM_GNUTLS_A_NO_APPLICATION_PROTOCOL
    case GNUTLS_A_NO_APPLICATION_PROTOCOL: return Val_long(-682021819);
#endif
#ifdef HAVE_ENUM_GNUTLS_A_INNER_APPLICATION_FAILURE
    case GNUTLS_A_INNER_APPLICATION_FAILURE: return Val_long(-786449262);
#endif
#ifdef HAVE_ENUM_GNUTLS_A_INNER_APPLICATION_VERIFICATION
    case GNUTLS_A_INNER_APPLICATION_VERIFICATION: return Val_long(697987667);
#endif
    default: break;
  };
  failwith("wrap_gnutls_alert_description_t: unexpected value");
}

static gnutls_alert_description_t unwrap_gnutls_alert_description_t(value v) {
  switch (Long_val(v)) {
    case -423491632: return GNUTLS_A_CLOSE_NOTIFY;
    case -437026247: return GNUTLS_A_UNEXPECTED_MESSAGE;
    case 143101499: return GNUTLS_A_BAD_RECORD_MAC;
    case 220224769: return GNUTLS_A_DECRYPTION_FAILED;
    case 190878160: return GNUTLS_A_RECORD_OVERFLOW;
    case 542737456: return GNUTLS_A_DECOMPRESSION_FAILURE;
    case 641382818: return GNUTLS_A_HANDSHAKE_FAILURE;
    case 53716561: return GNUTLS_A_SSL3_NO_CERTIFICATE;
    case -318159811: return GNUTLS_A_BAD_CERTIFICATE;
    case -738302163: return GNUTLS_A_UNSUPPORTED_CERTIFICATE;
    case -435353386: return GNUTLS_A_CERTIFICATE_REVOKED;
    case -370906851: return GNUTLS_A_CERTIFICATE_EXPIRED;
    case 636824834: return GNUTLS_A_CERTIFICATE_UNKNOWN;
    case 75791936: return GNUTLS_A_ILLEGAL_PARAMETER;
    case -233988205: return GNUTLS_A_UNKNOWN_CA;
    case 621588918: return GNUTLS_A_ACCESS_DENIED;
    case -791686441: return GNUTLS_A_DECODE_ERROR;
    case -271850762: return GNUTLS_A_DECRYPT_ERROR;
    case 54702497: return GNUTLS_A_EXPORT_RESTRICTION;
    case 139102993: return GNUTLS_A_PROTOCOL_VERSION;
    case -232832612: return GNUTLS_A_INSUFFICIENT_SECURITY;
    case 163682854: return GNUTLS_A_INTERNAL_ERROR;
    case 294497261: return GNUTLS_A_USER_CANCELED;
    case -346505104: return GNUTLS_A_NO_RENEGOTIATION;
    case 118448149: return GNUTLS_A_UNSUPPORTED_EXTENSION;
    case 1032770656: return GNUTLS_A_CERTIFICATE_UNOBTAINABLE;
    case -741729925: return GNUTLS_A_UNRECOGNIZED_NAME;
    case -823957558: return GNUTLS_A_UNKNOWN_PSK_IDENTITY;
#ifdef HAVE_ENUM_GNUTLS_A_NO_APPLICATION_PROTOCOL
    case -682021819: return GNUTLS_A_NO_APPLICATION_PROTOCOL;
#endif
#ifdef HAVE_ENUM_GNUTLS_A_INNER_APPLICATION_FAILURE
    case -786449262: return GNUTLS_A_INNER_APPLICATION_FAILURE;
#endif
#ifdef HAVE_ENUM_GNUTLS_A_INNER_APPLICATION_VERIFICATION
    case 697987667: return GNUTLS_A_INNER_APPLICATION_VERIFICATION;
#endif
    default: invalid_argument("unwrap_gnutls_alert_description_t");
  };
  failwith("unwrap_gnutls_alert_description_t: unexpected value");
}

/************** gnutls_handshake_description_t *************/

static value wrap_gnutls_handshake_description_t(gnutls_handshake_description_t x) {
  switch (x) {
    case GNUTLS_HANDSHAKE_HELLO_REQUEST: return Val_long(13835714);
    case GNUTLS_HANDSHAKE_CLIENT_HELLO: return Val_long(919067390);
    case GNUTLS_HANDSHAKE_SERVER_HELLO: return Val_long(996083062);
#ifdef HAVE_ENUM_GNUTLS_HANDSHAKE_HELLO_VERIFY_REQUEST
    case GNUTLS_HANDSHAKE_HELLO_VERIFY_REQUEST: return Val_long(638657334);
#endif
#ifdef HAVE_ENUM_GNUTLS_HANDSHAKE_NEW_SESSION_TICKET
    case GNUTLS_HANDSHAKE_NEW_SESSION_TICKET: return Val_long(-88824204);
#endif
    case GNUTLS_HANDSHAKE_CERTIFICATE_PKT: return Val_long(-859879535);
    case GNUTLS_HANDSHAKE_SERVER_KEY_EXCHANGE: return Val_long(608831007);
    case GNUTLS_HANDSHAKE_CERTIFICATE_REQUEST: return Val_long(150923463);
    case GNUTLS_HANDSHAKE_SERVER_HELLO_DONE: return Val_long(413825835);
    case GNUTLS_HANDSHAKE_CERTIFICATE_VERIFY: return Val_long(861491457);
    case GNUTLS_HANDSHAKE_CLIENT_KEY_EXCHANGE: return Val_long(-1046318697);
    case GNUTLS_HANDSHAKE_FINISHED: return Val_long(-289386606);
    case GNUTLS_HANDSHAKE_SUPPLEMENTAL: return Val_long(798404228);
#ifdef HAVE_ENUM_GNUTLS_HANDSHAKE_CHANGE_CIPHER_SPEC
    case GNUTLS_HANDSHAKE_CHANGE_CIPHER_SPEC: return Val_long(-167332928);
#endif
#ifdef HAVE_ENUM_GNUTLS_HANDSHAKE_CLIENT_HELLO_V2
    case GNUTLS_HANDSHAKE_CLIENT_HELLO_V2: return Val_long(-1068952355);
#endif
    default: break;
  };
  failwith("wrap_gnutls_handshake_description_t: unexpected value");
}

static gnutls_handshake_description_t unwrap_gnutls_handshake_description_t(value v) {
  switch (Long_val(v)) {
    case 13835714: return GNUTLS_HANDSHAKE_HELLO_REQUEST;
    case 919067390: return GNUTLS_HANDSHAKE_CLIENT_HELLO;
    case 996083062: return GNUTLS_HANDSHAKE_SERVER_HELLO;
#ifdef HAVE_ENUM_GNUTLS_HANDSHAKE_HELLO_VERIFY_REQUEST
    case 638657334: return GNUTLS_HANDSHAKE_HELLO_VERIFY_REQUEST;
#endif
#ifdef HAVE_ENUM_GNUTLS_HANDSHAKE_NEW_SESSION_TICKET
    case -88824204: return GNUTLS_HANDSHAKE_NEW_SESSION_TICKET;
#endif
    case -859879535: return GNUTLS_HANDSHAKE_CERTIFICATE_PKT;
    case 608831007: return GNUTLS_HANDSHAKE_SERVER_KEY_EXCHANGE;
    case 150923463: return GNUTLS_HANDSHAKE_CERTIFICATE_REQUEST;
    case 413825835: return GNUTLS_HANDSHAKE_SERVER_HELLO_DONE;
    case 861491457: return GNUTLS_HANDSHAKE_CERTIFICATE_VERIFY;
    case -1046318697: return GNUTLS_HANDSHAKE_CLIENT_KEY_EXCHANGE;
    case -289386606: return GNUTLS_HANDSHAKE_FINISHED;
    case 798404228: return GNUTLS_HANDSHAKE_SUPPLEMENTAL;
#ifdef HAVE_ENUM_GNUTLS_HANDSHAKE_CHANGE_CIPHER_SPEC
    case -167332928: return GNUTLS_HANDSHAKE_CHANGE_CIPHER_SPEC;
#endif
#ifdef HAVE_ENUM_GNUTLS_HANDSHAKE_CLIENT_HELLO_V2
    case -1068952355: return GNUTLS_HANDSHAKE_CLIENT_HELLO_V2;
#endif
    default: invalid_argument("unwrap_gnutls_handshake_description_t");
  };
  failwith("unwrap_gnutls_handshake_description_t: unexpected value");
}

/************** gnutls_certificate_status_t *************/

static value wrap_gnutls_certificate_status_t(gnutls_certificate_status_t x) {
  CAMLparam0();
  CAMLlocal2(v,u);
  v = Val_long(0);
  if (x & GNUTLS_CERT_INVALID) {
    u = caml_alloc(2,0);
    Field(u, 0) = Val_long(144245463);
    Field(u, 1) = v;
    v = u;
  };
  if (x & GNUTLS_CERT_REVOKED) {
    u = caml_alloc(2,0);
    Field(u, 0) = Val_long(-287037634);
    Field(u, 1) = v;
    v = u;
  };
  if (x & GNUTLS_CERT_SIGNER_NOT_FOUND) {
    u = caml_alloc(2,0);
    Field(u, 0) = Val_long(-196377311);
    Field(u, 1) = v;
    v = u;
  };
  if (x & GNUTLS_CERT_SIGNER_NOT_CA) {
    u = caml_alloc(2,0);
    Field(u, 0) = Val_long(-532225441);
    Field(u, 1) = v;
    v = u;
  };
  if (x & GNUTLS_CERT_INSECURE_ALGORITHM) {
    u = caml_alloc(2,0);
    Field(u, 0) = Val_long(-21185172);
    Field(u, 1) = v;
    v = u;
  };
  if (x & GNUTLS_CERT_NOT_ACTIVATED) {
    u = caml_alloc(2,0);
    Field(u, 0) = Val_long(335412165);
    Field(u, 1) = v;
    v = u;
  };
  if (x & GNUTLS_CERT_EXPIRED) {
    u = caml_alloc(2,0);
    Field(u, 0) = Val_long(-222591099);
    Field(u, 1) = v;
    v = u;
  };
#ifdef HAVE_ENUM_GNUTLS_CERT_SIGNATURE_FAILURE
  if (x & GNUTLS_CERT_SIGNATURE_FAILURE) {
    u = caml_alloc(2,0);
    Field(u, 0) = Val_long(150201507);
    Field(u, 1) = v;
    v = u;
  };
#endif
#ifdef HAVE_ENUM_GNUTLS_CERT_REVOCATION_DATA_SUPERSEDED
  if (x & GNUTLS_CERT_REVOCATION_DATA_SUPERSEDED) {
    u = caml_alloc(2,0);
    Field(u, 0) = Val_long(-579386102);
    Field(u, 1) = v;
    v = u;
  };
#endif
#ifdef HAVE_ENUM_GNUTLS_CERT_UNEXPECTED_OWNER
  if (x & GNUTLS_CERT_UNEXPECTED_OWNER) {
    u = caml_alloc(2,0);
    Field(u, 0) = Val_long(957383717);
    Field(u, 1) = v;
    v = u;
  };
#endif
#ifdef HAVE_ENUM_GNUTLS_CERT_REVOCATION_DATA_ISSUED_IN_FUTURE
  if (x & GNUTLS_CERT_REVOCATION_DATA_ISSUED_IN_FUTURE) {
    u = caml_alloc(2,0);
    Field(u, 0) = Val_long(-886616707);
    Field(u, 1) = v;
    v = u;
  };
#endif
#ifdef HAVE_ENUM_GNUTLS_CERT_SIGNER_CONSTRAINTS_FAILURE
  if (x & GNUTLS_CERT_SIGNER_CONSTRAINTS_FAILURE) {
    u = caml_alloc(2,0);
    Field(u, 0) = Val_long(-1065140404);
    Field(u, 1) = v;
    v = u;
  };
#endif
#ifdef HAVE_ENUM_GNUTLS_CERT_MISMATCH
  if (x & GNUTLS_CERT_MISMATCH) {
    u = caml_alloc(2,0);
    Field(u, 0) = Val_long(-1062743954);
    Field(u, 1) = v;
    v = u;
  };
#endif
#ifdef HAVE_ENUM_GNUTLS_CERT_PURPOSE_MISMATCH
  if (x & GNUTLS_CERT_PURPOSE_MISMATCH) {
    u = caml_alloc(2,0);
    Field(u, 0) = Val_long(-45419921);
    Field(u, 1) = v;
    v = u;
  };
#endif
  CAMLreturn(v);
}

static gnutls_certificate_status_t unwrap_gnutls_certificate_status_t(value v) {
  gnutls_certificate_status_t x = 0;
  while (Is_block(v)) {
    switch (Long_val(Field(v,0))) {
      case 144245463: x |= GNUTLS_CERT_INVALID; break;
      case -287037634: x |= GNUTLS_CERT_REVOKED; break;
      case -196377311: x |= GNUTLS_CERT_SIGNER_NOT_FOUND; break;
      case -532225441: x |= GNUTLS_CERT_SIGNER_NOT_CA; break;
      case -21185172: x |= GNUTLS_CERT_INSECURE_ALGORITHM; break;
      case 335412165: x |= GNUTLS_CERT_NOT_ACTIVATED; break;
      case -222591099: x |= GNUTLS_CERT_EXPIRED; break;
#ifdef HAVE_ENUM_GNUTLS_CERT_SIGNATURE_FAILURE
      case 150201507: x |= GNUTLS_CERT_SIGNATURE_FAILURE; break;
#endif
#ifdef HAVE_ENUM_GNUTLS_CERT_REVOCATION_DATA_SUPERSEDED
      case -579386102: x |= GNUTLS_CERT_REVOCATION_DATA_SUPERSEDED; break;
#endif
#ifdef HAVE_ENUM_GNUTLS_CERT_UNEXPECTED_OWNER
      case 957383717: x |= GNUTLS_CERT_UNEXPECTED_OWNER; break;
#endif
#ifdef HAVE_ENUM_GNUTLS_CERT_REVOCATION_DATA_ISSUED_IN_FUTURE
      case -886616707: x |= GNUTLS_CERT_REVOCATION_DATA_ISSUED_IN_FUTURE; break;
#endif
#ifdef HAVE_ENUM_GNUTLS_CERT_SIGNER_CONSTRAINTS_FAILURE
      case -1065140404: x |= GNUTLS_CERT_SIGNER_CONSTRAINTS_FAILURE; break;
#endif
#ifdef HAVE_ENUM_GNUTLS_CERT_MISMATCH
      case -1062743954: x |= GNUTLS_CERT_MISMATCH; break;
#endif
#ifdef HAVE_ENUM_GNUTLS_CERT_PURPOSE_MISMATCH
      case -45419921: x |= GNUTLS_CERT_PURPOSE_MISMATCH; break;
#endif
    };
    v = Field(v,1);
  };
  return x;
}

/************** gnutls_certificate_request_t *************/

static value wrap_gnutls_certificate_request_t(gnutls_certificate_request_t x) {
  switch (x) {
    case GNUTLS_CERT_IGNORE: return Val_long(-950194894);
    case GNUTLS_CERT_REQUEST: return Val_long(299239215);
    case GNUTLS_CERT_REQUIRE: return Val_long(299437893);
    default: break;
  };
  failwith("wrap_gnutls_certificate_request_t: unexpected value");
}

static gnutls_certificate_request_t unwrap_gnutls_certificate_request_t(value v) {
  switch (Long_val(v)) {
    case -950194894: return GNUTLS_CERT_IGNORE;
    case 299239215: return GNUTLS_CERT_REQUEST;
    case 299437893: return GNUTLS_CERT_REQUIRE;
    default: invalid_argument("unwrap_gnutls_certificate_request_t");
  };
  failwith("unwrap_gnutls_certificate_request_t: unexpected value");
}

/************** gnutls_certificate_verify_flags *************/

static value wrap_gnutls_certificate_verify_flags(gnutls_certificate_verify_flags x) {
  CAMLparam0();
  CAMLlocal2(v,u);
  v = Val_long(0);
  if (x & GNUTLS_VERIFY_DISABLE_CA_SIGN) {
    u = caml_alloc(2,0);
    Field(u, 0) = Val_long(-280720377);
    Field(u, 1) = v;
    v = u;
  };
  if (x & GNUTLS_VERIFY_ALLOW_X509_V1_CA_CRT) {
    u = caml_alloc(2,0);
    Field(u, 0) = Val_long(-376724699);
    Field(u, 1) = v;
    v = u;
  };
  if (x & GNUTLS_VERIFY_DO_NOT_ALLOW_SAME) {
    u = caml_alloc(2,0);
    Field(u, 0) = Val_long(154830428);
    Field(u, 1) = v;
    v = u;
  };
  if (x & GNUTLS_VERIFY_ALLOW_ANY_X509_V1_CA_CRT) {
    u = caml_alloc(2,0);
    Field(u, 0) = Val_long(762838360);
    Field(u, 1) = v;
    v = u;
  };
  if (x & GNUTLS_VERIFY_ALLOW_SIGN_RSA_MD2) {
    u = caml_alloc(2,0);
    Field(u, 0) = Val_long(618665744);
    Field(u, 1) = v;
    v = u;
  };
  if (x & GNUTLS_VERIFY_ALLOW_SIGN_RSA_MD5) {
    u = caml_alloc(2,0);
    Field(u, 0) = Val_long(618665747);
    Field(u, 1) = v;
    v = u;
  };
  if (x & GNUTLS_VERIFY_DISABLE_TIME_CHECKS) {
    u = caml_alloc(2,0);
    Field(u, 0) = Val_long(288715686);
    Field(u, 1) = v;
    v = u;
  };
#ifdef HAVE_ENUM_GNUTLS_VERIFY_DISABLE_TRUSTED_TIME_CHECKS
  if (x & GNUTLS_VERIFY_DISABLE_TRUSTED_TIME_CHECKS) {
    u = caml_alloc(2,0);
    Field(u, 0) = Val_long(488890654);
    Field(u, 1) = v;
    v = u;
  };
#endif
#ifdef HAVE_ENUM_GNUTLS_VERIFY_DO_NOT_ALLOW_X509_V1_CA_CRT
  if (x & GNUTLS_VERIFY_DO_NOT_ALLOW_X509_V1_CA_CRT) {
    u = caml_alloc(2,0);
    Field(u, 0) = Val_long(-141319355);
    Field(u, 1) = v;
    v = u;
  };
#endif
#ifdef HAVE_ENUM_GNUTLS_VERIFY_DISABLE_CRL_CHECKS
  if (x & GNUTLS_VERIFY_DISABLE_CRL_CHECKS) {
    u = caml_alloc(2,0);
    Field(u, 0) = Val_long(-750720156);
    Field(u, 1) = v;
    v = u;
  };
#endif
  CAMLreturn(v);
}

static gnutls_certificate_verify_flags unwrap_gnutls_certificate_verify_flags(value v) {
  gnutls_certificate_verify_flags x = 0;
  while (Is_block(v)) {
    switch (Long_val(Field(v,0))) {
      case -280720377: x |= GNUTLS_VERIFY_DISABLE_CA_SIGN; break;
      case -376724699: x |= GNUTLS_VERIFY_ALLOW_X509_V1_CA_CRT; break;
      case 154830428: x |= GNUTLS_VERIFY_DO_NOT_ALLOW_SAME; break;
      case 762838360: x |= GNUTLS_VERIFY_ALLOW_ANY_X509_V1_CA_CRT; break;
      case 618665744: x |= GNUTLS_VERIFY_ALLOW_SIGN_RSA_MD2; break;
      case 618665747: x |= GNUTLS_VERIFY_ALLOW_SIGN_RSA_MD5; break;
      case 288715686: x |= GNUTLS_VERIFY_DISABLE_TIME_CHECKS; break;
#ifdef HAVE_ENUM_GNUTLS_VERIFY_DISABLE_TRUSTED_TIME_CHECKS
      case 488890654: x |= GNUTLS_VERIFY_DISABLE_TRUSTED_TIME_CHECKS; break;
#endif
#ifdef HAVE_ENUM_GNUTLS_VERIFY_DO_NOT_ALLOW_X509_V1_CA_CRT
      case -141319355: x |= GNUTLS_VERIFY_DO_NOT_ALLOW_X509_V1_CA_CRT; break;
#endif
#ifdef HAVE_ENUM_GNUTLS_VERIFY_DISABLE_CRL_CHECKS
      case -750720156: x |= GNUTLS_VERIFY_DISABLE_CRL_CHECKS; break;
#endif
    };
    v = Field(v,1);
  };
  return x;
}

/************** gnutls_close_request_t *************/

static value wrap_gnutls_close_request_t(gnutls_close_request_t x) {
  switch (x) {
    case GNUTLS_SHUT_RDWR: return Val_long(914344045);
    case GNUTLS_SHUT_WR: return Val_long(19515);
    default: break;
  };
  failwith("wrap_gnutls_close_request_t: unexpected value");
}

static gnutls_close_request_t unwrap_gnutls_close_request_t(value v) {
  switch (Long_val(v)) {
    case 914344045: return GNUTLS_SHUT_RDWR;
    case 19515: return GNUTLS_SHUT_WR;
    default: invalid_argument("unwrap_gnutls_close_request_t");
  };
  failwith("unwrap_gnutls_close_request_t: unexpected value");
}

/************** gnutls_protocol_t *************/

static value wrap_gnutls_protocol_t(gnutls_protocol_t x) {
  switch (x) {
    case GNUTLS_SSL3: return Val_long(926177031);
    case GNUTLS_TLS1_0: return Val_long(292159591);
    case GNUTLS_TLS1_1: return Val_long(292159592);
    case GNUTLS_TLS1_2: return Val_long(292159593);
#ifdef HAVE_ENUM_GNUTLS_DTLS0_9
    case GNUTLS_DTLS0_9: return Val_long(-393718445);
#endif
#ifdef HAVE_ENUM_GNUTLS_DTLS1_0
    case GNUTLS_DTLS1_0: return Val_long(-393668725);
#endif
#ifdef HAVE_ENUM_GNUTLS_DTLS1_2
    case GNUTLS_DTLS1_2: return Val_long(-393668723);
#endif
    case GNUTLS_VERSION_UNKNOWN: return Val_long(-300792829);
    default: break;
  };
  failwith("wrap_gnutls_protocol_t: unexpected value");
}

static gnutls_protocol_t unwrap_gnutls_protocol_t(value v) {
  switch (Long_val(v)) {
    case 926177031: return GNUTLS_SSL3;
    case 292159591: return GNUTLS_TLS1_0;
    case 292159592: return GNUTLS_TLS1_1;
    case 292159593: return GNUTLS_TLS1_2;
#ifdef HAVE_ENUM_GNUTLS_DTLS0_9
    case -393718445: return GNUTLS_DTLS0_9;
#endif
#ifdef HAVE_ENUM_GNUTLS_DTLS1_0
    case -393668725: return GNUTLS_DTLS1_0;
#endif
#ifdef HAVE_ENUM_GNUTLS_DTLS1_2
    case -393668723: return GNUTLS_DTLS1_2;
#endif
    case -300792829: return GNUTLS_VERSION_UNKNOWN;
    default: invalid_argument("unwrap_gnutls_protocol_t");
  };
  failwith("unwrap_gnutls_protocol_t: unexpected value");
}

/************** gnutls_x509_crt_fmt_t *************/

static value wrap_gnutls_x509_crt_fmt_t(gnutls_x509_crt_fmt_t x) {
  switch (x) {
    case GNUTLS_X509_FMT_DER: return Val_long(3404209);
    case GNUTLS_X509_FMT_PEM: return Val_long(4000952);
    default: break;
  };
  failwith("wrap_gnutls_x509_crt_fmt_t: unexpected value");
}

static gnutls_x509_crt_fmt_t unwrap_gnutls_x509_crt_fmt_t(value v) {
  switch (Long_val(v)) {
    case 3404209: return GNUTLS_X509_FMT_DER;
    case 4000952: return GNUTLS_X509_FMT_PEM;
    default: invalid_argument("unwrap_gnutls_x509_crt_fmt_t");
  };
  failwith("unwrap_gnutls_x509_crt_fmt_t: unexpected value");
}

/************** gnutls_certificate_print_formats_t *************/

static value wrap_gnutls_certificate_print_formats_t(gnutls_certificate_print_formats_t x) {
  switch (x) {
    case GNUTLS_CRT_PRINT_FULL: return Val_long(782112175);
    case GNUTLS_CRT_PRINT_ONELINE: return Val_long(1066206522);
    case GNUTLS_CRT_PRINT_UNSIGNED_FULL: return Val_long(-118727623);
#ifdef HAVE_ENUM_GNUTLS_CRT_PRINT_COMPACT
    case GNUTLS_CRT_PRINT_COMPACT: return Val_long(-793103485);
#endif
#ifdef HAVE_ENUM_GNUTLS_CRT_PRINT_FULL_NUMBERS
    case GNUTLS_CRT_PRINT_FULL_NUMBERS: return Val_long(-524968006);
#endif
    default: break;
  };
  failwith("wrap_gnutls_certificate_print_formats_t: unexpected value");
}

static gnutls_certificate_print_formats_t unwrap_gnutls_certificate_print_formats_t(value v) {
  switch (Long_val(v)) {
    case 782112175: return GNUTLS_CRT_PRINT_FULL;
    case 1066206522: return GNUTLS_CRT_PRINT_ONELINE;
    case -118727623: return GNUTLS_CRT_PRINT_UNSIGNED_FULL;
#ifdef HAVE_ENUM_GNUTLS_CRT_PRINT_COMPACT
    case -793103485: return GNUTLS_CRT_PRINT_COMPACT;
#endif
#ifdef HAVE_ENUM_GNUTLS_CRT_PRINT_FULL_NUMBERS
    case -524968006: return GNUTLS_CRT_PRINT_FULL_NUMBERS;
#endif
    default: invalid_argument("unwrap_gnutls_certificate_print_formats_t");
  };
  failwith("unwrap_gnutls_certificate_print_formats_t: unexpected value");
}

/************** gnutls_ecc_curve_t *************/

#ifdef HAVE_TY_gnutls_ecc_curve_t
static value wrap_gnutls_ecc_curve_t(gnutls_ecc_curve_t x) {
  switch (x) {
    case GNUTLS_ECC_CURVE_INVALID: return Val_long(144245463);
    case GNUTLS_ECC_CURVE_SECP224R1: return Val_long(1026485428);
    case GNUTLS_ECC_CURVE_SECP256R1: return Val_long(1059853587);
    case GNUTLS_ECC_CURVE_SECP384R1: return Val_long(-728971025);
    case GNUTLS_ECC_CURVE_SECP521R1: return Val_long(-144678028);
    case GNUTLS_ECC_CURVE_SECP192R1: return Val_long(778523146);
    default: break;
  };
  failwith("wrap_gnutls_ecc_curve_t: unexpected value");
}

static gnutls_ecc_curve_t unwrap_gnutls_ecc_curve_t(value v) {
  switch (Long_val(v)) {
    case 144245463: return GNUTLS_ECC_CURVE_INVALID;
    case 1026485428: return GNUTLS_ECC_CURVE_SECP224R1;
    case 1059853587: return GNUTLS_ECC_CURVE_SECP256R1;
    case -728971025: return GNUTLS_ECC_CURVE_SECP384R1;
    case -144678028: return GNUTLS_ECC_CURVE_SECP521R1;
    case 778523146: return GNUTLS_ECC_CURVE_SECP192R1;
    default: invalid_argument("unwrap_gnutls_ecc_curve_t");
  };
  failwith("unwrap_gnutls_ecc_curve_t: unexpected value");
}
#endif

/************** gnutls_sec_param_t *************/

#ifdef HAVE_TY_gnutls_sec_param_t
static value wrap_gnutls_sec_param_t(gnutls_sec_param_t x) {
  switch (x) {
    case GNUTLS_SEC_PARAM_UNKNOWN: return Val_long(785140586);
    case GNUTLS_SEC_PARAM_WEAK: return Val_long(969836696);
    case GNUTLS_SEC_PARAM_LOW: return Val_long(3804276);
    case GNUTLS_SEC_PARAM_NORMAL: return Val_long(-453122489);
    case GNUTLS_SEC_PARAM_HIGH: return Val_long(803693442);
    case GNUTLS_SEC_PARAM_ULTRA: return Val_long(952812300);
#ifdef HAVE_ENUM_GNUTLS_SEC_PARAM_INSECURE
    case GNUTLS_SEC_PARAM_INSECURE: return Val_long(300413500);
#endif
#ifdef HAVE_ENUM_GNUTLS_SEC_PARAM_EXPORT
    case GNUTLS_SEC_PARAM_EXPORT: return Val_long(-116259436);
#endif
#ifdef HAVE_ENUM_GNUTLS_SEC_PARAM_VERY_WEAK
    case GNUTLS_SEC_PARAM_VERY_WEAK: return Val_long(-708554815);
#endif
#ifdef HAVE_ENUM_GNUTLS_SEC_PARAM_LEGACY
    case GNUTLS_SEC_PARAM_LEGACY: return Val_long(-822677911);
#endif
    default: break;
  };
  failwith("wrap_gnutls_sec_param_t: unexpected value");
}

static gnutls_sec_param_t unwrap_gnutls_sec_param_t(value v) {
  switch (Long_val(v)) {
    case 785140586: return GNUTLS_SEC_PARAM_UNKNOWN;
    case 969836696: return GNUTLS_SEC_PARAM_WEAK;
    case 3804276: return GNUTLS_SEC_PARAM_LOW;
    case -453122489: return GNUTLS_SEC_PARAM_NORMAL;
    case 803693442: return GNUTLS_SEC_PARAM_HIGH;
    case 952812300: return GNUTLS_SEC_PARAM_ULTRA;
#ifdef HAVE_ENUM_GNUTLS_SEC_PARAM_INSECURE
    case 300413500: return GNUTLS_SEC_PARAM_INSECURE;
#endif
#ifdef HAVE_ENUM_GNUTLS_SEC_PARAM_EXPORT
    case -116259436: return GNUTLS_SEC_PARAM_EXPORT;
#endif
#ifdef HAVE_ENUM_GNUTLS_SEC_PARAM_VERY_WEAK
    case -708554815: return GNUTLS_SEC_PARAM_VERY_WEAK;
#endif
#ifdef HAVE_ENUM_GNUTLS_SEC_PARAM_LEGACY
    case -822677911: return GNUTLS_SEC_PARAM_LEGACY;
#endif
    default: invalid_argument("unwrap_gnutls_sec_param_t");
  };
  failwith("unwrap_gnutls_sec_param_t: unexpected value");
}
#endif

/************** gnutls_init_flags *************/

static value wrap_gnutls_init_flags(gnutls_init_flags x) {
  CAMLparam0();
  CAMLlocal2(v,u);
  v = Val_long(0);
  if (x & GNUTLS_SERVER) {
    u = caml_alloc(2,0);
    Field(u, 0) = Val_long(583778019);
    Field(u, 1) = v;
    v = u;
  };
  if (x & GNUTLS_CLIENT) {
    u = caml_alloc(2,0);
    Field(u, 0) = Val_long(1055145579);
    Field(u, 1) = v;
    v = u;
  };
#ifdef HAVE_ENUM_GNUTLS_DATAGRAM
  if (x & GNUTLS_DATAGRAM) {
    u = caml_alloc(2,0);
    Field(u, 0) = Val_long(-222806239);
    Field(u, 1) = v;
    v = u;
  };
#endif
#ifdef HAVE_ENUM_GNUTLS_NONBLOCK
  if (x & GNUTLS_NONBLOCK) {
    u = caml_alloc(2,0);
    Field(u, 0) = Val_long(627796288);
    Field(u, 1) = v;
    v = u;
  };
#endif
#ifdef HAVE_ENUM_GNUTLS_NO_EXTENSIONS
  if (x & GNUTLS_NO_EXTENSIONS) {
    u = caml_alloc(2,0);
    Field(u, 0) = Val_long(513871474);
    Field(u, 1) = v;
    v = u;
  };
#endif
#ifdef HAVE_ENUM_GNUTLS_NO_REPLAY_PROTECTION
  if (x & GNUTLS_NO_REPLAY_PROTECTION) {
    u = caml_alloc(2,0);
    Field(u, 0) = Val_long(613071571);
    Field(u, 1) = v;
    v = u;
  };
#endif
  CAMLreturn(v);
}

static gnutls_init_flags unwrap_gnutls_init_flags(value v) {
  gnutls_init_flags x = 0;
  while (Is_block(v)) {
    switch (Long_val(Field(v,0))) {
      case 583778019: x |= GNUTLS_SERVER; break;
      case 1055145579: x |= GNUTLS_CLIENT; break;
#ifdef HAVE_ENUM_GNUTLS_DATAGRAM
      case -222806239: x |= GNUTLS_DATAGRAM; break;
#endif
#ifdef HAVE_ENUM_GNUTLS_NONBLOCK
      case 627796288: x |= GNUTLS_NONBLOCK; break;
#endif
#ifdef HAVE_ENUM_GNUTLS_NO_EXTENSIONS
      case 513871474: x |= GNUTLS_NO_EXTENSIONS; break;
#endif
#ifdef HAVE_ENUM_GNUTLS_NO_REPLAY_PROTECTION
      case 613071571: x |= GNUTLS_NO_REPLAY_PROTECTION; break;
#endif
    };
    v = Field(v,1);
  };
  return x;
}

/************** gnutls_server_name_type_t *************/

static value wrap_gnutls_server_name_type_t(gnutls_server_name_type_t x) {
  switch (x) {
    case GNUTLS_NAME_DNS: return Val_long(3406217);
    default: break;
  };
  failwith("wrap_gnutls_server_name_type_t: unexpected value");
}

static gnutls_server_name_type_t unwrap_gnutls_server_name_type_t(value v) {
  switch (Long_val(v)) {
    case 3406217: return GNUTLS_NAME_DNS;
    default: invalid_argument("unwrap_gnutls_server_name_type_t");
  };
  failwith("unwrap_gnutls_server_name_type_t: unexpected value");
}

/************** gnutls_supplemental_data_format_type_t *************/

static value wrap_gnutls_supplemental_data_format_type_t(gnutls_supplemental_data_format_type_t x) {
  switch (x) {
#ifdef HAVE_ENUM_GNUTLS_SUPPLEMENTAL_UNKNOWN
    case GNUTLS_SUPPLEMENTAL_UNKNOWN: return Val_long(785140586);
#endif
    default: break;
  };
  failwith("wrap_gnutls_supplemental_data_format_type_t: unexpected value");
}

static gnutls_supplemental_data_format_type_t unwrap_gnutls_supplemental_data_format_type_t(value v) {
  switch (Long_val(v)) {
#ifdef HAVE_ENUM_GNUTLS_SUPPLEMENTAL_UNKNOWN
    case 785140586: return GNUTLS_SUPPLEMENTAL_UNKNOWN;
#endif
    default: invalid_argument("unwrap_gnutls_supplemental_data_format_type_t");
  };
  failwith("unwrap_gnutls_supplemental_data_format_type_t: unexpected value");
}

/************** gnutls_psk_key_flags *************/

static value wrap_gnutls_psk_key_flags(gnutls_psk_key_flags x) {
  switch (x) {
    case GNUTLS_PSK_KEY_RAW: return Val_long(4099528);
    case GNUTLS_PSK_KEY_HEX: return Val_long(3603131);
    default: break;
  };
  failwith("wrap_gnutls_psk_key_flags: unexpected value");
}

static gnutls_psk_key_flags unwrap_gnutls_psk_key_flags(value v) {
  switch (Long_val(v)) {
    case 4099528: return GNUTLS_PSK_KEY_RAW;
    case 3603131: return GNUTLS_PSK_KEY_HEX;
    default: invalid_argument("unwrap_gnutls_psk_key_flags");
  };
  failwith("unwrap_gnutls_psk_key_flags: unexpected value");
}

/************** gnutls_x509_subject_alt_name_t *************/

static value wrap_gnutls_x509_subject_alt_name_t(gnutls_x509_subject_alt_name_t x) {
  switch (x) {
    case GNUTLS_SAN_DNSNAME: return Val_long(118056020);
    case GNUTLS_SAN_RFC822NAME: return Val_long(-333523436);
    case GNUTLS_SAN_URI: return Val_long(4252492);
    case GNUTLS_SAN_IPADDRESS: return Val_long(-906735475);
    case GNUTLS_SAN_OTHERNAME: return Val_long(919363835);
    case GNUTLS_SAN_DN: return Val_long(15274);
    case GNUTLS_SAN_OTHERNAME_XMPP: return Val_long(-962527431);
    default: break;
  };
  failwith("wrap_gnutls_x509_subject_alt_name_t: unexpected value");
}

static gnutls_x509_subject_alt_name_t unwrap_gnutls_x509_subject_alt_name_t(value v) {
  switch (Long_val(v)) {
    case 118056020: return GNUTLS_SAN_DNSNAME;
    case -333523436: return GNUTLS_SAN_RFC822NAME;
    case 4252492: return GNUTLS_SAN_URI;
    case -906735475: return GNUTLS_SAN_IPADDRESS;
    case 919363835: return GNUTLS_SAN_OTHERNAME;
    case 15274: return GNUTLS_SAN_DN;
    case -962527431: return GNUTLS_SAN_OTHERNAME_XMPP;
    default: invalid_argument("unwrap_gnutls_x509_subject_alt_name_t");
  };
  failwith("unwrap_gnutls_x509_subject_alt_name_t: unexpected value");
}

/************** gnutls_privkey_type_t *************/

#ifdef HAVE_TY_gnutls_privkey_type_t
static value wrap_gnutls_privkey_type_t(gnutls_privkey_type_t x) {
  switch (x) {
    case GNUTLS_PRIVKEY_X509: return Val_long(978528294);
    case GNUTLS_PRIVKEY_OPENPGP: return Val_long(228291823);
    case GNUTLS_PRIVKEY_PKCS11: return Val_long(-634094293);
#ifdef HAVE_ENUM_GNUTLS_PRIVKEY_EXT
    case GNUTLS_PRIVKEY_EXT: return Val_long(3458177);
#endif
    default: break;
  };
  failwith("wrap_gnutls_privkey_type_t: unexpected value");
}

static gnutls_privkey_type_t unwrap_gnutls_privkey_type_t(value v) {
  switch (Long_val(v)) {
    case 978528294: return GNUTLS_PRIVKEY_X509;
    case 228291823: return GNUTLS_PRIVKEY_OPENPGP;
    case -634094293: return GNUTLS_PRIVKEY_PKCS11;
#ifdef HAVE_ENUM_GNUTLS_PRIVKEY_EXT
    case 3458177: return GNUTLS_PRIVKEY_EXT;
#endif
    default: invalid_argument("unwrap_gnutls_privkey_type_t");
  };
  failwith("unwrap_gnutls_privkey_type_t: unexpected value");
}
#endif

/************** key_usage *************/

static value wrap_key_usage(key_usage x) {
  CAMLparam0();
  CAMLlocal2(v,u);
  v = Val_long(0);
  if (x & GNUTLS_KEY_DIGITAL_SIGNATURE) {
    u = caml_alloc(2,0);
    Field(u, 0) = Val_long(427680881);
    Field(u, 1) = v;
    v = u;
  };
  if (x & GNUTLS_KEY_NON_REPUDIATION) {
    u = caml_alloc(2,0);
    Field(u, 0) = Val_long(996208134);
    Field(u, 1) = v;
    v = u;
  };
  if (x & GNUTLS_KEY_KEY_ENCIPHERMENT) {
    u = caml_alloc(2,0);
    Field(u, 0) = Val_long(-209529646);
    Field(u, 1) = v;
    v = u;
  };
  if (x & GNUTLS_KEY_DATA_ENCIPHERMENT) {
    u = caml_alloc(2,0);
    Field(u, 0) = Val_long(-722947801);
    Field(u, 1) = v;
    v = u;
  };
  if (x & GNUTLS_KEY_KEY_AGREEMENT) {
    u = caml_alloc(2,0);
    Field(u, 0) = Val_long(-337701430);
    Field(u, 1) = v;
    v = u;
  };
  if (x & GNUTLS_KEY_KEY_CERT_SIGN) {
    u = caml_alloc(2,0);
    Field(u, 0) = Val_long(776617144);
    Field(u, 1) = v;
    v = u;
  };
  if (x & GNUTLS_KEY_CRL_SIGN) {
    u = caml_alloc(2,0);
    Field(u, 0) = Val_long(716334143);
    Field(u, 1) = v;
    v = u;
  };
  if (x & GNUTLS_KEY_ENCIPHER_ONLY) {
    u = caml_alloc(2,0);
    Field(u, 0) = Val_long(-1053262473);
    Field(u, 1) = v;
    v = u;
  };
  if (x & GNUTLS_KEY_DECIPHER_ONLY) {
    u = caml_alloc(2,0);
    Field(u, 0) = Val_long(733816159);
    Field(u, 1) = v;
    v = u;
  };
  CAMLreturn(v);
}

static key_usage unwrap_key_usage(value v) {
  key_usage x = 0;
  while (Is_block(v)) {
    switch (Long_val(Field(v,0))) {
      case 427680881: x |= GNUTLS_KEY_DIGITAL_SIGNATURE; break;
      case 996208134: x |= GNUTLS_KEY_NON_REPUDIATION; break;
      case -209529646: x |= GNUTLS_KEY_KEY_ENCIPHERMENT; break;
      case -722947801: x |= GNUTLS_KEY_DATA_ENCIPHERMENT; break;
      case -337701430: x |= GNUTLS_KEY_KEY_AGREEMENT; break;
      case 776617144: x |= GNUTLS_KEY_KEY_CERT_SIGN; break;
      case 716334143: x |= GNUTLS_KEY_CRL_SIGN; break;
      case -1053262473: x |= GNUTLS_KEY_ENCIPHER_ONLY; break;
      case 733816159: x |= GNUTLS_KEY_DECIPHER_ONLY; break;
    };
    v = Field(v,1);
  };
  return x;
}

/************** gnutls_channel_binding_t *************/

#ifdef HAVE_TY_gnutls_channel_binding_t
static value wrap_gnutls_channel_binding_t(gnutls_channel_binding_t x) {
  switch (x) {
    case GNUTLS_CB_TLS_UNIQUE: return Val_long(435849781);
    default: break;
  };
  failwith("wrap_gnutls_channel_binding_t: unexpected value");
}

static gnutls_channel_binding_t unwrap_gnutls_channel_binding_t(value v) {
  switch (Long_val(v)) {
    case 435849781: return GNUTLS_CB_TLS_UNIQUE;
    default: invalid_argument("unwrap_gnutls_channel_binding_t");
  };
  failwith("unwrap_gnutls_channel_binding_t: unexpected value");
}
#endif

/************** gnutls_info_access_what_t *************/

#ifdef HAVE_TY_gnutls_info_access_what_t
static value wrap_gnutls_info_access_what_t(gnutls_info_access_what_t x) {
  switch (x) {
    case GNUTLS_IA_ACCESSMETHOD_OID: return Val_long(146849072);
    case GNUTLS_IA_ACCESSLOCATION_GENERALNAME_TYPE: return Val_long(159671948);
    case GNUTLS_IA_URI: return Val_long(4252492);
    case GNUTLS_IA_OCSP_URI: return Val_long(-227669058);
    case GNUTLS_IA_CAISSUERS_URI: return Val_long(-1057509559);
#ifdef HAVE_ENUM_GNUTLS_IA_UNKNOWN
    case GNUTLS_IA_UNKNOWN: return Val_long(785140586);
#endif
    default: break;
  };
  failwith("wrap_gnutls_info_access_what_t: unexpected value");
}

static gnutls_info_access_what_t unwrap_gnutls_info_access_what_t(value v) {
  switch (Long_val(v)) {
    case 146849072: return GNUTLS_IA_ACCESSMETHOD_OID;
    case 159671948: return GNUTLS_IA_ACCESSLOCATION_GENERALNAME_TYPE;
    case 4252492: return GNUTLS_IA_URI;
    case -227669058: return GNUTLS_IA_OCSP_URI;
    case -1057509559: return GNUTLS_IA_CAISSUERS_URI;
#ifdef HAVE_ENUM_GNUTLS_IA_UNKNOWN
    case 785140586: return GNUTLS_IA_UNKNOWN;
#endif
    default: invalid_argument("unwrap_gnutls_info_access_what_t");
  };
  failwith("unwrap_gnutls_info_access_what_t: unexpected value");
}
#endif

/************** gnutls_certificate_import_flags *************/

static value wrap_gnutls_certificate_import_flags(gnutls_certificate_import_flags x) {
  CAMLparam0();
  CAMLlocal2(v,u);
  v = Val_long(0);
#ifdef HAVE_ENUM_GNUTLS_X509_CRT_LIST_FAIL_IF_UNSORTED
  if (x & GNUTLS_X509_CRT_LIST_FAIL_IF_UNSORTED) {
    u = caml_alloc(2,0);
    Field(u, 0) = Val_long(164827959);
    Field(u, 1) = v;
    v = u;
  };
#endif
  if (x & GNUTLS_X509_CRT_LIST_IMPORT_FAIL_IF_EXCEED) {
    u = caml_alloc(2,0);
    Field(u, 0) = Val_long(-151294801);
    Field(u, 1) = v;
    v = u;
  };
#ifdef HAVE_ENUM_GNUTLS_X509_CRT_LIST_SORT
  if (x & GNUTLS_X509_CRT_LIST_SORT) {
    u = caml_alloc(2,0);
    Field(u, 0) = Val_long(925979518);
    Field(u, 1) = v;
    v = u;
  };
#endif
  CAMLreturn(v);
}

static gnutls_certificate_import_flags unwrap_gnutls_certificate_import_flags(value v) {
  gnutls_certificate_import_flags x = 0;
  while (Is_block(v)) {
    switch (Long_val(Field(v,0))) {
#ifdef HAVE_ENUM_GNUTLS_X509_CRT_LIST_FAIL_IF_UNSORTED
      case 164827959: x |= GNUTLS_X509_CRT_LIST_FAIL_IF_UNSORTED; break;
#endif
      case -151294801: x |= GNUTLS_X509_CRT_LIST_IMPORT_FAIL_IF_EXCEED; break;
#ifdef HAVE_ENUM_GNUTLS_X509_CRT_LIST_SORT
      case 925979518: x |= GNUTLS_X509_CRT_LIST_SORT; break;
#endif
    };
    v = Field(v,1);
  };
  return x;
}

/************** error_code *************/

static value wrap_error_code(error_code x) {
  switch (x) {
    case GNUTLS_E_SUCCESS: return Val_long(94326179);
    case GNUTLS_E_UNKNOWN_COMPRESSION_ALGORITHM: return Val_long(-109600959);
    case GNUTLS_E_UNKNOWN_CIPHER_TYPE: return Val_long(347761945);
    case GNUTLS_E_LARGE_PACKET: return Val_long(1012678380);
    case GNUTLS_E_UNSUPPORTED_VERSION_PACKET: return Val_long(-749147975);
    case GNUTLS_E_UNEXPECTED_PACKET_LENGTH: return Val_long(430833135);
    case GNUTLS_E_INVALID_SESSION: return Val_long(578462382);
    case GNUTLS_E_FATAL_ALERT_RECEIVED: return Val_long(-6604065);
    case GNUTLS_E_UNEXPECTED_PACKET: return Val_long(-374546602);
    case GNUTLS_E_WARNING_ALERT_RECEIVED: return Val_long(-616283257);
    case GNUTLS_E_ERROR_IN_FINISHED_PACKET: return Val_long(893432274);
    case GNUTLS_E_UNEXPECTED_HANDSHAKE_PACKET: return Val_long(-157336994);
    case GNUTLS_E_UNKNOWN_CIPHER_SUITE: return Val_long(-128690311);
    case GNUTLS_E_UNWANTED_ALGORITHM: return Val_long(-463916904);
    case GNUTLS_E_MPI_SCAN_FAILED: return Val_long(366141958);
    case GNUTLS_E_DECRYPTION_FAILED: return Val_long(220224769);
    case GNUTLS_E_MEMORY_ERROR: return Val_long(65200362);
    case GNUTLS_E_DECOMPRESSION_FAILED: return Val_long(-180539241);
    case GNUTLS_E_COMPRESSION_FAILED: return Val_long(-136667946);
    case GNUTLS_E_AGAIN: return Val_long(829072704);
    case GNUTLS_E_EXPIRED: return Val_long(-222591099);
    case GNUTLS_E_DB_ERROR: return Val_long(196008231);
    case GNUTLS_E_SRP_PWD_ERROR: return Val_long(790453528);
    case GNUTLS_E_INSUFFICIENT_CREDENTIALS: return Val_long(831495520);
    case GNUTLS_E_HASH_FAILED: return Val_long(-265554642);
    case GNUTLS_E_BASE64_DECODING_ERROR: return Val_long(1028708100);
    case GNUTLS_E_MPI_PRINT_FAILED: return Val_long(1031509608);
    case GNUTLS_E_REHANDSHAKE: return Val_long(-657094972);
    case GNUTLS_E_GOT_APPLICATION_DATA: return Val_long(-233873300);
    case GNUTLS_E_RECORD_LIMIT_REACHED: return Val_long(-971148032);
    case GNUTLS_E_ENCRYPTION_FAILED: return Val_long(281387289);
    case GNUTLS_E_PK_ENCRYPTION_FAILED: return Val_long(-566202667);
    case GNUTLS_E_PK_DECRYPTION_FAILED: return Val_long(-627365187);
    case GNUTLS_E_PK_SIGN_FAILED: return Val_long(-278812325);
    case GNUTLS_E_X509_UNSUPPORTED_CRITICAL_EXTENSION: return Val_long(192467554);
    case GNUTLS_E_KEY_USAGE_VIOLATION: return Val_long(985619399);
    case GNUTLS_E_NO_CERTIFICATE_FOUND: return Val_long(-967371972);
    case GNUTLS_E_INVALID_REQUEST: return Val_long(-748197209);
    case GNUTLS_E_SHORT_MEMORY_BUFFER: return Val_long(-248175781);
    case GNUTLS_E_INTERRUPTED: return Val_long(-952706910);
    case GNUTLS_E_PUSH_ERROR: return Val_long(84740227);
    case GNUTLS_E_PULL_ERROR: return Val_long(453100718);
    case GNUTLS_E_RECEIVED_ILLEGAL_PARAMETER: return Val_long(535414946);
    case GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE: return Val_long(338256857);
    case GNUTLS_E_PKCS1_WRONG_PAD: return Val_long(215790984);
    case GNUTLS_E_RECEIVED_ILLEGAL_EXTENSION: return Val_long(1011492472);
    case GNUTLS_E_INTERNAL_ERROR: return Val_long(163682854);
    case GNUTLS_E_DH_PRIME_UNACCEPTABLE: return Val_long(238900086);
    case GNUTLS_E_FILE_ERROR: return Val_long(296872613);
    case GNUTLS_E_TOO_MANY_EMPTY_PACKETS: return Val_long(560516164);
    case GNUTLS_E_UNKNOWN_PK_ALGORITHM: return Val_long(-660873440);
#ifdef HAVE_ENUM_GNUTLS_E_TOO_MANY_HANDSHAKE_PACKETS
    case GNUTLS_E_TOO_MANY_HANDSHAKE_PACKETS: return Val_long(346746414);
#endif
    case GNUTLS_E_NO_TEMPORARY_RSA_PARAMS: return Val_long(-100301199);
    case GNUTLS_E_NO_COMPRESSION_ALGORITHMS: return Val_long(-331677701);
    case GNUTLS_E_NO_CIPHER_SUITES: return Val_long(930434577);
    case GNUTLS_E_OPENPGP_GETKEY_FAILED: return Val_long(1038627107);
    case GNUTLS_E_PK_SIG_VERIFY_FAILED: return Val_long(-382952015);
    case GNUTLS_E_ILLEGAL_SRP_USERNAME: return Val_long(-789061587);
    case GNUTLS_E_SRP_PWD_PARSING_ERROR: return Val_long(67200137);
    case GNUTLS_E_NO_TEMPORARY_DH_PARAMS: return Val_long(230319189);
    case GNUTLS_E_ASN1_ELEMENT_NOT_FOUND: return Val_long(498814953);
    case GNUTLS_E_ASN1_IDENTIFIER_NOT_FOUND: return Val_long(-1036114326);
    case GNUTLS_E_ASN1_DER_ERROR: return Val_long(-188438224);
    case GNUTLS_E_ASN1_VALUE_NOT_FOUND: return Val_long(209240926);
    case GNUTLS_E_ASN1_GENERIC_ERROR: return Val_long(-15737610);
    case GNUTLS_E_ASN1_VALUE_NOT_VALID: return Val_long(966407704);
    case GNUTLS_E_ASN1_TAG_ERROR: return Val_long(-173713799);
    case GNUTLS_E_ASN1_TAG_IMPLICIT: return Val_long(270256916);
    case GNUTLS_E_ASN1_TYPE_ANY_ERROR: return Val_long(-149505894);
    case GNUTLS_E_ASN1_SYNTAX_ERROR: return Val_long(621083094);
    case GNUTLS_E_ASN1_DER_OVERFLOW: return Val_long(348608218);
    case GNUTLS_E_OPENPGP_UID_REVOKED: return Val_long(-847062561);
    case GNUTLS_E_CERTIFICATE_ERROR: return Val_long(537134912);
    case GNUTLS_E_CERTIFICATE_KEY_MISMATCH: return Val_long(-690325066);
    case GNUTLS_E_UNSUPPORTED_CERTIFICATE_TYPE: return Val_long(865436748);
    case GNUTLS_E_X509_UNKNOWN_SAN: return Val_long(885195986);
    case GNUTLS_E_OPENPGP_FINGERPRINT_UNSUPPORTED: return Val_long(311704074);
    case GNUTLS_E_X509_UNSUPPORTED_ATTRIBUTE: return Val_long(-219899207);
    case GNUTLS_E_UNKNOWN_HASH_ALGORITHM: return Val_long(240503283);
    case GNUTLS_E_UNKNOWN_PKCS_CONTENT_TYPE: return Val_long(-467868673);
    case GNUTLS_E_UNKNOWN_PKCS_BAG_TYPE: return Val_long(-164590192);
    case GNUTLS_E_INVALID_PASSWORD: return Val_long(-1071151933);
    case GNUTLS_E_MAC_VERIFY_FAILED: return Val_long(-57992205);
    case GNUTLS_E_CONSTRAINT_ERROR: return Val_long(539235142);
    case GNUTLS_E_WARNING_IA_IPHF_RECEIVED: return Val_long(-289406985);
    case GNUTLS_E_WARNING_IA_FPHF_RECEIVED: return Val_long(494831476);
    case GNUTLS_E_IA_VERIFY_FAILED: return Val_long(-1028008132);
    case GNUTLS_E_UNKNOWN_ALGORITHM: return Val_long(467770394);
#ifdef HAVE_ENUM_GNUTLS_E_UNSUPPORTED_SIGNATURE_ALGORITHM
    case GNUTLS_E_UNSUPPORTED_SIGNATURE_ALGORITHM: return Val_long(1012651326);
#endif
#ifdef HAVE_ENUM_GNUTLS_E_SAFE_RENEGOTIATION_FAILED
    case GNUTLS_E_SAFE_RENEGOTIATION_FAILED: return Val_long(-337380256);
#endif
#ifdef HAVE_ENUM_GNUTLS_E_UNSAFE_RENEGOTIATION_DENIED
    case GNUTLS_E_UNSAFE_RENEGOTIATION_DENIED: return Val_long(1065966725);
#endif
#ifdef HAVE_ENUM_GNUTLS_E_UNKNOWN_SRP_USERNAME
    case GNUTLS_E_UNKNOWN_SRP_USERNAME: return Val_long(1014226329);
#endif
#ifdef HAVE_ENUM_GNUTLS_E_PREMATURE_TERMINATION
    case GNUTLS_E_PREMATURE_TERMINATION: return Val_long(-1056552368);
#endif
    case GNUTLS_E_BASE64_ENCODING_ERROR: return Val_long(-205351956);
    case GNUTLS_E_INCOMPATIBLE_CRYPTO_LIBRARY: return Val_long(-510967837);
    case GNUTLS_E_INCOMPATIBLE_LIBTASN1_LIBRARY: return Val_long(-821519962);
    case GNUTLS_E_OPENPGP_KEYRING_ERROR: return Val_long(-979089784);
    case GNUTLS_E_X509_UNSUPPORTED_OID: return Val_long(-509559577);
    case GNUTLS_E_RANDOM_FAILED: return Val_long(-445211751);
    case GNUTLS_E_BASE64_UNEXPECTED_HEADER_ERROR: return Val_long(355640564);
    case GNUTLS_E_OPENPGP_SUBKEY_ERROR: return Val_long(-684279560);
    case GNUTLS_E_CRYPTO_ALREADY_REGISTERED: return Val_long(-530625177);
    case GNUTLS_E_HANDSHAKE_TOO_LARGE: return Val_long(-325363736);
#ifdef HAVE_ENUM_GNUTLS_E_CRYPTODEV_IOCTL_ERROR
    case GNUTLS_E_CRYPTODEV_IOCTL_ERROR: return Val_long(-46433197);
#endif
#ifdef HAVE_ENUM_GNUTLS_E_CRYPTODEV_DEVICE_ERROR
    case GNUTLS_E_CRYPTODEV_DEVICE_ERROR: return Val_long(250523498);
#endif
#ifdef HAVE_ENUM_GNUTLS_E_CHANNEL_BINDING_NOT_AVAILABLE
    case GNUTLS_E_CHANNEL_BINDING_NOT_AVAILABLE: return Val_long(26755783);
#endif
#ifdef HAVE_ENUM_GNUTLS_E_BAD_COOKIE
    case GNUTLS_E_BAD_COOKIE: return Val_long(257339198);
#endif
#ifdef HAVE_ENUM_GNUTLS_E_OPENPGP_PREFERRED_KEY_ERROR
    case GNUTLS_E_OPENPGP_PREFERRED_KEY_ERROR: return Val_long(289570714);
#endif
#ifdef HAVE_ENUM_GNUTLS_E_INCOMPAT_DSA_KEY_WITH_TLS_PROTOCOL
    case GNUTLS_E_INCOMPAT_DSA_KEY_WITH_TLS_PROTOCOL: return Val_long(-596384176);
#endif
#ifdef HAVE_ENUM_GNUTLS_E_HEARTBEAT_PONG_RECEIVED
    case GNUTLS_E_HEARTBEAT_PONG_RECEIVED: return Val_long(305681061);
#endif
#ifdef HAVE_ENUM_GNUTLS_E_HEARTBEAT_PING_RECEIVED
    case GNUTLS_E_HEARTBEAT_PING_RECEIVED: return Val_long(129745131);
#endif
#ifdef HAVE_ENUM_GNUTLS_E_PKCS11_ERROR
    case GNUTLS_E_PKCS11_ERROR: return Val_long(-547601804);
#endif
#ifdef HAVE_ENUM_GNUTLS_E_PKCS11_LOAD_ERROR
    case GNUTLS_E_PKCS11_LOAD_ERROR: return Val_long(-265574525);
#endif
#ifdef HAVE_ENUM_GNUTLS_E_PARSING_ERROR
    case GNUTLS_E_PARSING_ERROR: return Val_long(1024742169);
#endif
#ifdef HAVE_ENUM_GNUTLS_E_PKCS11_PIN_ERROR
    case GNUTLS_E_PKCS11_PIN_ERROR: return Val_long(-886434358);
#endif
#ifdef HAVE_ENUM_GNUTLS_E_PKCS11_SLOT_ERROR
    case GNUTLS_E_PKCS11_SLOT_ERROR: return Val_long(-210084517);
#endif
#ifdef HAVE_ENUM_GNUTLS_E_LOCKING_ERROR
    case GNUTLS_E_LOCKING_ERROR: return Val_long(791565312);
#endif
#ifdef HAVE_ENUM_GNUTLS_E_PKCS11_ATTRIBUTE_ERROR
    case GNUTLS_E_PKCS11_ATTRIBUTE_ERROR: return Val_long(669025745);
#endif
#ifdef HAVE_ENUM_GNUTLS_E_PKCS11_DEVICE_ERROR
    case GNUTLS_E_PKCS11_DEVICE_ERROR: return Val_long(804682483);
#endif
#ifdef HAVE_ENUM_GNUTLS_E_PKCS11_DATA_ERROR
    case GNUTLS_E_PKCS11_DATA_ERROR: return Val_long(648167047);
#endif
#ifdef HAVE_ENUM_GNUTLS_E_PKCS11_UNSUPPORTED_FEATURE_ERROR
    case GNUTLS_E_PKCS11_UNSUPPORTED_FEATURE_ERROR: return Val_long(-251431263);
#endif
#ifdef HAVE_ENUM_GNUTLS_E_PKCS11_KEY_ERROR
    case GNUTLS_E_PKCS11_KEY_ERROR: return Val_long(286612116);
#endif
#ifdef HAVE_ENUM_GNUTLS_E_PKCS11_PIN_EXPIRED
    case GNUTLS_E_PKCS11_PIN_EXPIRED: return Val_long(898805287);
#endif
#ifdef HAVE_ENUM_GNUTLS_E_PKCS11_PIN_LOCKED
    case GNUTLS_E_PKCS11_PIN_LOCKED: return Val_long(34045128);
#endif
#ifdef HAVE_ENUM_GNUTLS_E_PKCS11_SESSION_ERROR
    case GNUTLS_E_PKCS11_SESSION_ERROR: return Val_long(425021835);
#endif
#ifdef HAVE_ENUM_GNUTLS_E_PKCS11_SIGNATURE_ERROR
    case GNUTLS_E_PKCS11_SIGNATURE_ERROR: return Val_long(1044333997);
#endif
#ifdef HAVE_ENUM_GNUTLS_E_PKCS11_TOKEN_ERROR
    case GNUTLS_E_PKCS11_TOKEN_ERROR: return Val_long(-1056614802);
#endif
#ifdef HAVE_ENUM_GNUTLS_E_PKCS11_USER_ERROR
    case GNUTLS_E_PKCS11_USER_ERROR: return Val_long(696633704);
#endif
#ifdef HAVE_ENUM_GNUTLS_E_CRYPTO_INIT_FAILED
    case GNUTLS_E_CRYPTO_INIT_FAILED: return Val_long(288185134);
#endif
#ifdef HAVE_ENUM_GNUTLS_E_TIMEDOUT
    case GNUTLS_E_TIMEDOUT: return Val_long(-777516713);
#endif
#ifdef HAVE_ENUM_GNUTLS_E_USER_ERROR
    case GNUTLS_E_USER_ERROR: return Val_long(531821492);
#endif
#ifdef HAVE_ENUM_GNUTLS_E_ECC_NO_SUPPORTED_CURVES
    case GNUTLS_E_ECC_NO_SUPPORTED_CURVES: return Val_long(-254531399);
#endif
#ifdef HAVE_ENUM_GNUTLS_E_ECC_UNSUPPORTED_CURVE
    case GNUTLS_E_ECC_UNSUPPORTED_CURVE: return Val_long(304126603);
#endif
#ifdef HAVE_ENUM_GNUTLS_E_PKCS11_REQUESTED_OBJECT_NOT_AVAILBLE
    case GNUTLS_E_PKCS11_REQUESTED_OBJECT_NOT_AVAILBLE: return Val_long(-334447405);
#endif
#ifdef HAVE_ENUM_GNUTLS_E_CERTIFICATE_LIST_UNSORTED
    case GNUTLS_E_CERTIFICATE_LIST_UNSORTED: return Val_long(-5466865);
#endif
#ifdef HAVE_ENUM_GNUTLS_E_ILLEGAL_PARAMETER
    case GNUTLS_E_ILLEGAL_PARAMETER: return Val_long(75791936);
#endif
#ifdef HAVE_ENUM_GNUTLS_E_NO_PRIORITIES_WERE_SET
    case GNUTLS_E_NO_PRIORITIES_WERE_SET: return Val_long(285356195);
#endif
#ifdef HAVE_ENUM_GNUTLS_E_X509_UNSUPPORTED_EXTENSION
    case GNUTLS_E_X509_UNSUPPORTED_EXTENSION: return Val_long(-586891748);
#endif
#ifdef HAVE_ENUM_GNUTLS_E_SESSION_EOF
    case GNUTLS_E_SESSION_EOF: return Val_long(514874355);
#endif
#ifdef HAVE_ENUM_GNUTLS_E_TPM_ERROR
    case GNUTLS_E_TPM_ERROR: return Val_long(-520288998);
#endif
#ifdef HAVE_ENUM_GNUTLS_E_TPM_KEY_PASSWORD_ERROR
    case GNUTLS_E_TPM_KEY_PASSWORD_ERROR: return Val_long(-329432654);
#endif
#ifdef HAVE_ENUM_GNUTLS_E_TPM_SRK_PASSWORD_ERROR
    case GNUTLS_E_TPM_SRK_PASSWORD_ERROR: return Val_long(-1053023483);
#endif
#ifdef HAVE_ENUM_GNUTLS_E_TPM_SESSION_ERROR
    case GNUTLS_E_TPM_SESSION_ERROR: return Val_long(-183358927);
#endif
#ifdef HAVE_ENUM_GNUTLS_E_TPM_KEY_NOT_FOUND
    case GNUTLS_E_TPM_KEY_NOT_FOUND: return Val_long(-266833848);
#endif
#ifdef HAVE_ENUM_GNUTLS_E_TPM_UNINITIALIZED
    case GNUTLS_E_TPM_UNINITIALIZED: return Val_long(584205997);
#endif
#ifdef HAVE_ENUM_GNUTLS_E_NO_CERTIFICATE_STATUS
    case GNUTLS_E_NO_CERTIFICATE_STATUS: return Val_long(-867520264);
#endif
#ifdef HAVE_ENUM_GNUTLS_E_OCSP_RESPONSE_ERROR
    case GNUTLS_E_OCSP_RESPONSE_ERROR: return Val_long(449664312);
#endif
#ifdef HAVE_ENUM_GNUTLS_E_RANDOM_DEVICE_ERROR
    case GNUTLS_E_RANDOM_DEVICE_ERROR: return Val_long(744564475);
#endif
#ifdef HAVE_ENUM_GNUTLS_E_AUTH_ERROR
    case GNUTLS_E_AUTH_ERROR: return Val_long(-102599535);
#endif
#ifdef HAVE_ENUM_GNUTLS_E_NO_APPLICATION_PROTOCOL
    case GNUTLS_E_NO_APPLICATION_PROTOCOL: return Val_long(-682021819);
#endif
#ifdef HAVE_ENUM_GNUTLS_E_SOCKETS_INIT_ERROR
    case GNUTLS_E_SOCKETS_INIT_ERROR: return Val_long(248444408);
#endif
#ifdef HAVE_ENUM_GNUTLS_E_KEY_IMPORT_FAILED
    case GNUTLS_E_KEY_IMPORT_FAILED: return Val_long(697274007);
#endif
#ifdef HAVE_ENUM_GNUTLS_E_SELF_TEST_ERROR
    case GNUTLS_E_SELF_TEST_ERROR: return Val_long(-513015346);
#endif
#ifdef HAVE_ENUM_GNUTLS_E_NO_SELF_TEST
    case GNUTLS_E_NO_SELF_TEST: return Val_long(-348168057);
#endif
#ifdef HAVE_ENUM_GNUTLS_E_LIB_IN_ERROR_STATE
    case GNUTLS_E_LIB_IN_ERROR_STATE: return Val_long(656306074);
#endif
#ifdef HAVE_ENUM_GNUTLS_E_PK_GENERATION_ERROR
    case GNUTLS_E_PK_GENERATION_ERROR: return Val_long(-807833723);
#endif
#ifdef HAVE_ENUM_GNUTLS_E_IDNA_ERROR
    case GNUTLS_E_IDNA_ERROR: return Val_long(-1487241);
#endif
#ifdef HAVE_ENUM_GNUTLS_E_NEED_FALLBACK
    case GNUTLS_E_NEED_FALLBACK: return Val_long(406292555);
#endif
    case GNUTLS_E_UNIMPLEMENTED_FEATURE: return Val_long(604003360);
    default: break;
  };
  failwith("wrap_error_code: unexpected value");
}

static error_code unwrap_error_code(value v) {
  switch (Long_val(v)) {
    case 94326179: return GNUTLS_E_SUCCESS;
    case -109600959: return GNUTLS_E_UNKNOWN_COMPRESSION_ALGORITHM;
    case 347761945: return GNUTLS_E_UNKNOWN_CIPHER_TYPE;
    case 1012678380: return GNUTLS_E_LARGE_PACKET;
    case -749147975: return GNUTLS_E_UNSUPPORTED_VERSION_PACKET;
    case 430833135: return GNUTLS_E_UNEXPECTED_PACKET_LENGTH;
    case 578462382: return GNUTLS_E_INVALID_SESSION;
    case -6604065: return GNUTLS_E_FATAL_ALERT_RECEIVED;
    case -374546602: return GNUTLS_E_UNEXPECTED_PACKET;
    case -616283257: return GNUTLS_E_WARNING_ALERT_RECEIVED;
    case 893432274: return GNUTLS_E_ERROR_IN_FINISHED_PACKET;
    case -157336994: return GNUTLS_E_UNEXPECTED_HANDSHAKE_PACKET;
    case -128690311: return GNUTLS_E_UNKNOWN_CIPHER_SUITE;
    case -463916904: return GNUTLS_E_UNWANTED_ALGORITHM;
    case 366141958: return GNUTLS_E_MPI_SCAN_FAILED;
    case 220224769: return GNUTLS_E_DECRYPTION_FAILED;
    case 65200362: return GNUTLS_E_MEMORY_ERROR;
    case -180539241: return GNUTLS_E_DECOMPRESSION_FAILED;
    case -136667946: return GNUTLS_E_COMPRESSION_FAILED;
    case 829072704: return GNUTLS_E_AGAIN;
    case -222591099: return GNUTLS_E_EXPIRED;
    case 196008231: return GNUTLS_E_DB_ERROR;
    case 790453528: return GNUTLS_E_SRP_PWD_ERROR;
    case 831495520: return GNUTLS_E_INSUFFICIENT_CREDENTIALS;
    case -265554642: return GNUTLS_E_HASH_FAILED;
    case 1028708100: return GNUTLS_E_BASE64_DECODING_ERROR;
    case 1031509608: return GNUTLS_E_MPI_PRINT_FAILED;
    case -657094972: return GNUTLS_E_REHANDSHAKE;
    case -233873300: return GNUTLS_E_GOT_APPLICATION_DATA;
    case -971148032: return GNUTLS_E_RECORD_LIMIT_REACHED;
    case 281387289: return GNUTLS_E_ENCRYPTION_FAILED;
    case -566202667: return GNUTLS_E_PK_ENCRYPTION_FAILED;
    case -627365187: return GNUTLS_E_PK_DECRYPTION_FAILED;
    case -278812325: return GNUTLS_E_PK_SIGN_FAILED;
    case 192467554: return GNUTLS_E_X509_UNSUPPORTED_CRITICAL_EXTENSION;
    case 985619399: return GNUTLS_E_KEY_USAGE_VIOLATION;
    case -967371972: return GNUTLS_E_NO_CERTIFICATE_FOUND;
    case -748197209: return GNUTLS_E_INVALID_REQUEST;
    case -248175781: return GNUTLS_E_SHORT_MEMORY_BUFFER;
    case -952706910: return GNUTLS_E_INTERRUPTED;
    case 84740227: return GNUTLS_E_PUSH_ERROR;
    case 453100718: return GNUTLS_E_PULL_ERROR;
    case 535414946: return GNUTLS_E_RECEIVED_ILLEGAL_PARAMETER;
    case 338256857: return GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE;
    case 215790984: return GNUTLS_E_PKCS1_WRONG_PAD;
    case 1011492472: return GNUTLS_E_RECEIVED_ILLEGAL_EXTENSION;
    case 163682854: return GNUTLS_E_INTERNAL_ERROR;
    case 238900086: return GNUTLS_E_DH_PRIME_UNACCEPTABLE;
    case 296872613: return GNUTLS_E_FILE_ERROR;
    case 560516164: return GNUTLS_E_TOO_MANY_EMPTY_PACKETS;
    case -660873440: return GNUTLS_E_UNKNOWN_PK_ALGORITHM;
#ifdef HAVE_ENUM_GNUTLS_E_TOO_MANY_HANDSHAKE_PACKETS
    case 346746414: return GNUTLS_E_TOO_MANY_HANDSHAKE_PACKETS;
#endif
    case -100301199: return GNUTLS_E_NO_TEMPORARY_RSA_PARAMS;
    case -331677701: return GNUTLS_E_NO_COMPRESSION_ALGORITHMS;
    case 930434577: return GNUTLS_E_NO_CIPHER_SUITES;
    case 1038627107: return GNUTLS_E_OPENPGP_GETKEY_FAILED;
    case -382952015: return GNUTLS_E_PK_SIG_VERIFY_FAILED;
    case -789061587: return GNUTLS_E_ILLEGAL_SRP_USERNAME;
    case 67200137: return GNUTLS_E_SRP_PWD_PARSING_ERROR;
    case 230319189: return GNUTLS_E_NO_TEMPORARY_DH_PARAMS;
    case 498814953: return GNUTLS_E_ASN1_ELEMENT_NOT_FOUND;
    case -1036114326: return GNUTLS_E_ASN1_IDENTIFIER_NOT_FOUND;
    case -188438224: return GNUTLS_E_ASN1_DER_ERROR;
    case 209240926: return GNUTLS_E_ASN1_VALUE_NOT_FOUND;
    case -15737610: return GNUTLS_E_ASN1_GENERIC_ERROR;
    case 966407704: return GNUTLS_E_ASN1_VALUE_NOT_VALID;
    case -173713799: return GNUTLS_E_ASN1_TAG_ERROR;
    case 270256916: return GNUTLS_E_ASN1_TAG_IMPLICIT;
    case -149505894: return GNUTLS_E_ASN1_TYPE_ANY_ERROR;
    case 621083094: return GNUTLS_E_ASN1_SYNTAX_ERROR;
    case 348608218: return GNUTLS_E_ASN1_DER_OVERFLOW;
    case -847062561: return GNUTLS_E_OPENPGP_UID_REVOKED;
    case 537134912: return GNUTLS_E_CERTIFICATE_ERROR;
    case -690325066: return GNUTLS_E_CERTIFICATE_KEY_MISMATCH;
    case 865436748: return GNUTLS_E_UNSUPPORTED_CERTIFICATE_TYPE;
    case 885195986: return GNUTLS_E_X509_UNKNOWN_SAN;
    case 311704074: return GNUTLS_E_OPENPGP_FINGERPRINT_UNSUPPORTED;
    case -219899207: return GNUTLS_E_X509_UNSUPPORTED_ATTRIBUTE;
    case 240503283: return GNUTLS_E_UNKNOWN_HASH_ALGORITHM;
    case -467868673: return GNUTLS_E_UNKNOWN_PKCS_CONTENT_TYPE;
    case -164590192: return GNUTLS_E_UNKNOWN_PKCS_BAG_TYPE;
    case -1071151933: return GNUTLS_E_INVALID_PASSWORD;
    case -57992205: return GNUTLS_E_MAC_VERIFY_FAILED;
    case 539235142: return GNUTLS_E_CONSTRAINT_ERROR;
    case -289406985: return GNUTLS_E_WARNING_IA_IPHF_RECEIVED;
    case 494831476: return GNUTLS_E_WARNING_IA_FPHF_RECEIVED;
    case -1028008132: return GNUTLS_E_IA_VERIFY_FAILED;
    case 467770394: return GNUTLS_E_UNKNOWN_ALGORITHM;
#ifdef HAVE_ENUM_GNUTLS_E_UNSUPPORTED_SIGNATURE_ALGORITHM
    case 1012651326: return GNUTLS_E_UNSUPPORTED_SIGNATURE_ALGORITHM;
#endif
#ifdef HAVE_ENUM_GNUTLS_E_SAFE_RENEGOTIATION_FAILED
    case -337380256: return GNUTLS_E_SAFE_RENEGOTIATION_FAILED;
#endif
#ifdef HAVE_ENUM_GNUTLS_E_UNSAFE_RENEGOTIATION_DENIED
    case 1065966725: return GNUTLS_E_UNSAFE_RENEGOTIATION_DENIED;
#endif
#ifdef HAVE_ENUM_GNUTLS_E_UNKNOWN_SRP_USERNAME
    case 1014226329: return GNUTLS_E_UNKNOWN_SRP_USERNAME;
#endif
#ifdef HAVE_ENUM_GNUTLS_E_PREMATURE_TERMINATION
    case -1056552368: return GNUTLS_E_PREMATURE_TERMINATION;
#endif
    case -205351956: return GNUTLS_E_BASE64_ENCODING_ERROR;
    case -510967837: return GNUTLS_E_INCOMPATIBLE_CRYPTO_LIBRARY;
    case -821519962: return GNUTLS_E_INCOMPATIBLE_LIBTASN1_LIBRARY;
    case -979089784: return GNUTLS_E_OPENPGP_KEYRING_ERROR;
    case -509559577: return GNUTLS_E_X509_UNSUPPORTED_OID;
    case -445211751: return GNUTLS_E_RANDOM_FAILED;
    case 355640564: return GNUTLS_E_BASE64_UNEXPECTED_HEADER_ERROR;
    case -684279560: return GNUTLS_E_OPENPGP_SUBKEY_ERROR;
    case -530625177: return GNUTLS_E_CRYPTO_ALREADY_REGISTERED;
    case -325363736: return GNUTLS_E_HANDSHAKE_TOO_LARGE;
#ifdef HAVE_ENUM_GNUTLS_E_CRYPTODEV_IOCTL_ERROR
    case -46433197: return GNUTLS_E_CRYPTODEV_IOCTL_ERROR;
#endif
#ifdef HAVE_ENUM_GNUTLS_E_CRYPTODEV_DEVICE_ERROR
    case 250523498: return GNUTLS_E_CRYPTODEV_DEVICE_ERROR;
#endif
#ifdef HAVE_ENUM_GNUTLS_E_CHANNEL_BINDING_NOT_AVAILABLE
    case 26755783: return GNUTLS_E_CHANNEL_BINDING_NOT_AVAILABLE;
#endif
#ifdef HAVE_ENUM_GNUTLS_E_BAD_COOKIE
    case 257339198: return GNUTLS_E_BAD_COOKIE;
#endif
#ifdef HAVE_ENUM_GNUTLS_E_OPENPGP_PREFERRED_KEY_ERROR
    case 289570714: return GNUTLS_E_OPENPGP_PREFERRED_KEY_ERROR;
#endif
#ifdef HAVE_ENUM_GNUTLS_E_INCOMPAT_DSA_KEY_WITH_TLS_PROTOCOL
    case -596384176: return GNUTLS_E_INCOMPAT_DSA_KEY_WITH_TLS_PROTOCOL;
#endif
#ifdef HAVE_ENUM_GNUTLS_E_HEARTBEAT_PONG_RECEIVED
    case 305681061: return GNUTLS_E_HEARTBEAT_PONG_RECEIVED;
#endif
#ifdef HAVE_ENUM_GNUTLS_E_HEARTBEAT_PING_RECEIVED
    case 129745131: return GNUTLS_E_HEARTBEAT_PING_RECEIVED;
#endif
#ifdef HAVE_ENUM_GNUTLS_E_PKCS11_ERROR
    case -547601804: return GNUTLS_E_PKCS11_ERROR;
#endif
#ifdef HAVE_ENUM_GNUTLS_E_PKCS11_LOAD_ERROR
    case -265574525: return GNUTLS_E_PKCS11_LOAD_ERROR;
#endif
#ifdef HAVE_ENUM_GNUTLS_E_PARSING_ERROR
    case 1024742169: return GNUTLS_E_PARSING_ERROR;
#endif
#ifdef HAVE_ENUM_GNUTLS_E_PKCS11_PIN_ERROR
    case -886434358: return GNUTLS_E_PKCS11_PIN_ERROR;
#endif
#ifdef HAVE_ENUM_GNUTLS_E_PKCS11_SLOT_ERROR
    case -210084517: return GNUTLS_E_PKCS11_SLOT_ERROR;
#endif
#ifdef HAVE_ENUM_GNUTLS_E_LOCKING_ERROR
    case 791565312: return GNUTLS_E_LOCKING_ERROR;
#endif
#ifdef HAVE_ENUM_GNUTLS_E_PKCS11_ATTRIBUTE_ERROR
    case 669025745: return GNUTLS_E_PKCS11_ATTRIBUTE_ERROR;
#endif
#ifdef HAVE_ENUM_GNUTLS_E_PKCS11_DEVICE_ERROR
    case 804682483: return GNUTLS_E_PKCS11_DEVICE_ERROR;
#endif
#ifdef HAVE_ENUM_GNUTLS_E_PKCS11_DATA_ERROR
    case 648167047: return GNUTLS_E_PKCS11_DATA_ERROR;
#endif
#ifdef HAVE_ENUM_GNUTLS_E_PKCS11_UNSUPPORTED_FEATURE_ERROR
    case -251431263: return GNUTLS_E_PKCS11_UNSUPPORTED_FEATURE_ERROR;
#endif
#ifdef HAVE_ENUM_GNUTLS_E_PKCS11_KEY_ERROR
    case 286612116: return GNUTLS_E_PKCS11_KEY_ERROR;
#endif
#ifdef HAVE_ENUM_GNUTLS_E_PKCS11_PIN_EXPIRED
    case 898805287: return GNUTLS_E_PKCS11_PIN_EXPIRED;
#endif
#ifdef HAVE_ENUM_GNUTLS_E_PKCS11_PIN_LOCKED
    case 34045128: return GNUTLS_E_PKCS11_PIN_LOCKED;
#endif
#ifdef HAVE_ENUM_GNUTLS_E_PKCS11_SESSION_ERROR
    case 425021835: return GNUTLS_E_PKCS11_SESSION_ERROR;
#endif
#ifdef HAVE_ENUM_GNUTLS_E_PKCS11_SIGNATURE_ERROR
    case 1044333997: return GNUTLS_E_PKCS11_SIGNATURE_ERROR;
#endif
#ifdef HAVE_ENUM_GNUTLS_E_PKCS11_TOKEN_ERROR
    case -1056614802: return GNUTLS_E_PKCS11_TOKEN_ERROR;
#endif
#ifdef HAVE_ENUM_GNUTLS_E_PKCS11_USER_ERROR
    case 696633704: return GNUTLS_E_PKCS11_USER_ERROR;
#endif
#ifdef HAVE_ENUM_GNUTLS_E_CRYPTO_INIT_FAILED
    case 288185134: return GNUTLS_E_CRYPTO_INIT_FAILED;
#endif
#ifdef HAVE_ENUM_GNUTLS_E_TIMEDOUT
    case -777516713: return GNUTLS_E_TIMEDOUT;
#endif
#ifdef HAVE_ENUM_GNUTLS_E_USER_ERROR
    case 531821492: return GNUTLS_E_USER_ERROR;
#endif
#ifdef HAVE_ENUM_GNUTLS_E_ECC_NO_SUPPORTED_CURVES
    case -254531399: return GNUTLS_E_ECC_NO_SUPPORTED_CURVES;
#endif
#ifdef HAVE_ENUM_GNUTLS_E_ECC_UNSUPPORTED_CURVE
    case 304126603: return GNUTLS_E_ECC_UNSUPPORTED_CURVE;
#endif
#ifdef HAVE_ENUM_GNUTLS_E_PKCS11_REQUESTED_OBJECT_NOT_AVAILBLE
    case -334447405: return GNUTLS_E_PKCS11_REQUESTED_OBJECT_NOT_AVAILBLE;
#endif
#ifdef HAVE_ENUM_GNUTLS_E_CERTIFICATE_LIST_UNSORTED
    case -5466865: return GNUTLS_E_CERTIFICATE_LIST_UNSORTED;
#endif
#ifdef HAVE_ENUM_GNUTLS_E_ILLEGAL_PARAMETER
    case 75791936: return GNUTLS_E_ILLEGAL_PARAMETER;
#endif
#ifdef HAVE_ENUM_GNUTLS_E_NO_PRIORITIES_WERE_SET
    case 285356195: return GNUTLS_E_NO_PRIORITIES_WERE_SET;
#endif
#ifdef HAVE_ENUM_GNUTLS_E_X509_UNSUPPORTED_EXTENSION
    case -586891748: return GNUTLS_E_X509_UNSUPPORTED_EXTENSION;
#endif
#ifdef HAVE_ENUM_GNUTLS_E_SESSION_EOF
    case 514874355: return GNUTLS_E_SESSION_EOF;
#endif
#ifdef HAVE_ENUM_GNUTLS_E_TPM_ERROR
    case -520288998: return GNUTLS_E_TPM_ERROR;
#endif
#ifdef HAVE_ENUM_GNUTLS_E_TPM_KEY_PASSWORD_ERROR
    case -329432654: return GNUTLS_E_TPM_KEY_PASSWORD_ERROR;
#endif
#ifdef HAVE_ENUM_GNUTLS_E_TPM_SRK_PASSWORD_ERROR
    case -1053023483: return GNUTLS_E_TPM_SRK_PASSWORD_ERROR;
#endif
#ifdef HAVE_ENUM_GNUTLS_E_TPM_SESSION_ERROR
    case -183358927: return GNUTLS_E_TPM_SESSION_ERROR;
#endif
#ifdef HAVE_ENUM_GNUTLS_E_TPM_KEY_NOT_FOUND
    case -266833848: return GNUTLS_E_TPM_KEY_NOT_FOUND;
#endif
#ifdef HAVE_ENUM_GNUTLS_E_TPM_UNINITIALIZED
    case 584205997: return GNUTLS_E_TPM_UNINITIALIZED;
#endif
#ifdef HAVE_ENUM_GNUTLS_E_NO_CERTIFICATE_STATUS
    case -867520264: return GNUTLS_E_NO_CERTIFICATE_STATUS;
#endif
#ifdef HAVE_ENUM_GNUTLS_E_OCSP_RESPONSE_ERROR
    case 449664312: return GNUTLS_E_OCSP_RESPONSE_ERROR;
#endif
#ifdef HAVE_ENUM_GNUTLS_E_RANDOM_DEVICE_ERROR
    case 744564475: return GNUTLS_E_RANDOM_DEVICE_ERROR;
#endif
#ifdef HAVE_ENUM_GNUTLS_E_AUTH_ERROR
    case -102599535: return GNUTLS_E_AUTH_ERROR;
#endif
#ifdef HAVE_ENUM_GNUTLS_E_NO_APPLICATION_PROTOCOL
    case -682021819: return GNUTLS_E_NO_APPLICATION_PROTOCOL;
#endif
#ifdef HAVE_ENUM_GNUTLS_E_SOCKETS_INIT_ERROR
    case 248444408: return GNUTLS_E_SOCKETS_INIT_ERROR;
#endif
#ifdef HAVE_ENUM_GNUTLS_E_KEY_IMPORT_FAILED
    case 697274007: return GNUTLS_E_KEY_IMPORT_FAILED;
#endif
#ifdef HAVE_ENUM_GNUTLS_E_SELF_TEST_ERROR
    case -513015346: return GNUTLS_E_SELF_TEST_ERROR;
#endif
#ifdef HAVE_ENUM_GNUTLS_E_NO_SELF_TEST
    case -348168057: return GNUTLS_E_NO_SELF_TEST;
#endif
#ifdef HAVE_ENUM_GNUTLS_E_LIB_IN_ERROR_STATE
    case 656306074: return GNUTLS_E_LIB_IN_ERROR_STATE;
#endif
#ifdef HAVE_ENUM_GNUTLS_E_PK_GENERATION_ERROR
    case -807833723: return GNUTLS_E_PK_GENERATION_ERROR;
#endif
#ifdef HAVE_ENUM_GNUTLS_E_IDNA_ERROR
    case -1487241: return GNUTLS_E_IDNA_ERROR;
#endif
#ifdef HAVE_ENUM_GNUTLS_E_NEED_FALLBACK
    case 406292555: return GNUTLS_E_NEED_FALLBACK;
#endif
    case 604003360: return GNUTLS_E_UNIMPLEMENTED_FEATURE;
    default: invalid_argument("unwrap_error_code");
  };
  failwith("unwrap_error_code: unexpected value");
}

/************** gnutls_pkcs_encrypt_flags_t *************/

static value wrap_gnutls_pkcs_encrypt_flags_t(gnutls_pkcs_encrypt_flags_t x) {
  CAMLparam0();
  CAMLlocal2(v,u);
  v = Val_long(0);
  if (x & GNUTLS_PKCS_PLAIN) {
    u = caml_alloc(2,0);
    Field(u, 0) = Val_long(-675583510);
    Field(u, 1) = v;
    v = u;
  };
  if (x & GNUTLS_PKCS_USE_PKCS12_3DES) {
    u = caml_alloc(2,0);
    Field(u, 0) = Val_long(610177914);
    Field(u, 1) = v;
    v = u;
  };
  if (x & GNUTLS_PKCS_USE_PKCS12_ARCFOUR) {
    u = caml_alloc(2,0);
    Field(u, 0) = Val_long(-936547811);
    Field(u, 1) = v;
    v = u;
  };
  if (x & GNUTLS_PKCS_USE_PKCS12_RC2_40) {
    u = caml_alloc(2,0);
    Field(u, 0) = Val_long(-485086315);
    Field(u, 1) = v;
    v = u;
  };
  if (x & GNUTLS_PKCS_USE_PBES2_3DES) {
    u = caml_alloc(2,0);
    Field(u, 0) = Val_long(789209380);
    Field(u, 1) = v;
    v = u;
  };
#ifdef HAVE_ENUM_GNUTLS_PKCS_USE_PBES2_DES
  if (x & GNUTLS_PKCS_USE_PBES2_DES) {
    u = caml_alloc(2,0);
    Field(u, 0) = Val_long(5976013);
    Field(u, 1) = v;
    v = u;
  };
#endif
#ifdef HAVE_ENUM_GNUTLS_PKCS_USE_PBES2_AES_128
  if (x & GNUTLS_PKCS_USE_PBES2_AES_128) {
    u = caml_alloc(2,0);
    Field(u, 0) = Val_long(-361507518);
    Field(u, 1) = v;
    v = u;
  };
#endif
#ifdef HAVE_ENUM_GNUTLS_PKCS_USE_PBES2_AES_192
  if (x & GNUTLS_PKCS_USE_PBES2_AES_192) {
    u = caml_alloc(2,0);
    Field(u, 0) = Val_long(-361505963);
    Field(u, 1) = v;
    v = u;
  };
#endif
#ifdef HAVE_ENUM_GNUTLS_PKCS_USE_PBES2_AES_256
  if (x & GNUTLS_PKCS_USE_PBES2_AES_256) {
    u = caml_alloc(2,0);
    Field(u, 0) = Val_long(-361457122);
    Field(u, 1) = v;
    v = u;
  };
#endif
#ifdef HAVE_ENUM_GNUTLS_PKCS_NULL_PASSWORD
  if (x & GNUTLS_PKCS_NULL_PASSWORD) {
    u = caml_alloc(2,0);
    Field(u, 0) = Val_long(-624344333);
    Field(u, 1) = v;
    v = u;
  };
#endif
  CAMLreturn(v);
}

static gnutls_pkcs_encrypt_flags_t unwrap_gnutls_pkcs_encrypt_flags_t(value v) {
  gnutls_pkcs_encrypt_flags_t x = 0;
  while (Is_block(v)) {
    switch (Long_val(Field(v,0))) {
      case -675583510: x |= GNUTLS_PKCS_PLAIN; break;
      case 610177914: x |= GNUTLS_PKCS_USE_PKCS12_3DES; break;
      case -936547811: x |= GNUTLS_PKCS_USE_PKCS12_ARCFOUR; break;
      case -485086315: x |= GNUTLS_PKCS_USE_PKCS12_RC2_40; break;
      case 789209380: x |= GNUTLS_PKCS_USE_PBES2_3DES; break;
#ifdef HAVE_ENUM_GNUTLS_PKCS_USE_PBES2_DES
      case 5976013: x |= GNUTLS_PKCS_USE_PBES2_DES; break;
#endif
#ifdef HAVE_ENUM_GNUTLS_PKCS_USE_PBES2_AES_128
      case -361507518: x |= GNUTLS_PKCS_USE_PBES2_AES_128; break;
#endif
#ifdef HAVE_ENUM_GNUTLS_PKCS_USE_PBES2_AES_192
      case -361505963: x |= GNUTLS_PKCS_USE_PBES2_AES_192; break;
#endif
#ifdef HAVE_ENUM_GNUTLS_PKCS_USE_PBES2_AES_256
      case -361457122: x |= GNUTLS_PKCS_USE_PBES2_AES_256; break;
#endif
#ifdef HAVE_ENUM_GNUTLS_PKCS_NULL_PASSWORD
      case -624344333: x |= GNUTLS_PKCS_NULL_PASSWORD; break;
#endif
    };
    v = Field(v,1);
  };
  return x;
}

/************** empty_flags *************/

static value wrap_empty_flags(empty_flags x) {
  CAMLparam0();
  CAMLlocal2(v,u);
  v = Val_long(0);
  if (x & DUMMY) {
    u = caml_alloc(2,0);
    Field(u, 0) = Val_long(-186089976);
    Field(u, 1) = v;
    v = u;
  };
  CAMLreturn(v);
}

static empty_flags unwrap_empty_flags(value v) {
  empty_flags x = 0;
  while (Is_block(v)) {
    switch (Long_val(Field(v,0))) {
      case -186089976: x |= DUMMY; break;
    };
    v = Field(v,1);
  };
  return x;
}

value net_nettls_init(value dummy) {
  CAMLparam1(dummy);
  nettls_init();
  nettls_init();
  CAMLreturn(Val_unit);
}

value net_nettls_deinit(value dummy) {
  CAMLparam1(dummy);
  nettls_init();
  nettls_deinit();
  CAMLreturn(Val_unit);
}

value net_gnutls_error_is_fatal(value error) {
  error_code error__c;
  int result__c;
  CAMLparam1(error);
  CAMLlocal1(result);
  error__c = unwrap_error_code(error);
  nettls_init();
  result__c = gnutls_error_is_fatal(error__c);
  result = Val_bool(result__c);
  CAMLreturn(result);
}

value net_gnutls_strerror(value error) {
  error_code error__c;
  const char * result__c;
  CAMLparam1(error);
  CAMLlocal1(result);
  error__c = unwrap_error_code(error);
  nettls_init();
  result__c = gnutls_strerror(error__c);
  result = protected_copy_string(result__c);
  CAMLreturn(result);
}

value net_gnutls_strerror_name(value error) {
  error_code error__c;
  const char * result__c;
  CAMLparam1(error);
  CAMLlocal1(result);
  error__c = unwrap_error_code(error);
  nettls_init();
  result__c = gnutls_strerror_name(error__c);
  result = protected_copy_string(result__c);
  CAMLreturn(result);
}

value net_gnutls_check_version(value dummy) {
  const_charp req_version__c;
  const char * result__c;
  CAMLparam1(dummy);
  CAMLlocal1(result);
  req_version__c = NULL;
  nettls_init();
  result__c = gnutls_check_version(req_version__c);
  result = protected_copy_string(result__c);
  CAMLreturn(result);
}

value net_gnutls_global_set_log_level(value level) {
  int level__c;
  CAMLparam1(level);
  level__c = Int_val(level);
  nettls_init();
  gnutls_global_set_log_level(level__c);
  CAMLreturn(Val_unit);
}

value net_gnutls_init(value flags) {
  gnutls_session_t session__c;
  gnutls_init_flags flags__c;
  error_code result__c;
  CAMLparam1(flags);
  CAMLlocal1(session);
  flags__c = unwrap_gnutls_init_flags(flags);
  nettls_init();
  result__c = gnutls_init(&session__c,flags__c);
  net_gnutls_error_check(result__c);
  attach_session_callbacks(session__c);
  session = wrap_gnutls_session_t(session__c);
  CAMLreturn(session);
}

value net_gnutls_certificate_server_set_request(value session,value req) {
  gnutls_session_t session__c;
  gnutls_certificate_request_t req__c;
  CAMLparam2(session,req);
  session__c = unwrap_gnutls_session_t(session);
  req__c = unwrap_gnutls_certificate_request_t(req);
  nettls_init();
  gnutls_certificate_server_set_request(session__c,req__c);
  CAMLreturn(Val_unit);
}

value net_gnutls_certificate_send_x509_rdn_sequence(value session,value status) {
  gnutls_session_t session__c;
  int status__c;
  CAMLparam2(session,status);
  session__c = unwrap_gnutls_session_t(session);
  status__c = Bool_val(status);
  nettls_init();
  gnutls_certificate_send_x509_rdn_sequence(session__c,status__c);
  CAMLreturn(Val_unit);
}

value net_gnutls_certificate_verify_peers2(value session) {
  gnutls_session_t session__c;
  gnutls_certificate_status_t status__c;
  error_code result__c;
  CAMLparam1(session);
  CAMLlocal1(status);
  session__c = unwrap_gnutls_session_t(session);
  nettls_init();
  result__c = gnutls_certificate_verify_peers2(session__c,&status__c);
  net_gnutls_error_check(result__c);
  status = wrap_gnutls_certificate_status_t(status__c);
  CAMLreturn(status);
}

value net_gnutls_certificate_get_peers(value session) {
  gnutls_session_t session__c;
  unsigned int list_size__c;
  const str_datum * result__c;
  long local_0;
  CAMLparam1(session);
  CAMLlocal1(result);
  session__c = unwrap_gnutls_session_t(session);
  nettls_init();
  result__c = gnutls_certificate_get_peers(session__c,&list_size__c);
  if (result__c == NULL)
    result = caml_alloc(0,0);
  else {
    result = caml_alloc(list_size__c,0);
    for (local_0 = 0; local_0 < list_size__c; local_0++) {
      Store_field(result, local_0, wrap_str_datum(result__c[local_0]));
    };
  };
  CAMLreturn(result);
}

value net_gnutls_record_get_direction(value session) {
  gnutls_session_t session__c;
  int result__c;
  CAMLparam1(session);
  CAMLlocal1(result);
  session__c = unwrap_gnutls_session_t(session);
  nettls_init();
  result__c = gnutls_record_get_direction(session__c);
  result = Val_bool(result__c);
  CAMLreturn(result);
}

value net_gnutls_handshake(value session) {
  gnutls_session_t session__c;
  error_code result__c;
  CAMLparam1(session);
  session__c = unwrap_gnutls_session_t(session);
  nettls_init();
  result__c = gnutls_handshake(session__c);
  net_gnutls_error_check(result__c);
  CAMLreturn(Val_unit);
}

value net_gnutls_rehandshake(value session) {
  gnutls_session_t session__c;
  error_code result__c;
  CAMLparam1(session);
  session__c = unwrap_gnutls_session_t(session);
  nettls_init();
  result__c = gnutls_rehandshake(session__c);
  net_gnutls_error_check(result__c);
  CAMLreturn(Val_unit);
}

value net_gnutls_record_send(value session,value data,value data_size) {
  gnutls_session_t session__c;
  void * data__c;
  int data_size__c;
  error_code result__c;
  CAMLparam3(session,data,data_size);
  CAMLlocal1(result);
  session__c = unwrap_gnutls_session_t(session);
  data__c = Caml_ba_data_val(data);
  data_size__c = Int_val(data_size);
  nettls_init();
  if (data_size__c < 0 || data_size__c > caml_ba_byte_size(Caml_ba_array_val(data))) invalid_argument("gnutls_record_send");
  result__c = gnutls_record_send(session__c,data__c,data_size__c);
  net_gnutls_error_check(result__c);
  result = Val_int(result__c);
  CAMLreturn(result);
}

value net_gnutls_record_recv(value session,value data) {
  gnutls_session_t session__c;
  void * data__c;
  size_t data_size__c;
  error_code result__c;
  CAMLparam2(session,data);
  CAMLlocal1(result);
  session__c = unwrap_gnutls_session_t(session);
  data__c = Caml_ba_data_val(data);
  data_size__c = caml_ba_byte_size(Caml_ba_array_val(data));
  nettls_init();
  result__c = gnutls_record_recv(session__c,data__c,data_size__c);
  net_gnutls_error_check(result__c);
  result = Val_int(result__c);
  CAMLreturn(result);
}

value net_gnutls_record_check_pending(value session) {
  gnutls_session_t session__c;
  unsigned int result__c;
  CAMLparam1(session);
  CAMLlocal1(result);
  session__c = unwrap_gnutls_session_t(session);
  nettls_init();
  result__c = gnutls_record_check_pending(session__c);
  result = Val_int(result__c);
  CAMLreturn(result);
}

value net_gnutls_bye(value session,value how) {
  gnutls_session_t session__c;
  gnutls_close_request_t how__c;
  error_code result__c;
  CAMLparam2(session,how);
  session__c = unwrap_gnutls_session_t(session);
  how__c = unwrap_gnutls_close_request_t(how);
  nettls_init();
  result__c = gnutls_bye(session__c,how__c);
  net_gnutls_error_check(result__c);
  CAMLreturn(Val_unit);
}

value net_gnutls_alert_get(value session) {
  gnutls_session_t session__c;
  gnutls_alert_description_t result__c;
  CAMLparam1(session);
  CAMLlocal1(result);
  session__c = unwrap_gnutls_session_t(session);
  nettls_init();
  result__c = gnutls_alert_get(session__c);
  result = wrap_gnutls_alert_description_t(result__c);
  CAMLreturn(result);
}

value net_gnutls_alert_get_name(value alert) {
  gnutls_alert_description_t alert__c;
  const char * result__c;
  CAMLparam1(alert);
  CAMLlocal1(result);
  alert__c = unwrap_gnutls_alert_description_t(alert);
  nettls_init();
  result__c = gnutls_alert_get_name(alert__c);
  result = protected_copy_string(result__c);
  CAMLreturn(result);
}

value net_gnutls_alert_get_strname(value alert) {
#ifdef HAVE_FUN_gnutls_alert_get_strname
  gnutls_alert_description_t alert__c;
  const char * result__c;
  CAMLparam1(alert);
  CAMLlocal1(result);
  alert__c = unwrap_gnutls_alert_description_t(alert);
  nettls_init();
  result__c = gnutls_alert_get_strname(alert__c);
  result = protected_copy_string(result__c);
  CAMLreturn(result);
#else
  invalid_argument("gnutls_alert_get_strname");
#endif
}

value net_gnutls_alert_send(value session,value level,value desc) {
  gnutls_session_t session__c;
  gnutls_alert_level_t level__c;
  gnutls_alert_description_t desc__c;
  error_code result__c;
  CAMLparam3(session,level,desc);
  session__c = unwrap_gnutls_session_t(session);
  level__c = unwrap_gnutls_alert_level_t(level);
  desc__c = unwrap_gnutls_alert_description_t(desc);
  nettls_init();
  result__c = gnutls_alert_send(session__c,level__c,desc__c);
  net_gnutls_error_check(result__c);
  CAMLreturn(Val_unit);
}

value net_gnutls_alert_send_appropriate(value session,value err) {
  gnutls_session_t session__c;
  int err__c;
  error_code result__c;
  CAMLparam2(session,err);
  session__c = unwrap_gnutls_session_t(session);
  err__c = Int_val(err);
  nettls_init();
  result__c = gnutls_alert_send_appropriate(session__c,err__c);
  net_gnutls_error_check(result__c);
  CAMLreturn(Val_unit);
}

value net_gnutls_error_to_alert(value error) {
  error_code error__c;
  int level__c;
  gnutls_alert_description_t result__c;
  CAMLparam1(error);
  CAMLlocal3(level,result,local_0);
  error__c = unwrap_error_code(error);
  nettls_init();
  result__c = gnutls_error_to_alert(error__c,&level__c);
  level = wrap_gnutls_alert_level_t(level__c);
  result = wrap_gnutls_alert_description_t(result__c);
  local_0 = caml_alloc(2,0);
  Field(local_0, 0) = result;
  Field(local_0, 1) = level;
  CAMLreturn(local_0);
}

value net_gnutls_priority_init(value priorities) {
  gnutls_priority_t prority_cache__c;
  char * priorities__c;
  const char ** err_pos__c;
  error_code result__c;
  CAMLparam1(priorities);
  CAMLlocal1(prority_cache);
  const char *err_pos_dummy;
  priorities__c = String_val(priorities);
  nettls_init();
  err_pos__c = &err_pos_dummy;
  result__c = gnutls_priority_init(&prority_cache__c,priorities__c,err_pos__c);
  net_gnutls_error_check(result__c);
  prority_cache = wrap_gnutls_priority_t(prority_cache__c);
  CAMLreturn(prority_cache);
}

value net_gnutls_priority_get_cipher_suite_index(value pcache,value idx) {
#ifdef HAVE_FUN_gnutls_priority_get_cipher_suite_index
  gnutls_priority_t pcache__c;
  unsigned int idx__c;
  unsigned int sidx__c;
  error_code result__c;
  CAMLparam2(pcache,idx);
  CAMLlocal1(sidx);
  pcache__c = unwrap_gnutls_priority_t(pcache);
  idx__c = uint_val(idx);
  nettls_init();
  result__c = gnutls_priority_get_cipher_suite_index(pcache__c,idx__c,&sidx__c);
  net_gnutls_error_check(result__c);
  sidx = Val_int(sidx__c);
  CAMLreturn(sidx);
#else
  invalid_argument("gnutls_priority_get_cipher_suite_index");
#endif
}

value net_gnutls_priority_set(value session,value priority) {
  gnutls_session_t session__c;
  gnutls_priority_t priority__c;
  error_code result__c;
  CAMLparam2(session,priority);
  session__c = unwrap_gnutls_session_t(session);
  priority__c = unwrap_gnutls_priority_t(priority);
  nettls_init();
  result__c = gnutls_priority_set(session__c,priority__c);
  net_gnutls_error_check(result__c);
  CAMLreturn(Val_unit);
}

value net_gnutls_set_default_priority(value session) {
  gnutls_session_t session__c;
  error_code result__c;
  CAMLparam1(session);
  session__c = unwrap_gnutls_session_t(session);
  nettls_init();
  result__c = gnutls_set_default_priority(session__c);
  net_gnutls_error_check(result__c);
  CAMLreturn(Val_unit);
}

value net_gnutls_sec_param_to_pk_bits(value algo,value param) {
#ifdef HAVE_FUN_gnutls_sec_param_to_pk_bits
  gnutls_pk_algorithm_t algo__c;
  gnutls_sec_param_t param__c;
  unsigned int result__c;
  CAMLparam2(algo,param);
  CAMLlocal1(result);
  algo__c = unwrap_gnutls_pk_algorithm_t(algo);
  param__c = unwrap_gnutls_sec_param_t(param);
  nettls_init();
  result__c = gnutls_sec_param_to_pk_bits(algo__c,param__c);
  result = Val_int(result__c);
  CAMLreturn(result);
#else
  invalid_argument("gnutls_sec_param_to_pk_bits");
#endif
}

value net_gnutls_pk_bits_to_sec_param(value algo,value bits) {
#ifdef HAVE_FUN_gnutls_pk_bits_to_sec_param
  gnutls_pk_algorithm_t algo__c;
  unsigned int bits__c;
  gnutls_sec_param_t result__c;
  CAMLparam2(algo,bits);
  CAMLlocal1(result);
  algo__c = unwrap_gnutls_pk_algorithm_t(algo);
  bits__c = uint_val(bits);
  nettls_init();
  result__c = gnutls_pk_bits_to_sec_param(algo__c,bits__c);
  result = wrap_gnutls_sec_param_t(result__c);
  CAMLreturn(result);
#else
  invalid_argument("gnutls_pk_bits_to_sec_param");
#endif
}

value net_gnutls_sec_param_get_name(value param) {
#ifdef HAVE_FUN_gnutls_sec_param_get_name
  gnutls_sec_param_t param__c;
  const char * result__c;
  CAMLparam1(param);
  CAMLlocal1(result);
  param__c = unwrap_gnutls_sec_param_t(param);
  nettls_init();
  result__c = gnutls_sec_param_get_name(param__c);
  result = protected_copy_string(result__c);
  CAMLreturn(result);
#else
  invalid_argument("gnutls_sec_param_get_name");
#endif
}

value net_gnutls_session_set_data(value session,value session_data) {
  gnutls_session_t session__c;
  void * session_data__c;
  size_t session_data_size__c;
  error_code result__c;
  CAMLparam2(session,session_data);
  session__c = unwrap_gnutls_session_t(session);
  session_data__c = String_val(session_data);
  session_data_size__c = caml_string_length(session_data);
  nettls_init();
  result__c = gnutls_session_set_data(session__c,session_data__c,session_data_size__c);
  net_gnutls_error_check(result__c);
  CAMLreturn(Val_unit);
}

value net_gnutls_session_get_data(value session) {
  gnutls_session_t session__c;
  void * session_data__c;
  size_t session_data_size__c;
  error_code result__c;
  CAMLparam1(session);
  CAMLlocal1(session_data);
  session__c = unwrap_gnutls_session_t(session);
  nettls_init();
  session_data__c = NULL;
  session_data_size__c = 0;
  session_data = caml_alloc_string(0);
  result__c = gnutls_session_get_data(session__c,session_data__c,&session_data_size__c);
  if (result__c == 0 || result__c == GNUTLS_E_SHORT_MEMORY_BUFFER) {
    session_data = caml_alloc_string(session_data_size__c);
    session_data__c = String_val(session_data);
    result__c = gnutls_session_get_data(session__c,session_data__c,&session_data_size__c);
  };
  net_gnutls_error_check(result__c);
  CAMLreturn(session_data);
}

value net_gnutls_session_get_id(value session) {
  gnutls_session_t session__c;
  void * session_id__c;
  size_t session_id_size__c;
  error_code result__c;
  CAMLparam1(session);
  CAMLlocal1(session_id);
  session__c = unwrap_gnutls_session_t(session);
  nettls_init();
  session_id__c = NULL;
  session_id_size__c = 0;
  session_id = caml_alloc_string(0);
  result__c = gnutls_session_get_id(session__c,session_id__c,&session_id_size__c);
  if (result__c == 0 || result__c == GNUTLS_E_SHORT_MEMORY_BUFFER) {
    session_id = caml_alloc_string(session_id_size__c);
    session_id__c = String_val(session_id);
    result__c = gnutls_session_get_id(session__c,session_id__c,&session_id_size__c);
  };
  net_gnutls_error_check(result__c);
  CAMLreturn(session_id);
}

value net_gnutls_session_is_resumed(value session) {
  gnutls_session_t session__c;
  int result__c;
  CAMLparam1(session);
  CAMLlocal1(result);
  session__c = unwrap_gnutls_session_t(session);
  nettls_init();
  result__c = gnutls_session_is_resumed(session__c);
  result = Val_bool(result__c);
  CAMLreturn(result);
}

value net_gnutls_db_set_cache_expiration(value session,value seconds) {
  gnutls_session_t session__c;
  int seconds__c;
  CAMLparam2(session,seconds);
  session__c = unwrap_gnutls_session_t(session);
  seconds__c = Int_val(seconds);
  nettls_init();
  gnutls_db_set_cache_expiration(session__c,seconds__c);
  CAMLreturn(Val_unit);
}

value net_gnutls_db_remove_session(value session) {
  gnutls_session_t session__c;
  CAMLparam1(session);
  session__c = unwrap_gnutls_session_t(session);
  nettls_init();
  gnutls_db_remove_session(session__c);
  CAMLreturn(Val_unit);
}

value net_gnutls_db_check_entry(value session,value session_entry) {
  gnutls_session_t session__c;
  str_datum session_entry__c;
  int result__c;
  CAMLparam2(session,session_entry);
  CAMLlocal1(result);
  session__c = unwrap_gnutls_session_t(session);
  session_entry__c = unwrap_str_datum(session_entry);
  nettls_init();
  result__c = gnutls_db_check_entry(session__c,session_entry__c);
  free_str_datum(session_entry__c);
  result = Val_int(result__c);
  CAMLreturn(result);
}

value net_gnutls_session_ticket_key_generate(value dummy) {
#ifdef HAVE_FUN_gnutls_session_ticket_key_generate
  str_datum key__c;
  error_code result__c;
  CAMLparam1(dummy);
  CAMLlocal1(key);
  nettls_init();
  result__c = gnutls_session_ticket_key_generate(&key__c);
  net_gnutls_error_check(result__c);
  key = wrap_str_datum(key__c);
  CAMLreturn(key);
#else
  invalid_argument("gnutls_session_ticket_key_generate");
#endif
}

value net_gnutls_session_ticket_enable_client(value session) {
#ifdef HAVE_FUN_gnutls_session_ticket_enable_client
  gnutls_session_t session__c;
  error_code result__c;
  CAMLparam1(session);
  session__c = unwrap_gnutls_session_t(session);
  nettls_init();
  result__c = gnutls_session_ticket_enable_client(session__c);
  net_gnutls_error_check(result__c);
  CAMLreturn(Val_unit);
#else
  invalid_argument("gnutls_session_ticket_enable_client");
#endif
}

value net_gnutls_session_ticket_enable_server(value session,value key) {
#ifdef HAVE_FUN_gnutls_session_ticket_enable_server
  gnutls_session_t session__c;
  str_datum_p key__c;
  error_code result__c;
  CAMLparam2(session,key);
  session__c = unwrap_gnutls_session_t(session);
  key__c = unwrap_str_datum_p(key);
  nettls_init();
  result__c = gnutls_session_ticket_enable_server(session__c,key__c);
  free_str_datum_p(key__c);
  net_gnutls_error_check(result__c);
  CAMLreturn(Val_unit);
#else
  invalid_argument("gnutls_session_ticket_enable_server");
#endif
}

value net_gnutls_prf(value session,value label,value server_random_first,value extra,value out) {
  gnutls_session_t session__c;
  size_t label_size__c;
  void * label__c;
  int server_random_first__c;
  size_t extra_size__c;
  void * extra__c;
  size_t outsize__c;
  void * out__c;
  error_code result__c;
  CAMLparam5(session,label,server_random_first,extra,out);
  session__c = unwrap_gnutls_session_t(session);
  label_size__c = caml_ba_byte_size(Caml_ba_array_val(label));
  label__c = Caml_ba_data_val(label);
  server_random_first__c = Bool_val(server_random_first);
  extra_size__c = caml_ba_byte_size(Caml_ba_array_val(extra));
  extra__c = Caml_ba_data_val(extra);
  outsize__c = caml_ba_byte_size(Caml_ba_array_val(out));
  out__c = Caml_ba_data_val(out);
  nettls_init();
  result__c = gnutls_prf(session__c,label_size__c,label__c,server_random_first__c,extra_size__c,extra__c,outsize__c,out__c);
  net_gnutls_error_check(result__c);
  CAMLreturn(Val_unit);
}

value net_gnutls_prf_raw(value session,value label,value seed,value out) {
  gnutls_session_t session__c;
  size_t label_size__c;
  void * label__c;
  size_t seed_size__c;
  void * seed__c;
  size_t outsize__c;
  void * out__c;
  error_code result__c;
  CAMLparam4(session,label,seed,out);
  session__c = unwrap_gnutls_session_t(session);
  label_size__c = caml_ba_byte_size(Caml_ba_array_val(label));
  label__c = Caml_ba_data_val(label);
  seed_size__c = caml_ba_byte_size(Caml_ba_array_val(seed));
  seed__c = Caml_ba_data_val(seed);
  outsize__c = caml_ba_byte_size(Caml_ba_array_val(out));
  out__c = Caml_ba_data_val(out);
  nettls_init();
  result__c = gnutls_prf_raw(session__c,label_size__c,label__c,seed_size__c,seed__c,outsize__c,out__c);
  net_gnutls_error_check(result__c);
  CAMLreturn(Val_unit);
}

value net_gnutls_session_channel_binding(value session,value cbtype) {
#ifdef HAVE_FUN_gnutls_session_channel_binding
  gnutls_session_t session__c;
  gnutls_channel_binding_t cbtype__c;
  str_datum cb__c;
  error_code result__c;
  CAMLparam2(session,cbtype);
  CAMLlocal1(cb);
  session__c = unwrap_gnutls_session_t(session);
  cbtype__c = unwrap_gnutls_channel_binding_t(cbtype);
  nettls_init();
  result__c = gnutls_session_channel_binding(session__c,cbtype__c,&cb__c);
  net_gnutls_error_check(result__c);
  cb = wrap_str_datum(cb__c);
  CAMLreturn(cb);
#else
  invalid_argument("gnutls_session_channel_binding");
#endif
}

value net_gnutls_ecc_curve_get_name(value curve) {
#ifdef HAVE_FUN_gnutls_ecc_curve_get_name
  gnutls_ecc_curve_t curve__c;
  const char * result__c;
  CAMLparam1(curve);
  CAMLlocal1(result);
  curve__c = unwrap_gnutls_ecc_curve_t(curve);
  nettls_init();
  result__c = gnutls_ecc_curve_get_name(curve__c);
  result = protected_copy_string(result__c);
  CAMLreturn(result);
#else
  invalid_argument("gnutls_ecc_curve_get_name");
#endif
}

value net_gnutls_ecc_curve_get_size(value curve) {
#ifdef HAVE_FUN_gnutls_ecc_curve_get_size
  gnutls_ecc_curve_t curve__c;
  int result__c;
  CAMLparam1(curve);
  CAMLlocal1(result);
  curve__c = unwrap_gnutls_ecc_curve_t(curve);
  nettls_init();
  result__c = gnutls_ecc_curve_get_size(curve__c);
  result = Val_int(result__c);
  CAMLreturn(result);
#else
  invalid_argument("gnutls_ecc_curve_get_size");
#endif
}

value net_gnutls_ecc_curve_get(value session) {
#ifdef HAVE_FUN_gnutls_ecc_curve_get
  gnutls_session_t session__c;
  gnutls_ecc_curve_t result__c;
  CAMLparam1(session);
  CAMLlocal1(result);
  session__c = unwrap_gnutls_session_t(session);
  nettls_init();
  result__c = gnutls_ecc_curve_get(session__c);
  result = wrap_gnutls_ecc_curve_t(result__c);
  CAMLreturn(result);
#else
  invalid_argument("gnutls_ecc_curve_get");
#endif
}

value net_gnutls_cipher_get(value session) {
  gnutls_session_t session__c;
  gnutls_cipher_algorithm_t result__c;
  CAMLparam1(session);
  CAMLlocal1(result);
  session__c = unwrap_gnutls_session_t(session);
  nettls_init();
  result__c = gnutls_cipher_get(session__c);
  result = wrap_gnutls_cipher_algorithm_t(result__c);
  CAMLreturn(result);
}

value net_gnutls_kx_get(value session) {
  gnutls_session_t session__c;
  gnutls_kx_algorithm_t result__c;
  CAMLparam1(session);
  CAMLlocal1(result);
  session__c = unwrap_gnutls_session_t(session);
  nettls_init();
  result__c = gnutls_kx_get(session__c);
  result = wrap_gnutls_kx_algorithm_t(result__c);
  CAMLreturn(result);
}

value net_gnutls_mac_get(value session) {
  gnutls_session_t session__c;
  gnutls_mac_algorithm_t result__c;
  CAMLparam1(session);
  CAMLlocal1(result);
  session__c = unwrap_gnutls_session_t(session);
  nettls_init();
  result__c = gnutls_mac_get(session__c);
  result = wrap_gnutls_mac_algorithm_t(result__c);
  CAMLreturn(result);
}

value net_gnutls_compression_get(value session) {
  gnutls_session_t session__c;
  gnutls_compression_method_t result__c;
  CAMLparam1(session);
  CAMLlocal1(result);
  session__c = unwrap_gnutls_session_t(session);
  nettls_init();
  result__c = gnutls_compression_get(session__c);
  result = wrap_gnutls_compression_method_t(result__c);
  CAMLreturn(result);
}

value net_gnutls_certificate_type_get(value session) {
  gnutls_session_t session__c;
  gnutls_certificate_type_t result__c;
  CAMLparam1(session);
  CAMLlocal1(result);
  session__c = unwrap_gnutls_session_t(session);
  nettls_init();
  result__c = gnutls_certificate_type_get(session__c);
  result = wrap_gnutls_certificate_type_t(result__c);
  CAMLreturn(result);
}

value net_gnutls_cipher_get_key_size(value algorithm) {
  gnutls_cipher_algorithm_t algorithm__c;
  unsigned int result__c;
  CAMLparam1(algorithm);
  CAMLlocal1(result);
  algorithm__c = unwrap_gnutls_cipher_algorithm_t(algorithm);
  nettls_init();
  result__c = gnutls_cipher_get_key_size(algorithm__c);
  result = Val_int(result__c);
  CAMLreturn(result);
}

value net_gnutls_mac_get_key_size(value algorithm) {
  gnutls_mac_algorithm_t algorithm__c;
  unsigned int result__c;
  CAMLparam1(algorithm);
  CAMLlocal1(result);
  algorithm__c = unwrap_gnutls_mac_algorithm_t(algorithm);
  nettls_init();
  result__c = gnutls_mac_get_key_size(algorithm__c);
  result = Val_int(result__c);
  CAMLreturn(result);
}

value net_gnutls_cipher_get_name(value algorithm) {
  gnutls_cipher_algorithm_t algorithm__c;
  const char * result__c;
  CAMLparam1(algorithm);
  CAMLlocal1(result);
  algorithm__c = unwrap_gnutls_cipher_algorithm_t(algorithm);
  nettls_init();
  result__c = gnutls_cipher_get_name(algorithm__c);
  result = protected_copy_string(result__c);
  CAMLreturn(result);
}

value net_gnutls_mac_get_name(value algorithm) {
  gnutls_mac_algorithm_t algorithm__c;
  const char * result__c;
  CAMLparam1(algorithm);
  CAMLlocal1(result);
  algorithm__c = unwrap_gnutls_mac_algorithm_t(algorithm);
  nettls_init();
  result__c = gnutls_mac_get_name(algorithm__c);
  result = protected_copy_string(result__c);
  CAMLreturn(result);
}

value net_gnutls_compression_get_name(value algo) {
  gnutls_compression_method_t algo__c;
  const char * result__c;
  CAMLparam1(algo);
  CAMLlocal1(result);
  algo__c = unwrap_gnutls_compression_method_t(algo);
  nettls_init();
  result__c = gnutls_compression_get_name(algo__c);
  result = protected_copy_string(result__c);
  CAMLreturn(result);
}

value net_gnutls_kx_get_name(value algorithm) {
  gnutls_kx_algorithm_t algorithm__c;
  const char * result__c;
  CAMLparam1(algorithm);
  CAMLlocal1(result);
  algorithm__c = unwrap_gnutls_kx_algorithm_t(algorithm);
  nettls_init();
  result__c = gnutls_kx_get_name(algorithm__c);
  result = protected_copy_string(result__c);
  CAMLreturn(result);
}

value net_gnutls_certificate_type_get_name(value ty) {
  gnutls_certificate_type_t ty__c;
  const char * result__c;
  CAMLparam1(ty);
  CAMLlocal1(result);
  ty__c = unwrap_gnutls_certificate_type_t(ty);
  nettls_init();
  result__c = gnutls_certificate_type_get_name(ty__c);
  result = protected_copy_string(result__c);
  CAMLreturn(result);
}

value net_gnutls_pk_get_name(value algorithm) {
  gnutls_pk_algorithm_t algorithm__c;
  const char * result__c;
  CAMLparam1(algorithm);
  CAMLlocal1(result);
  algorithm__c = unwrap_gnutls_pk_algorithm_t(algorithm);
  nettls_init();
  result__c = gnutls_pk_get_name(algorithm__c);
  result = protected_copy_string(result__c);
  CAMLreturn(result);
}

value net_gnutls_sign_get_name(value algorithm) {
  gnutls_sign_algorithm_t algorithm__c;
  const char * result__c;
  CAMLparam1(algorithm);
  CAMLlocal1(result);
  algorithm__c = unwrap_gnutls_sign_algorithm_t(algorithm);
  nettls_init();
  result__c = gnutls_sign_get_name(algorithm__c);
  result = protected_copy_string(result__c);
  CAMLreturn(result);
}

value net_gnutls_mac_get_id(value name) {
  char * name__c;
  gnutls_mac_algorithm_t result__c;
  CAMLparam1(name);
  CAMLlocal1(result);
  name__c = String_val(name);
  nettls_init();
  result__c = gnutls_mac_get_id(name__c);
  result = wrap_gnutls_mac_algorithm_t(result__c);
  CAMLreturn(result);
}

value net_gnutls_compression_get_id(value name) {
  char * name__c;
  gnutls_compression_method_t result__c;
  CAMLparam1(name);
  CAMLlocal1(result);
  name__c = String_val(name);
  nettls_init();
  result__c = gnutls_compression_get_id(name__c);
  result = wrap_gnutls_compression_method_t(result__c);
  CAMLreturn(result);
}

value net_gnutls_cipher_get_id(value name) {
  char * name__c;
  gnutls_cipher_algorithm_t result__c;
  CAMLparam1(name);
  CAMLlocal1(result);
  name__c = String_val(name);
  nettls_init();
  result__c = gnutls_cipher_get_id(name__c);
  result = wrap_gnutls_cipher_algorithm_t(result__c);
  CAMLreturn(result);
}

value net_gnutls_kx_get_id(value name) {
  char * name__c;
  gnutls_kx_algorithm_t result__c;
  CAMLparam1(name);
  CAMLlocal1(result);
  name__c = String_val(name);
  nettls_init();
  result__c = gnutls_kx_get_id(name__c);
  result = wrap_gnutls_kx_algorithm_t(result__c);
  CAMLreturn(result);
}

value net_gnutls_protocol_get_id(value name) {
  char * name__c;
  gnutls_protocol_t result__c;
  CAMLparam1(name);
  CAMLlocal1(result);
  name__c = String_val(name);
  nettls_init();
  result__c = gnutls_protocol_get_id(name__c);
  result = wrap_gnutls_protocol_t(result__c);
  CAMLreturn(result);
}

value net_gnutls_certificate_type_get_id(value name) {
  char * name__c;
  gnutls_certificate_type_t result__c;
  CAMLparam1(name);
  CAMLlocal1(result);
  name__c = String_val(name);
  nettls_init();
  result__c = gnutls_certificate_type_get_id(name__c);
  result = wrap_gnutls_certificate_type_t(result__c);
  CAMLreturn(result);
}

value net_gnutls_pk_get_id(value name) {
  char * name__c;
  gnutls_pk_algorithm_t result__c;
  CAMLparam1(name);
  CAMLlocal1(result);
  name__c = String_val(name);
  nettls_init();
  result__c = gnutls_pk_get_id(name__c);
  result = wrap_gnutls_pk_algorithm_t(result__c);
  CAMLreturn(result);
}

value net_gnutls_sign_get_id(value name) {
  char * name__c;
  gnutls_sign_algorithm_t result__c;
  CAMLparam1(name);
  CAMLlocal1(result);
  name__c = String_val(name);
  nettls_init();
  result__c = gnutls_sign_get_id(name__c);
  result = wrap_gnutls_sign_algorithm_t(result__c);
  CAMLreturn(result);
}

value net_gnutls_cipher_list(value dummy) {
  const gnutls_cipher_algorithm_t * result__c;
  long local_0;
  CAMLparam1(dummy);
  CAMLlocal2(result,local_1);
  nettls_init();
  result__c = gnutls_cipher_list();
  local_0 = 0;
  while (result__c[local_0] != 0) local_0++;
  result = Val_int(0);
  while (local_0 > 0) {
    local_0--;
    local_1 = caml_alloc(2,0);
    Field(local_1,0) = wrap_gnutls_cipher_algorithm_t(result__c[local_0]);
    Field(local_1,1) = result;
    result = local_1;
  };
  CAMLreturn(result);
}

value net_gnutls_mac_list(value dummy) {
  const gnutls_mac_algorithm_t * result__c;
  long local_0;
  CAMLparam1(dummy);
  CAMLlocal2(result,local_1);
  nettls_init();
  result__c = gnutls_mac_list();
  local_0 = 0;
  while (result__c[local_0] != 0) local_0++;
  result = Val_int(0);
  while (local_0 > 0) {
    local_0--;
    local_1 = caml_alloc(2,0);
    Field(local_1,0) = wrap_gnutls_mac_algorithm_t(result__c[local_0]);
    Field(local_1,1) = result;
    result = local_1;
  };
  CAMLreturn(result);
}

value net_gnutls_compression_list(value dummy) {
  const gnutls_compression_method_t * result__c;
  long local_0;
  CAMLparam1(dummy);
  CAMLlocal2(result,local_1);
  nettls_init();
  result__c = gnutls_compression_list();
  local_0 = 0;
  while (result__c[local_0] != 0) local_0++;
  result = Val_int(0);
  while (local_0 > 0) {
    local_0--;
    local_1 = caml_alloc(2,0);
    Field(local_1,0) = wrap_gnutls_compression_method_t(result__c[local_0]);
    Field(local_1,1) = result;
    result = local_1;
  };
  CAMLreturn(result);
}

value net_gnutls_protocol_list(value dummy) {
  const gnutls_protocol_t * result__c;
  long local_0;
  CAMLparam1(dummy);
  CAMLlocal2(result,local_1);
  nettls_init();
  result__c = gnutls_protocol_list();
  local_0 = 0;
  while (result__c[local_0] != 0) local_0++;
  result = Val_int(0);
  while (local_0 > 0) {
    local_0--;
    local_1 = caml_alloc(2,0);
    Field(local_1,0) = wrap_gnutls_protocol_t(result__c[local_0]);
    Field(local_1,1) = result;
    result = local_1;
  };
  CAMLreturn(result);
}

value net_gnutls_certificate_type_list(value dummy) {
  const gnutls_certificate_type_t * result__c;
  long local_0;
  CAMLparam1(dummy);
  CAMLlocal2(result,local_1);
  nettls_init();
  result__c = gnutls_certificate_type_list();
  local_0 = 0;
  while (result__c[local_0] != 0) local_0++;
  result = Val_int(0);
  while (local_0 > 0) {
    local_0--;
    local_1 = caml_alloc(2,0);
    Field(local_1,0) = wrap_gnutls_certificate_type_t(result__c[local_0]);
    Field(local_1,1) = result;
    result = local_1;
  };
  CAMLreturn(result);
}

value net_gnutls_kx_list(value dummy) {
  const gnutls_kx_algorithm_t * result__c;
  long local_0;
  CAMLparam1(dummy);
  CAMLlocal2(result,local_1);
  nettls_init();
  result__c = gnutls_kx_list();
  local_0 = 0;
  while (result__c[local_0] != 0) local_0++;
  result = Val_int(0);
  while (local_0 > 0) {
    local_0--;
    local_1 = caml_alloc(2,0);
    Field(local_1,0) = wrap_gnutls_kx_algorithm_t(result__c[local_0]);
    Field(local_1,1) = result;
    result = local_1;
  };
  CAMLreturn(result);
}

value net_gnutls_pk_list(value dummy) {
  const gnutls_pk_algorithm_t * result__c;
  long local_0;
  CAMLparam1(dummy);
  CAMLlocal2(result,local_1);
  nettls_init();
  result__c = gnutls_pk_list();
  local_0 = 0;
  while (result__c[local_0] != 0) local_0++;
  result = Val_int(0);
  while (local_0 > 0) {
    local_0--;
    local_1 = caml_alloc(2,0);
    Field(local_1,0) = wrap_gnutls_pk_algorithm_t(result__c[local_0]);
    Field(local_1,1) = result;
    result = local_1;
  };
  CAMLreturn(result);
}

value net_gnutls_sign_list(value dummy) {
  const gnutls_sign_algorithm_t * result__c;
  long local_0;
  CAMLparam1(dummy);
  CAMLlocal2(result,local_1);
  nettls_init();
  result__c = gnutls_sign_list();
  local_0 = 0;
  while (result__c[local_0] != 0) local_0++;
  result = Val_int(0);
  while (local_0 > 0) {
    local_0--;
    local_1 = caml_alloc(2,0);
    Field(local_1,0) = wrap_gnutls_sign_algorithm_t(result__c[local_0]);
    Field(local_1,1) = result;
    result = local_1;
  };
  CAMLreturn(result);
}

value net_gnutls_protocol_get_version(value session) {
  gnutls_session_t session__c;
  gnutls_protocol_t result__c;
  CAMLparam1(session);
  CAMLlocal1(result);
  session__c = unwrap_gnutls_session_t(session);
  nettls_init();
  result__c = gnutls_protocol_get_version(session__c);
  result = wrap_gnutls_protocol_t(result__c);
  CAMLreturn(result);
}

value net_gnutls_protocol_get_name(value version) {
  gnutls_protocol_t version__c;
  const char * result__c;
  CAMLparam1(version);
  CAMLlocal1(result);
  version__c = unwrap_gnutls_protocol_t(version);
  nettls_init();
  result__c = gnutls_protocol_get_name(version__c);
  result = protected_copy_string(result__c);
  CAMLreturn(result);
}

value net_gnutls_cipher_suite_get_name(value kx_algorithm,value cipher_algorithm,value mac_algorithm) {
  gnutls_kx_algorithm_t kx_algorithm__c;
  gnutls_cipher_algorithm_t cipher_algorithm__c;
  gnutls_mac_algorithm_t mac_algorithm__c;
  const char * result__c;
  CAMLparam3(kx_algorithm,cipher_algorithm,mac_algorithm);
  CAMLlocal1(result);
  kx_algorithm__c = unwrap_gnutls_kx_algorithm_t(kx_algorithm);
  cipher_algorithm__c = unwrap_gnutls_cipher_algorithm_t(cipher_algorithm);
  mac_algorithm__c = unwrap_gnutls_mac_algorithm_t(mac_algorithm);
  nettls_init();
  result__c = gnutls_cipher_suite_get_name(kx_algorithm__c,cipher_algorithm__c,mac_algorithm__c);
  result = protected_copy_string(result__c);
  CAMLreturn(result);
}

value net_net_have_crypto(value dummy) {
  int result__c;
  CAMLparam1(dummy);
  CAMLlocal1(result);
  nettls_init();
  result__c = net_have_crypto();
  result = Val_bool(result__c);
  CAMLreturn(result);
}

value net_gnutls_cipher_init(value cipher,value key,value iv) {
#ifdef HAVE_FUN_gnutls_cipher_init
  gnutls_cipher_hd_t handle__c;
  gnutls_cipher_algorithm_t cipher__c;
  str_datum_p key__c;
  str_datum_p iv__c;
  error_code result__c;
  CAMLparam3(cipher,key,iv);
  CAMLlocal1(handle);
  cipher__c = unwrap_gnutls_cipher_algorithm_t(cipher);
  key__c = unwrap_str_datum_p(key);
  iv__c = unwrap_str_datum_p(iv);
  nettls_init();
  result__c = gnutls_cipher_init(&handle__c,cipher__c,key__c,iv__c);
  free_str_datum_p(key__c);
  free_str_datum_p(iv__c);
  net_gnutls_error_check(result__c);
  handle = wrap_gnutls_cipher_hd_t(handle__c);
  CAMLreturn(handle);
#else
  invalid_argument("gnutls_cipher_init");
#endif
}

value net_gnutls_cipher_encrypt2(value handle,value text,value ciphertext) {
#ifdef HAVE_FUN_gnutls_cipher_encrypt2
  gnutls_cipher_hd_t handle__c;
  void * text__c;
  size_t textlen__c;
  void * ciphertext__c;
  size_t ciphertextlen__c;
  error_code result__c;
  CAMLparam3(handle,text,ciphertext);
  handle__c = unwrap_gnutls_cipher_hd_t(handle);
  text__c = Caml_ba_data_val(text);
  textlen__c = caml_ba_byte_size(Caml_ba_array_val(text));
  ciphertext__c = Caml_ba_data_val(ciphertext);
  ciphertextlen__c = caml_ba_byte_size(Caml_ba_array_val(ciphertext));
  nettls_init();
  result__c = gnutls_cipher_encrypt2(handle__c,text__c,textlen__c,ciphertext__c,ciphertextlen__c);
  net_gnutls_error_check(result__c);
  CAMLreturn(Val_unit);
#else
  invalid_argument("gnutls_cipher_encrypt2");
#endif
}

value net_gnutls_cipher_decrypt2(value handle,value ciphertext,value text) {
#ifdef HAVE_FUN_gnutls_cipher_decrypt2
  gnutls_cipher_hd_t handle__c;
  void * ciphertext__c;
  size_t ciphertextlen__c;
  void * text__c;
  size_t textlen__c;
  error_code result__c;
  CAMLparam3(handle,ciphertext,text);
  handle__c = unwrap_gnutls_cipher_hd_t(handle);
  ciphertext__c = Caml_ba_data_val(ciphertext);
  ciphertextlen__c = caml_ba_byte_size(Caml_ba_array_val(text));
  text__c = Caml_ba_data_val(text);
  textlen__c = caml_ba_byte_size(Caml_ba_array_val(text));
  nettls_init();
  result__c = gnutls_cipher_decrypt2(handle__c,ciphertext__c,ciphertextlen__c,text__c,textlen__c);
  net_gnutls_error_check(result__c);
  CAMLreturn(Val_unit);
#else
  invalid_argument("gnutls_cipher_decrypt2");
#endif
}

value net_gnutls_cipher_add_auth(value handle,value text) {
#ifdef HAVE_FUN_gnutls_cipher_add_auth
  gnutls_cipher_hd_t handle__c;
  void * text__c;
  size_t text_size__c;
  error_code result__c;
  CAMLparam2(handle,text);
  handle__c = unwrap_gnutls_cipher_hd_t(handle);
  text__c = String_val(text);
  text_size__c = caml_string_length(text);
  nettls_init();
  result__c = gnutls_cipher_add_auth(handle__c,text__c,text_size__c);
  net_gnutls_error_check(result__c);
  CAMLreturn(Val_unit);
#else
  invalid_argument("gnutls_cipher_add_auth");
#endif
}

value net_gnutls_cipher_tag(value handle,value tag) {
#ifdef HAVE_FUN_gnutls_cipher_tag
  gnutls_cipher_hd_t handle__c;
  void * tag__c;
  size_t tag_size__c;
  error_code result__c;
  CAMLparam2(handle,tag);
  handle__c = unwrap_gnutls_cipher_hd_t(handle);
  tag__c = String_val(tag);
  tag_size__c = caml_string_length(tag);
  nettls_init();
  result__c = gnutls_cipher_tag(handle__c,tag__c,tag_size__c);
  net_gnutls_error_check(result__c);
  CAMLreturn(Val_unit);
#else
  invalid_argument("gnutls_cipher_tag");
#endif
}

value net_gnutls_handshake_set_private_extensions(value session,value allow) {
  gnutls_session_t session__c;
  int allow__c;
  CAMLparam2(session,allow);
  session__c = unwrap_gnutls_session_t(session);
  allow__c = Bool_val(allow);
  nettls_init();
  gnutls_handshake_set_private_extensions(session__c,allow__c);
  CAMLreturn(Val_unit);
}

value net_gnutls_handshake_get_last_out(value session) {
  gnutls_session_t session__c;
  gnutls_handshake_description_t result__c;
  CAMLparam1(session);
  CAMLlocal1(result);
  session__c = unwrap_gnutls_session_t(session);
  nettls_init();
  result__c = gnutls_handshake_get_last_out(session__c);
  result = wrap_gnutls_handshake_description_t(result__c);
  CAMLreturn(result);
}

value net_gnutls_handshake_get_last_in(value session) {
  gnutls_session_t session__c;
  gnutls_handshake_description_t result__c;
  CAMLparam1(session);
  CAMLlocal1(result);
  session__c = unwrap_gnutls_session_t(session);
  nettls_init();
  result__c = gnutls_handshake_get_last_in(session__c);
  result = wrap_gnutls_handshake_description_t(result__c);
  CAMLreturn(result);
}

value net_gnutls_handshake_set_max_packet_length(value session,value max) {
  gnutls_session_t session__c;
  unsigned int max__c;
  CAMLparam2(session,max);
  session__c = unwrap_gnutls_session_t(session);
  max__c = uint_val(max);
  nettls_init();
  gnutls_handshake_set_max_packet_length(session__c,max__c);
  CAMLreturn(Val_unit);
}

value net_gnutls_sign_algorithm_get_requested(value session,value indx) {
#ifdef HAVE_FUN_gnutls_sign_algorithm_get_requested
  gnutls_session_t session__c;
  unsigned int indx__c;
  gnutls_sign_algorithm_t algo__c;
  error_code result__c;
  CAMLparam2(session,indx);
  CAMLlocal1(algo);
  session__c = unwrap_gnutls_session_t(session);
  indx__c = uint_val(indx);
  nettls_init();
  result__c = gnutls_sign_algorithm_get_requested(session__c,indx__c,&algo__c);
  net_gnutls_error_check(result__c);
  algo = wrap_gnutls_sign_algorithm_t(algo__c);
  CAMLreturn(algo);
#else
  invalid_argument("gnutls_sign_algorithm_get_requested");
#endif
}

value net_gnutls_session_enable_compatibility_mode(value session) {
  gnutls_session_t session__c;
  CAMLparam1(session);
  session__c = unwrap_gnutls_session_t(session);
  nettls_init();
  gnutls_session_enable_compatibility_mode(session__c);
  CAMLreturn(Val_unit);
}

value net_gnutls_record_disable_padding(value session) {
  gnutls_session_t session__c;
  CAMLparam1(session);
  session__c = unwrap_gnutls_session_t(session);
  nettls_init();
  gnutls_record_disable_padding(session__c);
  CAMLreturn(Val_unit);
}

value net_gnutls_record_get_max_size(value session) {
  gnutls_session_t session__c;
  unsigned int result__c;
  CAMLparam1(session);
  CAMLlocal1(result);
  session__c = unwrap_gnutls_session_t(session);
  nettls_init();
  result__c = gnutls_record_get_max_size(session__c);
  result = Val_int(result__c);
  CAMLreturn(result);
}

value net_gnutls_record_set_max_size(value session,value size) {
  gnutls_session_t session__c;
  unsigned int size__c;
  error_code result__c;
  CAMLparam2(session,size);
  session__c = unwrap_gnutls_session_t(session);
  size__c = uint_val(size);
  nettls_init();
  result__c = gnutls_record_set_max_size(session__c,size__c);
  net_gnutls_error_check(result__c);
  CAMLreturn(Val_unit);
}

value net_gnutls_server_name_set(value session,value type,value name) {
  gnutls_session_t session__c;
  gnutls_server_name_type_t type__c;
  void * name__c;
  size_t name_length__c;
  error_code result__c;
  CAMLparam3(session,type,name);
  session__c = unwrap_gnutls_session_t(session);
  type__c = unwrap_gnutls_server_name_type_t(type);
  name__c = String_val(name);
  name_length__c = caml_string_length(name);
  nettls_init();
  result__c = gnutls_server_name_set(session__c,type__c,name__c,name_length__c);
  net_gnutls_error_check(result__c);
  CAMLreturn(Val_unit);
}

value net_gnutls_server_name_get(value session,value indx) {
  gnutls_session_t session__c;
  void * data__c;
  size_t data_length__c;
  gnutls_server_name_type_t ty__c;
  unsigned int indx__c;
  error_code result__c;
  CAMLparam2(session,indx);
  CAMLlocal3(data,ty,local_0);
  session__c = unwrap_gnutls_session_t(session);
  indx__c = uint_val(indx);
  nettls_init();
  data__c = NULL;
  data_length__c = 0;
  data = caml_alloc_string(0);
  result__c = gnutls_server_name_get(session__c,data__c,&data_length__c,&ty__c,indx__c);
  if (result__c == 0 || result__c == GNUTLS_E_SHORT_MEMORY_BUFFER) {
    long n__stub;
    data_length__c++;
    n__stub = data_length__c;
    data__c = stat_alloc(data_length__c+1);
    result__c = gnutls_server_name_get(session__c,data__c,&data_length__c,&ty__c,indx__c);
    if (result__c == 0) {
      ((char *) data__c)[n__stub] = 0;
      data = caml_copy_string(data__c);
    };
    stat_free(data__c);
  };
  net_gnutls_error_check(result__c);
  ty = wrap_gnutls_server_name_type_t(ty__c);
  local_0 = caml_alloc(2,0);
  Field(local_0, 0) = data;
  Field(local_0, 1) = ty;
  CAMLreturn(local_0);
}

value net_gnutls_safe_renegotiation_status(value session) {
#ifdef HAVE_FUN_gnutls_safe_renegotiation_status
  gnutls_session_t session__c;
  int result__c;
  CAMLparam1(session);
  CAMLlocal1(result);
  session__c = unwrap_gnutls_session_t(session);
  nettls_init();
  result__c = gnutls_safe_renegotiation_status(session__c);
  result = Val_bool(result__c);
  CAMLreturn(result);
#else
  invalid_argument("gnutls_safe_renegotiation_status");
#endif
}

value net_gnutls_supplemental_get_name(value ty) {
  gnutls_supplemental_data_format_type_t ty__c;
  const char * result__c;
  CAMLparam1(ty);
  CAMLlocal1(result);
  ty__c = unwrap_gnutls_supplemental_data_format_type_t(ty);
  nettls_init();
  result__c = gnutls_supplemental_get_name(ty__c);
  result = protected_copy_string(result__c);
  CAMLreturn(result);
}

value net_gnutls_credentials_clear(value session) {
  gnutls_session_t session__c;
  CAMLparam1(session);
  session__c = unwrap_gnutls_session_t(session);
  nettls_init();
  gnutls_credentials_clear(session__c);
  CAMLreturn(Val_unit);
}

value net_gnutls_key_generate(value key_size) {
#ifdef HAVE_FUN_gnutls_key_generate
  str_datum key__c;
  unsigned int key_size__c;
  error_code result__c;
  CAMLparam1(key_size);
  CAMLlocal1(key);
  key_size__c = uint_val(key_size);
  nettls_init();
  result__c = gnutls_key_generate(&key__c,key_size__c);
  net_gnutls_error_check(result__c);
  key = wrap_str_datum(key__c);
  CAMLreturn(key);
#else
  invalid_argument("gnutls_key_generate");
#endif
}

value net_gnutls_fingerprint(value algo,value data) {
  gnutls_digest_algorithm_t algo__c;
  str_datum_p data__c;
  void * output_data__c;
  size_t output_data_size__c;
  error_code result__c;
  CAMLparam2(algo,data);
  CAMLlocal1(output_data);
  algo__c = unwrap_gnutls_digest_algorithm_t(algo);
  data__c = unwrap_str_datum_p(data);
  nettls_init();
  output_data__c = NULL;
  output_data_size__c = 0;
  output_data = caml_alloc_string(0);
  result__c = gnutls_fingerprint(algo__c,data__c,output_data__c,&output_data_size__c);
  if (result__c == 0 || result__c == GNUTLS_E_SHORT_MEMORY_BUFFER) {
    output_data = caml_alloc_string(output_data_size__c);
    output_data__c = String_val(output_data);
    result__c = gnutls_fingerprint(algo__c,data__c,output_data__c,&output_data_size__c);
  };
  free_str_datum_p(data__c);
  net_gnutls_error_check(result__c);
  CAMLreturn(output_data);
}

value net_gnutls_certificate_allocate_credentials(value dummy) {
  gnutls_certificate_credentials_t sc__c;
  error_code result__c;
  CAMLparam1(dummy);
  CAMLlocal1(sc);
  nettls_init();
  result__c = gnutls_certificate_allocate_credentials(&sc__c);
  net_gnutls_error_check(result__c);
  sc = wrap_gnutls_certificate_credentials_t(sc__c);
  CAMLreturn(sc);
}

value net_gnutls_certificate_set_x509_system_trust(value cred) {
#ifdef HAVE_FUN_gnutls_certificate_set_x509_system_trust
  gnutls_certificate_credentials_t cred__c;
  error_code result__c;
  CAMLparam1(cred);
  cred__c = unwrap_gnutls_certificate_credentials_t(cred);
  nettls_init();
  result__c = gnutls_certificate_set_x509_system_trust(cred__c);
  net_gnutls_error_check(result__c);
  CAMLreturn(Val_unit);
#else
  invalid_argument("gnutls_certificate_set_x509_system_trust");
#endif
}

value net_gnutls_certificate_set_x509_trust_file(value cred,value cafile,value type) {
  gnutls_certificate_credentials_t cred__c;
  char * cafile__c;
  gnutls_x509_crt_fmt_t type__c;
  error_code result__c;
  CAMLparam3(cred,cafile,type);
  cred__c = unwrap_gnutls_certificate_credentials_t(cred);
  cafile__c = String_val(cafile);
  type__c = unwrap_gnutls_x509_crt_fmt_t(type);
  nettls_init();
  result__c = gnutls_certificate_set_x509_trust_file(cred__c,cafile__c,type__c);
  net_gnutls_error_check(result__c);
  CAMLreturn(Val_unit);
}

value net_gnutls_certificate_set_x509_trust_mem(value res,value ca,value type) {
  gnutls_certificate_credentials_t res__c;
  str_datum_p ca__c;
  gnutls_x509_crt_fmt_t type__c;
  error_code result__c;
  CAMLparam3(res,ca,type);
  res__c = unwrap_gnutls_certificate_credentials_t(res);
  ca__c = unwrap_str_datum_p(ca);
  type__c = unwrap_gnutls_x509_crt_fmt_t(type);
  nettls_init();
  result__c = gnutls_certificate_set_x509_trust_mem(res__c,ca__c,type__c);
  free_str_datum_p(ca__c);
  net_gnutls_error_check(result__c);
  CAMLreturn(Val_unit);
}

value net_gnutls_certificate_set_x509_crl_file(value cred,value crlfile,value type) {
  gnutls_certificate_credentials_t cred__c;
  char * crlfile__c;
  gnutls_x509_crt_fmt_t type__c;
  error_code result__c;
  CAMLparam3(cred,crlfile,type);
  cred__c = unwrap_gnutls_certificate_credentials_t(cred);
  crlfile__c = String_val(crlfile);
  type__c = unwrap_gnutls_x509_crt_fmt_t(type);
  nettls_init();
  result__c = gnutls_certificate_set_x509_crl_file(cred__c,crlfile__c,type__c);
  net_gnutls_error_check(result__c);
  CAMLreturn(Val_unit);
}

value net_gnutls_certificate_set_x509_crl_mem(value res,value crl,value type) {
  gnutls_certificate_credentials_t res__c;
  str_datum_p crl__c;
  gnutls_x509_crt_fmt_t type__c;
  error_code result__c;
  CAMLparam3(res,crl,type);
  res__c = unwrap_gnutls_certificate_credentials_t(res);
  crl__c = unwrap_str_datum_p(crl);
  type__c = unwrap_gnutls_x509_crt_fmt_t(type);
  nettls_init();
  result__c = gnutls_certificate_set_x509_crl_mem(res__c,crl__c,type__c);
  free_str_datum_p(crl__c);
  net_gnutls_error_check(result__c);
  CAMLreturn(Val_unit);
}

value net_gnutls_certificate_set_x509_key_file(value cred,value certfile,value keyfile,value type) {
  gnutls_certificate_credentials_t cred__c;
  char * certfile__c;
  char * keyfile__c;
  gnutls_x509_crt_fmt_t type__c;
  error_code result__c;
  CAMLparam4(cred,certfile,keyfile,type);
  cred__c = unwrap_gnutls_certificate_credentials_t(cred);
  certfile__c = String_val(certfile);
  keyfile__c = String_val(keyfile);
  type__c = unwrap_gnutls_x509_crt_fmt_t(type);
  nettls_init();
  result__c = gnutls_certificate_set_x509_key_file(cred__c,certfile__c,keyfile__c,type__c);
  net_gnutls_error_check(result__c);
  CAMLreturn(Val_unit);
}

value net_gnutls_certificate_set_x509_key_mem(value res,value cert,value key,value type) {
  gnutls_certificate_credentials_t res__c;
  str_datum_p cert__c;
  str_datum_p key__c;
  gnutls_x509_crt_fmt_t type__c;
  error_code result__c;
  CAMLparam4(res,cert,key,type);
  res__c = unwrap_gnutls_certificate_credentials_t(res);
  cert__c = unwrap_str_datum_p(cert);
  key__c = unwrap_str_datum_p(key);
  type__c = unwrap_gnutls_x509_crt_fmt_t(type);
  nettls_init();
  result__c = gnutls_certificate_set_x509_key_mem(res__c,cert__c,key__c,type__c);
  free_str_datum_p(cert__c);
  free_str_datum_p(key__c);
  net_gnutls_error_check(result__c);
  CAMLreturn(Val_unit);
}

value net_gnutls_certificate_set_x509_simple_pkcs12_file(value res,value pkcs12file,value type,value password) {
  gnutls_certificate_credentials_t res__c;
  char * pkcs12file__c;
  gnutls_x509_crt_fmt_t type__c;
  char * password__c;
  error_code result__c;
  CAMLparam4(res,pkcs12file,type,password);
  res__c = unwrap_gnutls_certificate_credentials_t(res);
  pkcs12file__c = String_val(pkcs12file);
  type__c = unwrap_gnutls_x509_crt_fmt_t(type);
  password__c = String_val(password);
  nettls_init();
  result__c = gnutls_certificate_set_x509_simple_pkcs12_file(res__c,pkcs12file__c,type__c,password__c);
  net_gnutls_error_check(result__c);
  CAMLreturn(Val_unit);
}

value net_gnutls_certificate_set_x509_simple_pkcs12_mem(value res,value p12blob,value type,value password) {
  gnutls_certificate_credentials_t res__c;
  str_datum_p p12blob__c;
  gnutls_x509_crt_fmt_t type__c;
  char * password__c;
  error_code result__c;
  CAMLparam4(res,p12blob,type,password);
  res__c = unwrap_gnutls_certificate_credentials_t(res);
  p12blob__c = unwrap_str_datum_p(p12blob);
  type__c = unwrap_gnutls_x509_crt_fmt_t(type);
  password__c = String_val(password);
  nettls_init();
  result__c = gnutls_certificate_set_x509_simple_pkcs12_mem(res__c,p12blob__c,type__c,password__c);
  free_str_datum_p(p12blob__c);
  net_gnutls_error_check(result__c);
  CAMLreturn(Val_unit);
}

value net_gnutls_certificate_set_x509_key(value res,value cert_list,value key) {
  gnutls_certificate_credentials_t res__c;
  gnutls_x509_crt_t * cert_list__c;
  long local_0;
  size_t cert_list_size__c;
  gnutls_x509_privkey_t key__c;
  error_code result__c;
  CAMLparam3(res,cert_list,key);
  res__c = unwrap_gnutls_certificate_credentials_t(res);
  cert_list__c = (gnutls_x509_crt_t *) stat_alloc(Wosize_val(cert_list)*sizeof(gnutls_x509_crt_t));
  for (local_0=0; local_0 < Wosize_val(cert_list); local_0++) {
    cert_list__c[local_0] = unwrap_gnutls_x509_crt_t(Field(cert_list,local_0));
  };
  cert_list_size__c = (size_t) Wosize_val(cert_list);
  key__c = unwrap_gnutls_x509_privkey_t(key);
  nettls_init();
  result__c = gnutls_certificate_set_x509_key(res__c,cert_list__c,cert_list_size__c,key__c);
  stat_free(cert_list__c);
  net_gnutls_error_check(result__c);
  CAMLreturn(Val_unit);
}

value net_gnutls_certificate_set_x509_trust(value res,value ca_list) {
  gnutls_certificate_credentials_t res__c;
  gnutls_x509_crt_t * ca_list__c;
  long local_0;
  size_t ca_list_size__c;
  error_code result__c;
  CAMLparam2(res,ca_list);
  CAMLlocal1(result);
  res__c = unwrap_gnutls_certificate_credentials_t(res);
  ca_list__c = (gnutls_x509_crt_t *) stat_alloc(Wosize_val(ca_list)*sizeof(gnutls_x509_crt_t));
  for (local_0=0; local_0 < Wosize_val(ca_list); local_0++) {
    ca_list__c[local_0] = unwrap_gnutls_x509_crt_t(Field(ca_list,local_0));
  };
  ca_list_size__c = (size_t) Wosize_val(ca_list);
  nettls_init();
  result__c = gnutls_certificate_set_x509_trust(res__c,ca_list__c,ca_list_size__c);
  stat_free(ca_list__c);
  net_gnutls_error_check(result__c);
  result = Val_int(result__c);
  CAMLreturn(result);
}

value net_gnutls_certificate_set_x509_crl(value res,value crl_list) {
  gnutls_certificate_credentials_t res__c;
  gnutls_x509_crl_t * crl_list__c;
  long local_0;
  size_t crl_list_size__c;
  error_code result__c;
  CAMLparam2(res,crl_list);
  CAMLlocal1(result);
  res__c = unwrap_gnutls_certificate_credentials_t(res);
  crl_list__c = (gnutls_x509_crl_t *) stat_alloc(Wosize_val(crl_list)*sizeof(gnutls_x509_crl_t));
  for (local_0=0; local_0 < Wosize_val(crl_list); local_0++) {
    crl_list__c[local_0] = unwrap_gnutls_x509_crl_t(Field(crl_list,local_0));
  };
  crl_list_size__c = (size_t) Wosize_val(crl_list);
  nettls_init();
  result__c = gnutls_certificate_set_x509_crl(res__c,crl_list__c,crl_list_size__c);
  stat_free(crl_list__c);
  net_gnutls_error_check(result__c);
  result = Val_int(result__c);
  CAMLreturn(result);
}

value net_gnutls_anon_allocate_server_credentials(value dummy) {
  gnutls_anon_server_credentials_t sc__c;
  error_code result__c;
  CAMLparam1(dummy);
  CAMLlocal1(sc);
  nettls_init();
  result__c = gnutls_anon_allocate_server_credentials(&sc__c);
  net_gnutls_error_check(result__c);
  sc = wrap_gnutls_anon_server_credentials_t(sc__c);
  CAMLreturn(sc);
}

value net_gnutls_anon_allocate_client_credentials(value dummy) {
  gnutls_anon_client_credentials_t sc__c;
  error_code result__c;
  CAMLparam1(dummy);
  CAMLlocal1(sc);
  nettls_init();
  result__c = gnutls_anon_allocate_client_credentials(&sc__c);
  net_gnutls_error_check(result__c);
  sc = wrap_gnutls_anon_client_credentials_t(sc__c);
  CAMLreturn(sc);
}

value net_gnutls_dh_params_init(value dummy) {
  gnutls_dh_params_t dh_params__c;
  error_code result__c;
  CAMLparam1(dummy);
  CAMLlocal1(dh_params);
  nettls_init();
  result__c = gnutls_dh_params_init(&dh_params__c);
  net_gnutls_error_check(result__c);
  dh_params = wrap_gnutls_dh_params_t(dh_params__c);
  CAMLreturn(dh_params);
}

value net_gnutls_dh_params_import_raw(value dh_params,value prime,value generator) {
  gnutls_dh_params_t dh_params__c;
  str_datum_p prime__c;
  str_datum_p generator__c;
  error_code result__c;
  CAMLparam3(dh_params,prime,generator);
  dh_params__c = unwrap_gnutls_dh_params_t(dh_params);
  prime__c = unwrap_str_datum_p(prime);
  generator__c = unwrap_str_datum_p(generator);
  nettls_init();
  result__c = gnutls_dh_params_import_raw(dh_params__c,prime__c,generator__c);
  free_str_datum_p(prime__c);
  free_str_datum_p(generator__c);
  net_gnutls_error_check(result__c);
  CAMLreturn(Val_unit);
}

value net_gnutls_dh_params_import_pkcs3(value params,value pkcs3_params,value format) {
  gnutls_dh_params_t params__c;
  str_datum_p pkcs3_params__c;
  gnutls_x509_crt_fmt_t format__c;
  error_code result__c;
  CAMLparam3(params,pkcs3_params,format);
  params__c = unwrap_gnutls_dh_params_t(params);
  pkcs3_params__c = unwrap_str_datum_p(pkcs3_params);
  format__c = unwrap_gnutls_x509_crt_fmt_t(format);
  nettls_init();
  result__c = gnutls_dh_params_import_pkcs3(params__c,pkcs3_params__c,format__c);
  free_str_datum_p(pkcs3_params__c);
  net_gnutls_error_check(result__c);
  CAMLreturn(Val_unit);
}

value net_gnutls_dh_params_generate2(value params,value bits) {
  gnutls_dh_params_t params__c;
  unsigned int bits__c;
  error_code result__c;
  CAMLparam2(params,bits);
  params__c = unwrap_gnutls_dh_params_t(params);
  bits__c = uint_val(bits);
  nettls_init();
  result__c = gnutls_dh_params_generate2(params__c,bits__c);
  net_gnutls_error_check(result__c);
  CAMLreturn(Val_unit);
}

value net_gnutls_dh_params_cpy(value dst,value src) {
  gnutls_dh_params_t dst__c;
  gnutls_dh_params_t src__c;
  error_code result__c;
  CAMLparam2(dst,src);
  dst__c = unwrap_gnutls_dh_params_t(dst);
  src__c = unwrap_gnutls_dh_params_t(src);
  nettls_init();
  result__c = gnutls_dh_params_cpy(dst__c,src__c);
  net_gnutls_error_check(result__c);
  CAMLreturn(Val_unit);
}

value net_gnutls_certificate_set_dh_params(value res,value dh_params) {
  gnutls_certificate_credentials_t res__c;
  gnutls_dh_params_t dh_params__c;
  CAMLparam2(res,dh_params);
  res__c = unwrap_gnutls_certificate_credentials_t(res);
  dh_params__c = unwrap_gnutls_dh_params_t(dh_params);
  nettls_init();
  gnutls_certificate_set_dh_params(res__c,dh_params__c);
  CAMLreturn(Val_unit);
}

value net_gnutls_anon_set_server_dh_params(value res,value dh_params) {
  gnutls_anon_server_credentials_t res__c;
  gnutls_dh_params_t dh_params__c;
  CAMLparam2(res,dh_params);
  res__c = unwrap_gnutls_anon_server_credentials_t(res);
  dh_params__c = unwrap_gnutls_dh_params_t(dh_params);
  nettls_init();
  gnutls_anon_set_server_dh_params(res__c,dh_params__c);
  CAMLreturn(Val_unit);
}

value net_gnutls_psk_set_server_dh_params(value res,value dh_params) {
  gnutls_psk_server_credentials_t res__c;
  gnutls_dh_params_t dh_params__c;
  CAMLparam2(res,dh_params);
  res__c = unwrap_gnutls_psk_server_credentials_t(res);
  dh_params__c = unwrap_gnutls_dh_params_t(dh_params);
  nettls_init();
  gnutls_psk_set_server_dh_params(res__c,dh_params__c);
  CAMLreturn(Val_unit);
}

value net_gnutls_dh_set_prime_bits(value session,value bits) {
  gnutls_session_t session__c;
  unsigned int bits__c;
  CAMLparam2(session,bits);
  session__c = unwrap_gnutls_session_t(session);
  bits__c = uint_val(bits);
  nettls_init();
  gnutls_dh_set_prime_bits(session__c,bits__c);
  CAMLreturn(Val_unit);
}

value net_gnutls_dh_get_secret_bits(value session) {
  gnutls_session_t session__c;
  int result__c;
  CAMLparam1(session);
  CAMLlocal1(result);
  session__c = unwrap_gnutls_session_t(session);
  nettls_init();
  result__c = gnutls_dh_get_secret_bits(session__c);
  result = Val_int(result__c);
  CAMLreturn(result);
}

value net_gnutls_dh_get_peers_public_bits(value session) {
  gnutls_session_t session__c;
  int result__c;
  CAMLparam1(session);
  CAMLlocal1(result);
  session__c = unwrap_gnutls_session_t(session);
  nettls_init();
  result__c = gnutls_dh_get_peers_public_bits(session__c);
  result = Val_int(result__c);
  CAMLreturn(result);
}

value net_gnutls_dh_get_prime_bits(value session) {
  gnutls_session_t session__c;
  int result__c;
  CAMLparam1(session);
  CAMLlocal1(result);
  session__c = unwrap_gnutls_session_t(session);
  nettls_init();
  result__c = gnutls_dh_get_prime_bits(session__c);
  result = Val_int(result__c);
  CAMLreturn(result);
}

value net_gnutls_dh_get_group(value session) {
  gnutls_session_t session__c;
  str_datum raw_gen__c;
  str_datum raw_prime__c;
  error_code result__c;
  CAMLparam1(session);
  CAMLlocal3(raw_gen,raw_prime,local_0);
  session__c = unwrap_gnutls_session_t(session);
  nettls_init();
  result__c = gnutls_dh_get_group(session__c,&raw_gen__c,&raw_prime__c);
  net_gnutls_error_check(result__c);
  raw_gen = wrap_str_datum(raw_gen__c);
  raw_prime = wrap_str_datum(raw_prime__c);
  local_0 = caml_alloc(2,0);
  Field(local_0, 0) = raw_gen;
  Field(local_0, 1) = raw_prime;
  CAMLreturn(local_0);
}

value net_gnutls_dh_get_pubkey(value session) {
  gnutls_session_t session__c;
  str_datum raw_key__c;
  error_code result__c;
  CAMLparam1(session);
  CAMLlocal1(raw_key);
  session__c = unwrap_gnutls_session_t(session);
  nettls_init();
  result__c = gnutls_dh_get_pubkey(session__c,&raw_key__c);
  net_gnutls_error_check(result__c);
  raw_key = wrap_str_datum(raw_key__c);
  CAMLreturn(raw_key);
}

value net_gnutls_auth_get_type(value session) {
  gnutls_session_t session__c;
  gnutls_credentials_type_t result__c;
  CAMLparam1(session);
  CAMLlocal1(result);
  session__c = unwrap_gnutls_session_t(session);
  nettls_init();
  result__c = gnutls_auth_get_type(session__c);
  result = wrap_gnutls_credentials_type_t(result__c);
  CAMLreturn(result);
}

value net_gnutls_auth_server_get_type(value session) {
  gnutls_session_t session__c;
  gnutls_credentials_type_t result__c;
  CAMLparam1(session);
  CAMLlocal1(result);
  session__c = unwrap_gnutls_session_t(session);
  nettls_init();
  result__c = gnutls_auth_server_get_type(session__c);
  result = wrap_gnutls_credentials_type_t(result__c);
  CAMLreturn(result);
}

value net_gnutls_auth_client_get_type(value session) {
  gnutls_session_t session__c;
  gnutls_credentials_type_t result__c;
  CAMLparam1(session);
  CAMLlocal1(result);
  session__c = unwrap_gnutls_session_t(session);
  nettls_init();
  result__c = gnutls_auth_client_get_type(session__c);
  result = wrap_gnutls_credentials_type_t(result__c);
  CAMLreturn(result);
}

value net_gnutls_certificate_get_ours(value session) {
  gnutls_session_t session__c;
  const_str_datum_p result__c;
  CAMLparam1(session);
  CAMLlocal1(result);
  session__c = unwrap_gnutls_session_t(session);
  nettls_init();
  result__c = gnutls_certificate_get_ours(session__c);
  result = wrap_const_str_datum_p(result__c);
  CAMLreturn(result);
}

value net_gnutls_certificate_activation_time_peers(value session) {
  gnutls_session_t session__c;
  double result__c;
  CAMLparam1(session);
  CAMLlocal1(result);
  session__c = unwrap_gnutls_session_t(session);
  nettls_init();
  result__c = gnutls_certificate_activation_time_peers(session__c);
  result = caml_copy_double(result__c);
  CAMLreturn(result);
}

value net_gnutls_certificate_expiration_time_peers(value session) {
  gnutls_session_t session__c;
  double result__c;
  CAMLparam1(session);
  CAMLlocal1(result);
  session__c = unwrap_gnutls_session_t(session);
  nettls_init();
  result__c = gnutls_certificate_expiration_time_peers(session__c);
  result = caml_copy_double(result__c);
  CAMLreturn(result);
}

value net_gnutls_certificate_client_get_request_status(value session) {
  gnutls_session_t session__c;
  error_code result__c;
  CAMLparam1(session);
  CAMLlocal1(result);
  session__c = unwrap_gnutls_session_t(session);
  nettls_init();
  result__c = gnutls_certificate_client_get_request_status(session__c);
  net_gnutls_error_check(result__c);
  result = Val_bool(result__c);
  CAMLreturn(result);
}

value net_gnutls_certificate_get_issuer(value sc,value cert,value flags) {
#ifdef HAVE_FUN_gnutls_certificate_get_issuer
  gnutls_certificate_credentials_t sc__c;
  gnutls_x509_crt_t cert__c;
  gnutls_x509_crt_t issuer__c;
  empty_flags flags__c;
  error_code result__c;
  CAMLparam3(sc,cert,flags);
  CAMLlocal1(issuer);
  sc__c = unwrap_gnutls_certificate_credentials_t(sc);
  cert__c = unwrap_gnutls_x509_crt_t(cert);
  flags__c = unwrap_empty_flags(flags);
  nettls_init();
  result__c = gnutls_certificate_get_issuer(sc__c,cert__c,&issuer__c,flags__c);
  net_gnutls_error_check(result__c);
  issuer = wrap_gnutls_x509_crt_t(issuer__c);
  CAMLreturn(issuer);
#else
  invalid_argument("gnutls_certificate_get_issuer");
#endif
}

value net_gnutls_certificate_set_verify_flags(value res,value flags) {
  gnutls_certificate_credentials_t res__c;
  gnutls_certificate_verify_flags flags__c;
  CAMLparam2(res,flags);
  res__c = unwrap_gnutls_certificate_credentials_t(res);
  flags__c = unwrap_gnutls_certificate_verify_flags(flags);
  nettls_init();
  gnutls_certificate_set_verify_flags(res__c,flags__c);
  CAMLreturn(Val_unit);
}

value net_gnutls_certificate_set_verify_limits(value res,value max_bits,value max_depth) {
  gnutls_certificate_credentials_t res__c;
  unsigned int max_bits__c;
  unsigned int max_depth__c;
  CAMLparam3(res,max_bits,max_depth);
  res__c = unwrap_gnutls_certificate_credentials_t(res);
  max_bits__c = uint_val(max_bits);
  max_depth__c = uint_val(max_depth);
  nettls_init();
  gnutls_certificate_set_verify_limits(res__c,max_bits__c,max_depth__c);
  CAMLreturn(Val_unit);
}

value net_gnutls_srp_allocate_client_credentials(value dummy) {
#ifdef HAVE_FUN_gnutls_srp_allocate_client_credentials
  gnutls_srp_client_credentials_t sc__c;
  error_code result__c;
  CAMLparam1(dummy);
  CAMLlocal1(sc);
  nettls_init();
  result__c = gnutls_srp_allocate_client_credentials(&sc__c);
  net_gnutls_error_check(result__c);
  sc = wrap_gnutls_srp_client_credentials_t(sc__c);
  CAMLreturn(sc);
#else
  invalid_argument("gnutls_srp_allocate_client_credentials");
#endif
}

value net_gnutls_srp_set_client_credentials(value res,value username,value password) {
#ifdef HAVE_FUN_gnutls_srp_set_client_credentials
  gnutls_srp_client_credentials_t res__c;
  char * username__c;
  char * password__c;
  error_code result__c;
  CAMLparam3(res,username,password);
  res__c = unwrap_gnutls_srp_client_credentials_t(res);
  username__c = String_val(username);
  password__c = String_val(password);
  nettls_init();
  result__c = gnutls_srp_set_client_credentials(res__c,username__c,password__c);
  net_gnutls_error_check(result__c);
  CAMLreturn(Val_unit);
#else
  invalid_argument("gnutls_srp_set_client_credentials");
#endif
}

value net_gnutls_srp_allocate_server_credentials(value dummy) {
#ifdef HAVE_FUN_gnutls_srp_allocate_server_credentials
  gnutls_srp_server_credentials_t sc__c;
  error_code result__c;
  CAMLparam1(dummy);
  CAMLlocal1(sc);
  nettls_init();
  result__c = gnutls_srp_allocate_server_credentials(&sc__c);
  net_gnutls_error_check(result__c);
  sc = wrap_gnutls_srp_server_credentials_t(sc__c);
  CAMLreturn(sc);
#else
  invalid_argument("gnutls_srp_allocate_server_credentials");
#endif
}

value net_gnutls_srp_set_server_credentials_file(value res,value password_file,value password_conf_file) {
#ifdef HAVE_FUN_gnutls_srp_set_server_credentials_file
  gnutls_srp_server_credentials_t res__c;
  char * password_file__c;
  char * password_conf_file__c;
  error_code result__c;
  CAMLparam3(res,password_file,password_conf_file);
  res__c = unwrap_gnutls_srp_server_credentials_t(res);
  password_file__c = String_val(password_file);
  password_conf_file__c = String_val(password_conf_file);
  nettls_init();
  result__c = gnutls_srp_set_server_credentials_file(res__c,password_file__c,password_conf_file__c);
  net_gnutls_error_check(result__c);
  CAMLreturn(Val_unit);
#else
  invalid_argument("gnutls_srp_set_server_credentials_file");
#endif
}

value net_gnutls_srp_server_get_username(value session) {
#ifdef HAVE_FUN_gnutls_srp_server_get_username
  gnutls_session_t session__c;
  const char * result__c;
  CAMLparam1(session);
  CAMLlocal1(result);
  session__c = unwrap_gnutls_session_t(session);
  nettls_init();
  result__c = gnutls_srp_server_get_username(session__c);
  result = protected_copy_string(result__c);
  CAMLreturn(result);
#else
  invalid_argument("gnutls_srp_server_get_username");
#endif
}

value net_gnutls_srp_set_prime_bits(value session,value bits) {
#ifdef HAVE_FUN_gnutls_srp_set_prime_bits
  gnutls_session_t session__c;
  unsigned int bits__c;
  CAMLparam2(session,bits);
  session__c = unwrap_gnutls_session_t(session);
  bits__c = uint_val(bits);
  nettls_init();
  gnutls_srp_set_prime_bits(session__c,bits__c);
  CAMLreturn(Val_unit);
#else
  invalid_argument("gnutls_srp_set_prime_bits");
#endif
}

value net_gnutls_srp_verifier(value username,value password,value salt,value generator,value prime) {
#ifdef HAVE_FUN_gnutls_srp_verifier
  char * username__c;
  char * password__c;
  str_datum_p salt__c;
  str_datum_p generator__c;
  str_datum_p prime__c;
  str_datum res__c;
  error_code result__c;
  CAMLparam5(username,password,salt,generator,prime);
  CAMLlocal1(res);
  username__c = String_val(username);
  password__c = String_val(password);
  salt__c = unwrap_str_datum_p(salt);
  generator__c = unwrap_str_datum_p(generator);
  prime__c = unwrap_str_datum_p(prime);
  nettls_init();
  result__c = gnutls_srp_verifier(username__c,password__c,salt__c,generator__c,prime__c,&res__c);
  free_str_datum_p(salt__c);
  free_str_datum_p(generator__c);
  free_str_datum_p(prime__c);
  net_gnutls_error_check(result__c);
  res = wrap_str_datum(res__c);
  CAMLreturn(res);
#else
  invalid_argument("gnutls_srp_verifier");
#endif
}

value net_gnutls_psk_allocate_client_credentials(value dummy) {
  gnutls_psk_client_credentials_t sc__c;
  error_code result__c;
  CAMLparam1(dummy);
  CAMLlocal1(sc);
  nettls_init();
  result__c = gnutls_psk_allocate_client_credentials(&sc__c);
  net_gnutls_error_check(result__c);
  sc = wrap_gnutls_psk_client_credentials_t(sc__c);
  CAMLreturn(sc);
}

value net_gnutls_psk_set_client_credentials(value res,value username,value key,value flags) {
  gnutls_psk_client_credentials_t res__c;
  char * username__c;
  str_datum_p key__c;
  gnutls_psk_key_flags flags__c;
  error_code result__c;
  CAMLparam4(res,username,key,flags);
  res__c = unwrap_gnutls_psk_client_credentials_t(res);
  username__c = String_val(username);
  key__c = unwrap_str_datum_p(key);
  flags__c = unwrap_gnutls_psk_key_flags(flags);
  nettls_init();
  result__c = gnutls_psk_set_client_credentials(res__c,username__c,key__c,flags__c);
  free_str_datum_p(key__c);
  net_gnutls_error_check(result__c);
  CAMLreturn(Val_unit);
}

value net_gnutls_psk_allocate_server_credentials(value dummy) {
  gnutls_psk_server_credentials_t sc__c;
  error_code result__c;
  CAMLparam1(dummy);
  CAMLlocal1(sc);
  nettls_init();
  result__c = gnutls_psk_allocate_server_credentials(&sc__c);
  net_gnutls_error_check(result__c);
  sc = wrap_gnutls_psk_server_credentials_t(sc__c);
  CAMLreturn(sc);
}

value net_gnutls_psk_set_server_credentials_file(value res,value password_file) {
  gnutls_psk_server_credentials_t res__c;
  char * password_file__c;
  error_code result__c;
  CAMLparam2(res,password_file);
  res__c = unwrap_gnutls_psk_server_credentials_t(res);
  password_file__c = String_val(password_file);
  nettls_init();
  result__c = gnutls_psk_set_server_credentials_file(res__c,password_file__c);
  net_gnutls_error_check(result__c);
  CAMLreturn(Val_unit);
}

value net_gnutls_psk_set_server_credentials_hint(value res,value hint) {
  gnutls_psk_server_credentials_t res__c;
  char * hint__c;
  error_code result__c;
  CAMLparam2(res,hint);
  res__c = unwrap_gnutls_psk_server_credentials_t(res);
  hint__c = String_val(hint);
  nettls_init();
  result__c = gnutls_psk_set_server_credentials_hint(res__c,hint__c);
  net_gnutls_error_check(result__c);
  CAMLreturn(Val_unit);
}

value net_gnutls_psk_server_get_username(value session) {
  gnutls_session_t session__c;
  const char * result__c;
  CAMLparam1(session);
  CAMLlocal1(result);
  session__c = unwrap_gnutls_session_t(session);
  nettls_init();
  result__c = gnutls_psk_server_get_username(session__c);
  result = protected_copy_string(result__c);
  CAMLreturn(result);
}

value net_gnutls_psk_client_get_hint(value session) {
  gnutls_session_t session__c;
  const char * result__c;
  CAMLparam1(session);
  CAMLlocal1(result);
  session__c = unwrap_gnutls_session_t(session);
  nettls_init();
  result__c = gnutls_psk_client_get_hint(session__c);
  result = protected_copy_string(result__c);
  CAMLreturn(result);
}

value net_gnutls_x509_crt_init(value dummy) {
  gnutls_x509_crt_t cert__c;
  error_code result__c;
  CAMLparam1(dummy);
  CAMLlocal1(cert);
  nettls_init();
  result__c = gnutls_x509_crt_init(&cert__c);
  net_gnutls_error_check(result__c);
  cert = wrap_gnutls_x509_crt_t(cert__c);
  CAMLreturn(cert);
}

value net_gnutls_x509_crt_import(value cert,value data,value format) {
  gnutls_x509_crt_t cert__c;
  const_str_datum_p data__c;
  gnutls_x509_crt_fmt_t format__c;
  error_code result__c;
  CAMLparam3(cert,data,format);
  cert__c = unwrap_gnutls_x509_crt_t(cert);
  data__c = unwrap_const_str_datum_p(data);
  format__c = unwrap_gnutls_x509_crt_fmt_t(format);
  nettls_init();
  result__c = gnutls_x509_crt_import(cert__c,data__c,format__c);
  net_gnutls_error_check(result__c);
  CAMLreturn(Val_unit);
}

value net_gnutls_x509_crt_export(value cert,value format) {
  gnutls_x509_crt_t cert__c;
  gnutls_x509_crt_fmt_t format__c;
  void * output_data__c;
  size_t output_data_size__c;
  error_code result__c;
  CAMLparam2(cert,format);
  CAMLlocal1(output_data);
  cert__c = unwrap_gnutls_x509_crt_t(cert);
  format__c = unwrap_gnutls_x509_crt_fmt_t(format);
  nettls_init();
  output_data__c = NULL;
  output_data_size__c = 0;
  output_data = caml_alloc_string(0);
  result__c = gnutls_x509_crt_export(cert__c,format__c,output_data__c,&output_data_size__c);
  if (result__c == 0 || result__c == GNUTLS_E_SHORT_MEMORY_BUFFER) {
    output_data = caml_alloc_string(output_data_size__c);
    output_data__c = String_val(output_data);
    result__c = gnutls_x509_crt_export(cert__c,format__c,output_data__c,&output_data_size__c);
  };
  net_gnutls_error_check(result__c);
  CAMLreturn(output_data);
}

value net_gnutls_x509_crt_get_issuer_dn(value cert) {
  gnutls_x509_crt_t cert__c;
  void * output_data__c;
  size_t output_data_size__c;
  error_code result__c;
  CAMLparam1(cert);
  CAMLlocal1(output_data);
  cert__c = unwrap_gnutls_x509_crt_t(cert);
  nettls_init();
  output_data__c = NULL;
  output_data_size__c = 0;
  output_data = caml_alloc_string(0);
  result__c = gnutls_x509_crt_get_issuer_dn(cert__c,output_data__c,&output_data_size__c);
  if (result__c == 0 || result__c == GNUTLS_E_SHORT_MEMORY_BUFFER) {
    long n__stub;
    output_data_size__c++;
    n__stub = output_data_size__c;
    output_data__c = stat_alloc(output_data_size__c+1);
    result__c = gnutls_x509_crt_get_issuer_dn(cert__c,output_data__c,&output_data_size__c);
    if (result__c == 0) {
      ((char *) output_data__c)[n__stub] = 0;
      output_data = caml_copy_string(output_data__c);
    };
    stat_free(output_data__c);
  };
  net_gnutls_error_check(result__c);
  CAMLreturn(output_data);
}

value net_gnutls_x509_crt_get_issuer_dn_oid(value cert,value indx) {
  gnutls_x509_crt_t cert__c;
  int indx__c;
  void * output_data__c;
  size_t output_data_size__c;
  error_code result__c;
  CAMLparam2(cert,indx);
  CAMLlocal1(output_data);
  cert__c = unwrap_gnutls_x509_crt_t(cert);
  indx__c = Int_val(indx);
  nettls_init();
  output_data__c = NULL;
  output_data_size__c = 0;
  output_data = caml_alloc_string(0);
  result__c = gnutls_x509_crt_get_issuer_dn_oid(cert__c,indx__c,output_data__c,&output_data_size__c);
  if (result__c == 0 || result__c == GNUTLS_E_SHORT_MEMORY_BUFFER) {
    long n__stub;
    output_data_size__c++;
    n__stub = output_data_size__c;
    output_data__c = stat_alloc(output_data_size__c+1);
    result__c = gnutls_x509_crt_get_issuer_dn_oid(cert__c,indx__c,output_data__c,&output_data_size__c);
    if (result__c == 0) {
      ((char *) output_data__c)[n__stub] = 0;
      output_data = caml_copy_string(output_data__c);
    };
    stat_free(output_data__c);
  };
  net_gnutls_error_check(result__c);
  CAMLreturn(output_data);
}

value net_gnutls_x509_crt_get_dn(value cert) {
  gnutls_x509_crt_t cert__c;
  void * output_data__c;
  size_t output_data_size__c;
  error_code result__c;
  CAMLparam1(cert);
  CAMLlocal1(output_data);
  cert__c = unwrap_gnutls_x509_crt_t(cert);
  nettls_init();
  output_data__c = NULL;
  output_data_size__c = 0;
  output_data = caml_alloc_string(0);
  result__c = gnutls_x509_crt_get_dn(cert__c,output_data__c,&output_data_size__c);
  if (result__c == 0 || result__c == GNUTLS_E_SHORT_MEMORY_BUFFER) {
    long n__stub;
    output_data_size__c++;
    n__stub = output_data_size__c;
    output_data__c = stat_alloc(output_data_size__c+1);
    result__c = gnutls_x509_crt_get_dn(cert__c,output_data__c,&output_data_size__c);
    if (result__c == 0) {
      ((char *) output_data__c)[n__stub] = 0;
      output_data = caml_copy_string(output_data__c);
    };
    stat_free(output_data__c);
  };
  net_gnutls_error_check(result__c);
  CAMLreturn(output_data);
}

value net_gnutls_x509_crt_get_dn_oid(value cert,value indx) {
  gnutls_x509_crt_t cert__c;
  int indx__c;
  void * output_data__c;
  size_t output_data_size__c;
  error_code result__c;
  CAMLparam2(cert,indx);
  CAMLlocal1(output_data);
  cert__c = unwrap_gnutls_x509_crt_t(cert);
  indx__c = Int_val(indx);
  nettls_init();
  output_data__c = NULL;
  output_data_size__c = 0;
  output_data = caml_alloc_string(0);
  result__c = gnutls_x509_crt_get_dn_oid(cert__c,indx__c,output_data__c,&output_data_size__c);
  if (result__c == 0 || result__c == GNUTLS_E_SHORT_MEMORY_BUFFER) {
    long n__stub;
    output_data_size__c++;
    n__stub = output_data_size__c;
    output_data__c = stat_alloc(output_data_size__c+1);
    result__c = gnutls_x509_crt_get_dn_oid(cert__c,indx__c,output_data__c,&output_data_size__c);
    if (result__c == 0) {
      ((char *) output_data__c)[n__stub] = 0;
      output_data = caml_copy_string(output_data__c);
    };
    stat_free(output_data__c);
  };
  net_gnutls_error_check(result__c);
  CAMLreturn(output_data);
}

value net_gnutls_x509_crt_check_hostname(value cert,value hostname) {
  gnutls_x509_crt_t cert__c;
  const char * hostname__c;
  int result__c;
  CAMLparam2(cert,hostname);
  CAMLlocal1(result);
  cert__c = unwrap_gnutls_x509_crt_t(cert);
  hostname__c = String_val(hostname);
  nettls_init();
  result__c = gnutls_x509_crt_check_hostname(cert__c,hostname__c);
  result = Val_bool(result__c);
  CAMLreturn(result);
}

value net_gnutls_x509_crt_get_signature_algorithm(value cert) {
  gnutls_x509_crt_t cert__c;
  error_code result__c;
  CAMLparam1(cert);
  CAMLlocal1(result);
  cert__c = unwrap_gnutls_x509_crt_t(cert);
  nettls_init();
  result__c = gnutls_x509_crt_get_signature_algorithm(cert__c);
  net_gnutls_error_check(result__c);
  result = wrap_gnutls_sign_algorithm_t(result__c);
  CAMLreturn(result);
}

value net_gnutls_x509_crt_get_signature(value cert) {
  gnutls_x509_crt_t cert__c;
  void * output_data__c;
  size_t output_data_size__c;
  error_code result__c;
  CAMLparam1(cert);
  CAMLlocal1(output_data);
  cert__c = unwrap_gnutls_x509_crt_t(cert);
  nettls_init();
  output_data__c = NULL;
  output_data_size__c = 0;
  output_data = caml_alloc_string(0);
  result__c = gnutls_x509_crt_get_signature(cert__c,output_data__c,&output_data_size__c);
  if (result__c == 0 || result__c == GNUTLS_E_SHORT_MEMORY_BUFFER) {
    output_data = caml_alloc_string(output_data_size__c);
    output_data__c = String_val(output_data);
    result__c = gnutls_x509_crt_get_signature(cert__c,output_data__c,&output_data_size__c);
  };
  net_gnutls_error_check(result__c);
  CAMLreturn(output_data);
}

value net_gnutls_x509_crt_get_version(value cert) {
  gnutls_x509_crt_t cert__c;
  error_code result__c;
  CAMLparam1(cert);
  CAMLlocal1(result);
  cert__c = unwrap_gnutls_x509_crt_t(cert);
  nettls_init();
  result__c = gnutls_x509_crt_get_version(cert__c);
  net_gnutls_error_check(result__c);
  result = Val_int(result__c);
  CAMLreturn(result);
}

value net_gnutls_x509_crt_get_key_id(value crt,value flags) {
  gnutls_x509_crt_t crt__c;
  empty_flags flags__c;
  void * output_data__c;
  size_t output_data_size__c;
  error_code result__c;
  CAMLparam2(crt,flags);
  CAMLlocal1(output_data);
  crt__c = unwrap_gnutls_x509_crt_t(crt);
  flags__c = unwrap_empty_flags(flags);
  nettls_init();
  output_data__c = NULL;
  output_data_size__c = 0;
  output_data = caml_alloc_string(0);
  result__c = gnutls_x509_crt_get_key_id(crt__c,flags__c,output_data__c,&output_data_size__c);
  if (result__c == 0 || result__c == GNUTLS_E_SHORT_MEMORY_BUFFER) {
    output_data = caml_alloc_string(output_data_size__c);
    output_data__c = String_val(output_data);
    result__c = gnutls_x509_crt_get_key_id(crt__c,flags__c,output_data__c,&output_data_size__c);
  };
  net_gnutls_error_check(result__c);
  CAMLreturn(output_data);
}

value net_gnutls_x509_crt_get_authority_key_id(value cert) {
  gnutls_x509_crt_t cert__c;
  void * output_data__c;
  size_t output_data_size__c;
  unsigned int critical__c;
  error_code result__c;
  CAMLparam1(cert);
  CAMLlocal3(output_data,critical,local_0);
  cert__c = unwrap_gnutls_x509_crt_t(cert);
  nettls_init();
  output_data__c = NULL;
  output_data_size__c = 0;
  output_data = caml_alloc_string(0);
  result__c = gnutls_x509_crt_get_authority_key_id(cert__c,output_data__c,&output_data_size__c,&critical__c);
  if (result__c == 0 || result__c == GNUTLS_E_SHORT_MEMORY_BUFFER) {
    output_data = caml_alloc_string(output_data_size__c);
    output_data__c = String_val(output_data);
    result__c = gnutls_x509_crt_get_authority_key_id(cert__c,output_data__c,&output_data_size__c,&critical__c);
  };
  net_gnutls_error_check(result__c);
  critical = Val_bool(critical__c);
  local_0 = caml_alloc(2,0);
  Field(local_0, 0) = output_data;
  Field(local_0, 1) = critical;
  CAMLreturn(local_0);
}

value net_gnutls_x509_crt_get_subject_key_id(value cert) {
  gnutls_x509_crt_t cert__c;
  void * output_data__c;
  size_t output_data_size__c;
  unsigned int critical__c;
  error_code result__c;
  CAMLparam1(cert);
  CAMLlocal3(output_data,critical,local_0);
  cert__c = unwrap_gnutls_x509_crt_t(cert);
  nettls_init();
  output_data__c = NULL;
  output_data_size__c = 0;
  output_data = caml_alloc_string(0);
  result__c = gnutls_x509_crt_get_subject_key_id(cert__c,output_data__c,&output_data_size__c,&critical__c);
  if (result__c == 0 || result__c == GNUTLS_E_SHORT_MEMORY_BUFFER) {
    output_data = caml_alloc_string(output_data_size__c);
    output_data__c = String_val(output_data);
    result__c = gnutls_x509_crt_get_subject_key_id(cert__c,output_data__c,&output_data_size__c,&critical__c);
  };
  net_gnutls_error_check(result__c);
  critical = Val_bool(critical__c);
  local_0 = caml_alloc(2,0);
  Field(local_0, 0) = output_data;
  Field(local_0, 1) = critical;
  CAMLreturn(local_0);
}

value net_gnutls_x509_crt_get_subject_unique_id(value crt) {
#ifdef HAVE_FUN_gnutls_x509_crt_get_subject_unique_id
  gnutls_x509_crt_t crt__c;
  void * output_data__c;
  size_t output_data_size__c;
  error_code result__c;
  CAMLparam1(crt);
  CAMLlocal1(output_data);
  crt__c = unwrap_gnutls_x509_crt_t(crt);
  nettls_init();
  output_data__c = NULL;
  output_data_size__c = 0;
  output_data = caml_alloc_string(0);
  result__c = gnutls_x509_crt_get_subject_unique_id(crt__c,output_data__c,&output_data_size__c);
  if (result__c == 0 || result__c == GNUTLS_E_SHORT_MEMORY_BUFFER) {
    output_data = caml_alloc_string(output_data_size__c);
    output_data__c = String_val(output_data);
    result__c = gnutls_x509_crt_get_subject_unique_id(crt__c,output_data__c,&output_data_size__c);
  };
  net_gnutls_error_check(result__c);
  CAMLreturn(output_data);
#else
  invalid_argument("gnutls_x509_crt_get_subject_unique_id");
#endif
}

value net_gnutls_x509_crt_get_issuer_unique_id(value crt) {
#ifdef HAVE_FUN_gnutls_x509_crt_get_issuer_unique_id
  gnutls_x509_crt_t crt__c;
  void * output_data__c;
  size_t output_data_size__c;
  error_code result__c;
  CAMLparam1(crt);
  CAMLlocal1(output_data);
  crt__c = unwrap_gnutls_x509_crt_t(crt);
  nettls_init();
  output_data__c = NULL;
  output_data_size__c = 0;
  output_data = caml_alloc_string(0);
  result__c = gnutls_x509_crt_get_issuer_unique_id(crt__c,output_data__c,&output_data_size__c);
  if (result__c == 0 || result__c == GNUTLS_E_SHORT_MEMORY_BUFFER) {
    output_data = caml_alloc_string(output_data_size__c);
    output_data__c = String_val(output_data);
    result__c = gnutls_x509_crt_get_issuer_unique_id(crt__c,output_data__c,&output_data_size__c);
  };
  net_gnutls_error_check(result__c);
  CAMLreturn(output_data);
#else
  invalid_argument("gnutls_x509_crt_get_issuer_unique_id");
#endif
}

value net_gnutls_x509_crt_get_authority_info_access(value crt,value seq,value what) {
#ifdef HAVE_FUN_gnutls_x509_crt_get_authority_info_access
  gnutls_x509_crt_t crt__c;
  unsigned int seq__c;
  gnutls_info_access_what_t what__c;
  str_datum data__c;
  unsigned int critical__c;
  error_code result__c;
  CAMLparam3(crt,seq,what);
  CAMLlocal3(data,critical,local_0);
  crt__c = unwrap_gnutls_x509_crt_t(crt);
  seq__c = uint_val(seq);
  what__c = unwrap_gnutls_info_access_what_t(what);
  nettls_init();
  result__c = gnutls_x509_crt_get_authority_info_access(crt__c,seq__c,what__c,&data__c,&critical__c);
  net_gnutls_error_check(result__c);
  data = wrap_str_datum(data__c);
  critical = Val_bool(critical__c);
  local_0 = caml_alloc(2,0);
  Field(local_0, 0) = data;
  Field(local_0, 1) = critical;
  CAMLreturn(local_0);
#else
  invalid_argument("gnutls_x509_crt_get_authority_info_access");
#endif
}

value net_gnutls_x509_crt_get_activation_time(value cert) {
  gnutls_x509_crt_t cert__c;
  double result__c;
  CAMLparam1(cert);
  CAMLlocal1(result);
  cert__c = unwrap_gnutls_x509_crt_t(cert);
  nettls_init();
  result__c = gnutls_x509_crt_get_activation_time(cert__c);
  result = caml_copy_double(result__c);
  CAMLreturn(result);
}

value net_gnutls_x509_crt_get_expiration_time(value cert) {
  gnutls_x509_crt_t cert__c;
  double result__c;
  CAMLparam1(cert);
  CAMLlocal1(result);
  cert__c = unwrap_gnutls_x509_crt_t(cert);
  nettls_init();
  result__c = gnutls_x509_crt_get_expiration_time(cert__c);
  result = caml_copy_double(result__c);
  CAMLreturn(result);
}

value net_gnutls_x509_crt_get_serial(value cert) {
  gnutls_x509_crt_t cert__c;
  void * output_data__c;
  size_t output_data_size__c;
  error_code result__c;
  CAMLparam1(cert);
  CAMLlocal1(output_data);
  cert__c = unwrap_gnutls_x509_crt_t(cert);
  nettls_init();
  output_data__c = NULL;
  output_data_size__c = 0;
  output_data = caml_alloc_string(0);
  result__c = gnutls_x509_crt_get_serial(cert__c,output_data__c,&output_data_size__c);
  if (result__c == 0 || result__c == GNUTLS_E_SHORT_MEMORY_BUFFER) {
    output_data = caml_alloc_string(output_data_size__c);
    output_data__c = String_val(output_data);
    result__c = gnutls_x509_crt_get_serial(cert__c,output_data__c,&output_data_size__c);
  };
  net_gnutls_error_check(result__c);
  CAMLreturn(output_data);
}

value net_gnutls_x509_crt_get_pk_algorithm(value cert) {
  gnutls_x509_crt_t cert__c;
  unsigned int bits__c;
  error_code result__c;
  CAMLparam1(cert);
  CAMLlocal3(bits,result,local_0);
  cert__c = unwrap_gnutls_x509_crt_t(cert);
  nettls_init();
  result__c = gnutls_x509_crt_get_pk_algorithm(cert__c,&bits__c);
  net_gnutls_error_check(result__c);
  bits = Val_int(bits__c);
  result = wrap_gnutls_pk_algorithm_t(result__c);
  local_0 = caml_alloc(2,0);
  Field(local_0, 0) = result;
  Field(local_0, 1) = bits;
  CAMLreturn(local_0);
}

value net_gnutls_x509_crt_get_pk_rsa_raw(value crt) {
  gnutls_x509_crt_t crt__c;
  str_datum m__c;
  str_datum e__c;
  error_code result__c;
  CAMLparam1(crt);
  CAMLlocal3(m,e,local_0);
  crt__c = unwrap_gnutls_x509_crt_t(crt);
  nettls_init();
  result__c = gnutls_x509_crt_get_pk_rsa_raw(crt__c,&m__c,&e__c);
  net_gnutls_error_check(result__c);
  m = wrap_str_datum(m__c);
  e = wrap_str_datum(e__c);
  local_0 = caml_alloc(2,0);
  Field(local_0, 0) = m;
  Field(local_0, 1) = e;
  CAMLreturn(local_0);
}

value net_gnutls_x509_crt_get_pk_dsa_raw(value crt) {
  gnutls_x509_crt_t crt__c;
  str_datum p__c;
  str_datum q__c;
  str_datum g__c;
  str_datum y__c;
  error_code result__c;
  CAMLparam1(crt);
  CAMLlocal5(p,q,g,y,local_0);
  crt__c = unwrap_gnutls_x509_crt_t(crt);
  nettls_init();
  result__c = gnutls_x509_crt_get_pk_dsa_raw(crt__c,&p__c,&q__c,&g__c,&y__c);
  net_gnutls_error_check(result__c);
  p = wrap_str_datum(p__c);
  q = wrap_str_datum(q__c);
  g = wrap_str_datum(g__c);
  y = wrap_str_datum(y__c);
  local_0 = caml_alloc(4,0);
  Field(local_0, 0) = p;
  Field(local_0, 1) = q;
  Field(local_0, 2) = g;
  Field(local_0, 3) = y;
  CAMLreturn(local_0);
}

value net_gnutls_x509_crt_get_subject_alt_name(value cert,value seq) {
  gnutls_x509_crt_t cert__c;
  unsigned int seq__c;
  void * output_data__c;
  size_t output_data_size__c;
  unsigned int critical__c;
  error_code result__c;
  CAMLparam2(cert,seq);
  CAMLlocal4(output_data,critical,result,local_0);
  cert__c = unwrap_gnutls_x509_crt_t(cert);
  seq__c = uint_val(seq);
  nettls_init();
  output_data__c = NULL;
  output_data_size__c = 0;
  output_data = caml_alloc_string(0);
  result__c = gnutls_x509_crt_get_subject_alt_name(cert__c,seq__c,output_data__c,&output_data_size__c,&critical__c);
  if (result__c == 0 || result__c == GNUTLS_E_SHORT_MEMORY_BUFFER) {
    long n__stub;
    output_data_size__c++;
    n__stub = output_data_size__c;
    output_data__c = stat_alloc(output_data_size__c+1);
    result__c = gnutls_x509_crt_get_subject_alt_name(cert__c,seq__c,output_data__c,&output_data_size__c,&critical__c);
    if (result__c == 0) {
      ((char *) output_data__c)[n__stub] = 0;
      output_data = caml_copy_string(output_data__c);
    };
    stat_free(output_data__c);
  };
  net_gnutls_error_check(result__c);
  critical = Val_bool(critical__c);
  result = wrap_gnutls_x509_subject_alt_name_t(result__c);
  local_0 = caml_alloc(3,0);
  Field(local_0, 0) = result;
  Field(local_0, 1) = output_data;
  Field(local_0, 2) = critical;
  CAMLreturn(local_0);
}

value net_gnutls_x509_crt_get_subject_alt_othername_oid(value cert,value seq) {
  gnutls_x509_crt_t cert__c;
  unsigned int seq__c;
  void * output_data__c;
  size_t output_data_size__c;
  error_code result__c;
  CAMLparam2(cert,seq);
  CAMLlocal3(output_data,result,local_0);
  cert__c = unwrap_gnutls_x509_crt_t(cert);
  seq__c = uint_val(seq);
  nettls_init();
  output_data__c = NULL;
  output_data_size__c = 0;
  output_data = caml_alloc_string(0);
  result__c = gnutls_x509_crt_get_subject_alt_othername_oid(cert__c,seq__c,output_data__c,&output_data_size__c);
  if (result__c == 0 || result__c == GNUTLS_E_SHORT_MEMORY_BUFFER) {
    long n__stub;
    output_data_size__c++;
    n__stub = output_data_size__c;
    output_data__c = stat_alloc(output_data_size__c+1);
    result__c = gnutls_x509_crt_get_subject_alt_othername_oid(cert__c,seq__c,output_data__c,&output_data_size__c);
    if (result__c == 0) {
      ((char *) output_data__c)[n__stub] = 0;
      output_data = caml_copy_string(output_data__c);
    };
    stat_free(output_data__c);
  };
  net_gnutls_error_check(result__c);
  result = wrap_gnutls_x509_subject_alt_name_t(result__c);
  local_0 = caml_alloc(2,0);
  Field(local_0, 0) = result;
  Field(local_0, 1) = output_data;
  CAMLreturn(local_0);
}

value net_gnutls_x509_crt_get_issuer_alt_name(value cert,value seq) {
#ifdef HAVE_FUN_gnutls_x509_crt_get_issuer_alt_name
  gnutls_x509_crt_t cert__c;
  unsigned int seq__c;
  void * output_data__c;
  size_t output_data_size__c;
  unsigned int critical__c;
  error_code result__c;
  CAMLparam2(cert,seq);
  CAMLlocal4(output_data,critical,result,local_0);
  cert__c = unwrap_gnutls_x509_crt_t(cert);
  seq__c = uint_val(seq);
  nettls_init();
  output_data__c = NULL;
  output_data_size__c = 0;
  output_data = caml_alloc_string(0);
  result__c = gnutls_x509_crt_get_issuer_alt_name(cert__c,seq__c,output_data__c,&output_data_size__c,&critical__c);
  if (result__c == 0 || result__c == GNUTLS_E_SHORT_MEMORY_BUFFER) {
    long n__stub;
    output_data_size__c++;
    n__stub = output_data_size__c;
    output_data__c = stat_alloc(output_data_size__c+1);
    result__c = gnutls_x509_crt_get_issuer_alt_name(cert__c,seq__c,output_data__c,&output_data_size__c,&critical__c);
    if (result__c == 0) {
      ((char *) output_data__c)[n__stub] = 0;
      output_data = caml_copy_string(output_data__c);
    };
    stat_free(output_data__c);
  };
  net_gnutls_error_check(result__c);
  critical = Val_bool(critical__c);
  result = wrap_gnutls_x509_subject_alt_name_t(result__c);
  local_0 = caml_alloc(3,0);
  Field(local_0, 0) = result;
  Field(local_0, 1) = output_data;
  Field(local_0, 2) = critical;
  CAMLreturn(local_0);
#else
  invalid_argument("gnutls_x509_crt_get_issuer_alt_name");
#endif
}

value net_gnutls_x509_crt_get_issuer_alt_othername_oid(value cert,value seq) {
#ifdef HAVE_FUN_gnutls_x509_crt_get_issuer_alt_othername_oid
  gnutls_x509_crt_t cert__c;
  unsigned int seq__c;
  void * output_data__c;
  size_t output_data_size__c;
  error_code result__c;
  CAMLparam2(cert,seq);
  CAMLlocal3(output_data,result,local_0);
  cert__c = unwrap_gnutls_x509_crt_t(cert);
  seq__c = uint_val(seq);
  nettls_init();
  output_data__c = NULL;
  output_data_size__c = 0;
  output_data = caml_alloc_string(0);
  result__c = gnutls_x509_crt_get_issuer_alt_othername_oid(cert__c,seq__c,output_data__c,&output_data_size__c);
  if (result__c == 0 || result__c == GNUTLS_E_SHORT_MEMORY_BUFFER) {
    long n__stub;
    output_data_size__c++;
    n__stub = output_data_size__c;
    output_data__c = stat_alloc(output_data_size__c+1);
    result__c = gnutls_x509_crt_get_issuer_alt_othername_oid(cert__c,seq__c,output_data__c,&output_data_size__c);
    if (result__c == 0) {
      ((char *) output_data__c)[n__stub] = 0;
      output_data = caml_copy_string(output_data__c);
    };
    stat_free(output_data__c);
  };
  net_gnutls_error_check(result__c);
  result = wrap_gnutls_x509_subject_alt_name_t(result__c);
  local_0 = caml_alloc(2,0);
  Field(local_0, 0) = result;
  Field(local_0, 1) = output_data;
  CAMLreturn(local_0);
#else
  invalid_argument("gnutls_x509_crt_get_issuer_alt_othername_oid");
#endif
}

value net_gnutls_x509_crt_get_ca_status(value cert) {
  gnutls_x509_crt_t cert__c;
  unsigned int critical__c;
  error_code result__c;
  CAMLparam1(cert);
  CAMLlocal1(critical);
  cert__c = unwrap_gnutls_x509_crt_t(cert);
  nettls_init();
  result__c = gnutls_x509_crt_get_ca_status(cert__c,&critical__c);
  net_gnutls_error_check(result__c);
  critical = Val_bool(critical__c);
  CAMLreturn(critical);
}

value net_gnutls_x509_crt_get_basic_constraints(value cert) {
  gnutls_x509_crt_t cert__c;
  unsigned int critical__c;
  unsigned int ca__c;
  int pathlen__c;
  error_code result__c;
  CAMLparam1(cert);
  CAMLlocal4(critical,ca,pathlen,local_0);
  cert__c = unwrap_gnutls_x509_crt_t(cert);
  nettls_init();
  result__c = gnutls_x509_crt_get_basic_constraints(cert__c,&critical__c,&ca__c,&pathlen__c);
  net_gnutls_error_check(result__c);
  critical = Val_bool(critical__c);
  ca = Val_bool(ca__c);
  pathlen = Val_int(pathlen__c);
  local_0 = caml_alloc(3,0);
  Field(local_0, 0) = critical;
  Field(local_0, 1) = ca;
  Field(local_0, 2) = pathlen;
  CAMLreturn(local_0);
}

value net_gnutls_x509_crt_get_key_usage(value cert) {
  gnutls_x509_crt_t cert__c;
  key_usage key_usage__c;
  unsigned int critical__c;
  error_code result__c;
  CAMLparam1(cert);
  CAMLlocal3(key_usage,critical,local_0);
  cert__c = unwrap_gnutls_x509_crt_t(cert);
  nettls_init();
  result__c = gnutls_x509_crt_get_key_usage(cert__c,&key_usage__c,&critical__c);
  net_gnutls_error_check(result__c);
  key_usage = wrap_key_usage(key_usage__c);
  critical = Val_bool(critical__c);
  local_0 = caml_alloc(2,0);
  Field(local_0, 0) = key_usage;
  Field(local_0, 1) = critical;
  CAMLreturn(local_0);
}

value net_gnutls_x509_crt_get_extension_oid(value cert,value indx) {
  gnutls_x509_crt_t cert__c;
  int indx__c;
  void * output_data__c;
  size_t output_data_size__c;
  error_code result__c;
  CAMLparam2(cert,indx);
  CAMLlocal1(output_data);
  cert__c = unwrap_gnutls_x509_crt_t(cert);
  indx__c = Int_val(indx);
  nettls_init();
  output_data__c = NULL;
  output_data_size__c = 0;
  output_data = caml_alloc_string(0);
  result__c = gnutls_x509_crt_get_extension_oid(cert__c,indx__c,output_data__c,&output_data_size__c);
  if (result__c == 0 || result__c == GNUTLS_E_SHORT_MEMORY_BUFFER) {
    long n__stub;
    output_data_size__c++;
    n__stub = output_data_size__c;
    output_data__c = stat_alloc(output_data_size__c+1);
    result__c = gnutls_x509_crt_get_extension_oid(cert__c,indx__c,output_data__c,&output_data_size__c);
    if (result__c == 0) {
      ((char *) output_data__c)[n__stub] = 0;
      output_data = caml_copy_string(output_data__c);
    };
    stat_free(output_data__c);
  };
  net_gnutls_error_check(result__c);
  CAMLreturn(output_data);
}

value net_gnutls_x509_crt_get_extension_info(value cert,value indx) {
  gnutls_x509_crt_t cert__c;
  int indx__c;
  void * output_data__c;
  size_t output_data_size__c;
  unsigned int critical__c;
  error_code result__c;
  CAMLparam2(cert,indx);
  CAMLlocal3(output_data,critical,local_0);
  cert__c = unwrap_gnutls_x509_crt_t(cert);
  indx__c = Int_val(indx);
  nettls_init();
  output_data__c = NULL;
  output_data_size__c = 0;
  output_data = caml_alloc_string(0);
  result__c = gnutls_x509_crt_get_extension_info(cert__c,indx__c,output_data__c,&output_data_size__c,&critical__c);
  if (result__c == 0 || result__c == GNUTLS_E_SHORT_MEMORY_BUFFER) {
    long n__stub;
    output_data_size__c++;
    n__stub = output_data_size__c;
    output_data__c = stat_alloc(output_data_size__c+1);
    result__c = gnutls_x509_crt_get_extension_info(cert__c,indx__c,output_data__c,&output_data_size__c,&critical__c);
    if (result__c == 0) {
      ((char *) output_data__c)[n__stub] = 0;
      output_data = caml_copy_string(output_data__c);
    };
    stat_free(output_data__c);
  };
  net_gnutls_error_check(result__c);
  critical = Val_bool(critical__c);
  local_0 = caml_alloc(2,0);
  Field(local_0, 0) = output_data;
  Field(local_0, 1) = critical;
  CAMLreturn(local_0);
}

value net_gnutls_x509_crt_get_extension_data(value cert,value indx) {
  gnutls_x509_crt_t cert__c;
  int indx__c;
  void * output_data__c;
  size_t output_data_size__c;
  error_code result__c;
  CAMLparam2(cert,indx);
  CAMLlocal1(output_data);
  cert__c = unwrap_gnutls_x509_crt_t(cert);
  indx__c = Int_val(indx);
  nettls_init();
  output_data__c = NULL;
  output_data_size__c = 0;
  output_data = caml_alloc_string(0);
  result__c = gnutls_x509_crt_get_extension_data(cert__c,indx__c,output_data__c,&output_data_size__c);
  if (result__c == 0 || result__c == GNUTLS_E_SHORT_MEMORY_BUFFER) {
    output_data = caml_alloc_string(output_data_size__c);
    output_data__c = String_val(output_data);
    result__c = gnutls_x509_crt_get_extension_data(cert__c,indx__c,output_data__c,&output_data_size__c);
  };
  net_gnutls_error_check(result__c);
  CAMLreturn(output_data);
}

value net_gnutls_x509_crt_check_issuer(value cert,value issuer) {
  gnutls_x509_crt_t cert__c;
  gnutls_x509_crt_t issuer__c;
  int result__c;
  CAMLparam2(cert,issuer);
  CAMLlocal1(result);
  cert__c = unwrap_gnutls_x509_crt_t(cert);
  issuer__c = unwrap_gnutls_x509_crt_t(issuer);
  nettls_init();
  result__c = gnutls_x509_crt_check_issuer(cert__c,issuer__c);
  result = Val_bool(result__c);
  CAMLreturn(result);
}

value net_gnutls_x509_crt_list_verify(value cert_list,value ca_list,value crl_list,value flags) {
  gnutls_x509_crt_t * cert_list__c;
  long local_0;
  size_t cert_list_length__c;
  gnutls_x509_crt_t * ca_list__c;
  long local_1;
  size_t ca_list_length__c;
  gnutls_x509_crl_t * crl_list__c;
  long local_2;
  size_t crl_list_length__c;
  gnutls_certificate_verify_flags flags__c;
  gnutls_certificate_status_t verify__c;
  error_code result__c;
  CAMLparam4(cert_list,ca_list,crl_list,flags);
  CAMLlocal1(verify);
  cert_list__c = (gnutls_x509_crt_t *) stat_alloc(Wosize_val(cert_list)*sizeof(gnutls_x509_crt_t));
  for (local_0=0; local_0 < Wosize_val(cert_list); local_0++) {
    cert_list__c[local_0] = unwrap_gnutls_x509_crt_t(Field(cert_list,local_0));
  };
  cert_list_length__c = (size_t) Wosize_val(cert_list);
  ca_list__c = (gnutls_x509_crt_t *) stat_alloc(Wosize_val(ca_list)*sizeof(gnutls_x509_crt_t));
  for (local_1=0; local_1 < Wosize_val(ca_list); local_1++) {
    ca_list__c[local_1] = unwrap_gnutls_x509_crt_t(Field(ca_list,local_1));
  };
  ca_list_length__c = (size_t) Wosize_val(ca_list);
  crl_list__c = (gnutls_x509_crl_t *) stat_alloc(Wosize_val(crl_list)*sizeof(gnutls_x509_crl_t));
  for (local_2=0; local_2 < Wosize_val(crl_list); local_2++) {
    crl_list__c[local_2] = unwrap_gnutls_x509_crl_t(Field(crl_list,local_2));
  };
  crl_list_length__c = (size_t) Wosize_val(crl_list);
  flags__c = unwrap_gnutls_certificate_verify_flags(flags);
  nettls_init();
  result__c = gnutls_x509_crt_list_verify(cert_list__c,cert_list_length__c,ca_list__c,ca_list_length__c,crl_list__c,crl_list_length__c,flags__c,&verify__c);
  stat_free(cert_list__c);
  stat_free(ca_list__c);
  stat_free(crl_list__c);
  net_gnutls_error_check(result__c);
  verify = wrap_gnutls_certificate_status_t(verify__c);
  CAMLreturn(verify);
}

value net_gnutls_x509_crt_verify(value cert,value ca_list,value flags) {
  gnutls_x509_crt_t cert__c;
  gnutls_x509_crt_t * ca_list__c;
  long local_0;
  size_t ca_list_length__c;
  gnutls_certificate_verify_flags flags__c;
  gnutls_certificate_status_t verify__c;
  error_code result__c;
  CAMLparam3(cert,ca_list,flags);
  CAMLlocal1(verify);
  cert__c = unwrap_gnutls_x509_crt_t(cert);
  ca_list__c = (gnutls_x509_crt_t *) stat_alloc(Wosize_val(ca_list)*sizeof(gnutls_x509_crt_t));
  for (local_0=0; local_0 < Wosize_val(ca_list); local_0++) {
    ca_list__c[local_0] = unwrap_gnutls_x509_crt_t(Field(ca_list,local_0));
  };
  ca_list_length__c = (size_t) Wosize_val(ca_list);
  flags__c = unwrap_gnutls_certificate_verify_flags(flags);
  nettls_init();
  result__c = gnutls_x509_crt_verify(cert__c,ca_list__c,ca_list_length__c,flags__c,&verify__c);
  stat_free(ca_list__c);
  net_gnutls_error_check(result__c);
  verify = wrap_gnutls_certificate_status_t(verify__c);
  CAMLreturn(verify);
}

value net_gnutls_x509_crl_verify(value crl,value ca_list,value flags) {
  gnutls_x509_crl_t crl__c;
  gnutls_x509_crt_t * ca_list__c;
  long local_0;
  size_t ca_list_length__c;
  gnutls_certificate_verify_flags flags__c;
  gnutls_certificate_status_t verify__c;
  error_code result__c;
  CAMLparam3(crl,ca_list,flags);
  CAMLlocal1(verify);
  crl__c = unwrap_gnutls_x509_crl_t(crl);
  ca_list__c = (gnutls_x509_crt_t *) stat_alloc(Wosize_val(ca_list)*sizeof(gnutls_x509_crt_t));
  for (local_0=0; local_0 < Wosize_val(ca_list); local_0++) {
    ca_list__c[local_0] = unwrap_gnutls_x509_crt_t(Field(ca_list,local_0));
  };
  ca_list_length__c = (size_t) Wosize_val(ca_list);
  flags__c = unwrap_gnutls_certificate_verify_flags(flags);
  nettls_init();
  result__c = gnutls_x509_crl_verify(crl__c,ca_list__c,ca_list_length__c,flags__c,&verify__c);
  stat_free(ca_list__c);
  net_gnutls_error_check(result__c);
  verify = wrap_gnutls_certificate_status_t(verify__c);
  CAMLreturn(verify);
}

value net_gnutls_x509_crt_check_revocation(value cert,value crl_list) {
  gnutls_x509_crt_t cert__c;
  gnutls_x509_crl_t * crl_list__c;
  long local_0;
  size_t crl_list_length__c;
  error_code result__c;
  CAMLparam2(cert,crl_list);
  CAMLlocal1(result);
  cert__c = unwrap_gnutls_x509_crt_t(cert);
  crl_list__c = (gnutls_x509_crl_t *) stat_alloc(Wosize_val(crl_list)*sizeof(gnutls_x509_crl_t));
  for (local_0=0; local_0 < Wosize_val(crl_list); local_0++) {
    crl_list__c[local_0] = unwrap_gnutls_x509_crl_t(Field(crl_list,local_0));
  };
  crl_list_length__c = (size_t) Wosize_val(crl_list);
  nettls_init();
  result__c = gnutls_x509_crt_check_revocation(cert__c,crl_list__c,crl_list_length__c);
  stat_free(crl_list__c);
  net_gnutls_error_check(result__c);
  result = Val_bool(result__c);
  CAMLreturn(result);
}

value net_gnutls_x509_crt_get_fingerprint(value cert,value algo) {
  gnutls_x509_crt_t cert__c;
  gnutls_digest_algorithm_t algo__c;
  void * output_data__c;
  size_t output_data_size__c;
  error_code result__c;
  CAMLparam2(cert,algo);
  CAMLlocal1(output_data);
  cert__c = unwrap_gnutls_x509_crt_t(cert);
  algo__c = unwrap_gnutls_digest_algorithm_t(algo);
  nettls_init();
  output_data__c = NULL;
  output_data_size__c = 0;
  output_data = caml_alloc_string(0);
  result__c = gnutls_x509_crt_get_fingerprint(cert__c,algo__c,output_data__c,&output_data_size__c);
  if (result__c == 0 || result__c == GNUTLS_E_SHORT_MEMORY_BUFFER) {
    output_data = caml_alloc_string(output_data_size__c);
    output_data__c = String_val(output_data);
    result__c = gnutls_x509_crt_get_fingerprint(cert__c,algo__c,output_data__c,&output_data_size__c);
  };
  net_gnutls_error_check(result__c);
  CAMLreturn(output_data);
}

value net_gnutls_x509_crt_get_key_purpose_oid(value cert,value indx) {
  gnutls_x509_crt_t cert__c;
  int indx__c;
  void * output_data__c;
  size_t output_data_size__c;
  unsigned int critical__c;
  error_code result__c;
  CAMLparam2(cert,indx);
  CAMLlocal3(output_data,critical,local_0);
  cert__c = unwrap_gnutls_x509_crt_t(cert);
  indx__c = Int_val(indx);
  nettls_init();
  output_data__c = NULL;
  output_data_size__c = 0;
  output_data = caml_alloc_string(0);
  result__c = gnutls_x509_crt_get_key_purpose_oid(cert__c,indx__c,output_data__c,&output_data_size__c,&critical__c);
  if (result__c == 0 || result__c == GNUTLS_E_SHORT_MEMORY_BUFFER) {
    long n__stub;
    output_data_size__c++;
    n__stub = output_data_size__c;
    output_data__c = stat_alloc(output_data_size__c+1);
    result__c = gnutls_x509_crt_get_key_purpose_oid(cert__c,indx__c,output_data__c,&output_data_size__c,&critical__c);
    if (result__c == 0) {
      ((char *) output_data__c)[n__stub] = 0;
      output_data = caml_copy_string(output_data__c);
    };
    stat_free(output_data__c);
  };
  net_gnutls_error_check(result__c);
  critical = Val_bool(critical__c);
  local_0 = caml_alloc(2,0);
  Field(local_0, 0) = output_data;
  Field(local_0, 1) = critical;
  CAMLreturn(local_0);
}

value net_gnutls_x509_privkey_init(value dummy) {
  gnutls_x509_privkey_t cert__c;
  error_code result__c;
  CAMLparam1(dummy);
  CAMLlocal1(cert);
  nettls_init();
  result__c = gnutls_x509_privkey_init(&cert__c);
  net_gnutls_error_check(result__c);
  cert = wrap_gnutls_x509_privkey_t(cert__c);
  CAMLreturn(cert);
}

value net_gnutls_x509_privkey_import(value key,value data,value format) {
  gnutls_x509_privkey_t key__c;
  const_str_datum_p data__c;
  gnutls_x509_crt_fmt_t format__c;
  error_code result__c;
  CAMLparam3(key,data,format);
  key__c = unwrap_gnutls_x509_privkey_t(key);
  data__c = unwrap_const_str_datum_p(data);
  format__c = unwrap_gnutls_x509_crt_fmt_t(format);
  nettls_init();
  result__c = gnutls_x509_privkey_import(key__c,data__c,format__c);
  net_gnutls_error_check(result__c);
  CAMLreturn(Val_unit);
}

value net_gnutls_x509_privkey_import_pkcs8(value key,value data,value format,value password,value flags) {
  gnutls_x509_privkey_t key__c;
  const_str_datum_p data__c;
  gnutls_x509_crt_fmt_t format__c;
  char * password__c;
  gnutls_pkcs_encrypt_flags_t flags__c;
  error_code result__c;
  CAMLparam5(key,data,format,password,flags);
  key__c = unwrap_gnutls_x509_privkey_t(key);
  data__c = unwrap_const_str_datum_p(data);
  format__c = unwrap_gnutls_x509_crt_fmt_t(format);
  password__c = String_val(password);
  flags__c = unwrap_gnutls_pkcs_encrypt_flags_t(flags);
  nettls_init();
  result__c = gnutls_x509_privkey_import_pkcs8(key__c,data__c,format__c,password__c,flags__c);
  net_gnutls_error_check(result__c);
  CAMLreturn(Val_unit);
}

value net_gnutls_x509_crl_init(value dummy) {
  gnutls_x509_crl_t cert__c;
  error_code result__c;
  CAMLparam1(dummy);
  CAMLlocal1(cert);
  nettls_init();
  result__c = gnutls_x509_crl_init(&cert__c);
  net_gnutls_error_check(result__c);
  cert = wrap_gnutls_x509_crl_t(cert__c);
  CAMLreturn(cert);
}

value net_gnutls_x509_crl_import(value key,value data,value format) {
  gnutls_x509_crl_t key__c;
  const_str_datum_p data__c;
  gnutls_x509_crt_fmt_t format__c;
  error_code result__c;
  CAMLparam3(key,data,format);
  key__c = unwrap_gnutls_x509_crl_t(key);
  data__c = unwrap_const_str_datum_p(data);
  format__c = unwrap_gnutls_x509_crt_fmt_t(format);
  nettls_init();
  result__c = gnutls_x509_crl_import(key__c,data__c,format__c);
  net_gnutls_error_check(result__c);
  CAMLreturn(Val_unit);
}

value net_net_have_pubkey(value dummy) {
  int result__c;
  CAMLparam1(dummy);
  CAMLlocal1(result);
  nettls_init();
  result__c = net_have_pubkey();
  result = Val_bool(result__c);
  CAMLreturn(result);
}

value net_gnutls_pubkey_init(value dummy) {
#ifdef HAVE_FUN_gnutls_pubkey_init
  gnutls_pubkey_t key__c;
  error_code result__c;
  CAMLparam1(dummy);
  CAMLlocal1(key);
  nettls_init();
  result__c = gnutls_pubkey_init(&key__c);
  net_gnutls_error_check(result__c);
  key = wrap_gnutls_pubkey_t(key__c);
  CAMLreturn(key);
#else
  invalid_argument("gnutls_pubkey_init");
#endif
}

value net_gnutls_privkey_init(value dummy) {
#ifdef HAVE_FUN_gnutls_privkey_init
  gnutls_privkey_t key__c;
  error_code result__c;
  CAMLparam1(dummy);
  CAMLlocal1(key);
  nettls_init();
  result__c = gnutls_privkey_init(&key__c);
  net_gnutls_error_check(result__c);
  key = wrap_gnutls_privkey_t(key__c);
  CAMLreturn(key);
#else
  invalid_argument("gnutls_privkey_init");
#endif
}

value net_gnutls_pubkey_import(value key,value data,value format) {
#ifdef HAVE_FUN_gnutls_pubkey_import
  gnutls_pubkey_t key__c;
  const_str_datum_p data__c;
  gnutls_x509_crt_fmt_t format__c;
  error_code result__c;
  CAMLparam3(key,data,format);
  key__c = unwrap_gnutls_pubkey_t(key);
  data__c = unwrap_const_str_datum_p(data);
  format__c = unwrap_gnutls_x509_crt_fmt_t(format);
  nettls_init();
  result__c = gnutls_pubkey_import(key__c,data__c,format__c);
  net_gnutls_error_check(result__c);
  CAMLreturn(Val_unit);
#else
  invalid_argument("gnutls_pubkey_import");
#endif
}

value net_gnutls_pubkey_import_url(value key,value url,value flags) {
#ifdef HAVE_FUN_gnutls_pubkey_import_url
  gnutls_pubkey_t key__c;
  const char * url__c;
  unsigned int flags__c;
  error_code result__c;
  CAMLparam3(key,url,flags);
  key__c = unwrap_gnutls_pubkey_t(key);
  url__c = String_val(url);
  flags__c = uint_val(flags);
  nettls_init();
  result__c = gnutls_pubkey_import_url(key__c,url__c,flags__c);
  net_gnutls_error_check(result__c);
  CAMLreturn(Val_unit);
#else
  invalid_argument("gnutls_pubkey_import_url");
#endif
}

value net_gnutls_pubkey_import_privkey(value key,value pkey,value usage,value flags) {
#ifdef HAVE_FUN_gnutls_pubkey_import_privkey
  gnutls_pubkey_t key__c;
  gnutls_privkey_t pkey__c;
  unsigned int usage__c;
  unsigned int flags__c;
  error_code result__c;
  CAMLparam4(key,pkey,usage,flags);
  key__c = unwrap_gnutls_pubkey_t(key);
  pkey__c = unwrap_gnutls_privkey_t(pkey);
  usage__c = uint_val(usage);
  flags__c = uint_val(flags);
  nettls_init();
  result__c = gnutls_pubkey_import_privkey(key__c,pkey__c,usage__c,flags__c);
  net_gnutls_error_check(result__c);
  CAMLreturn(Val_unit);
#else
  invalid_argument("gnutls_pubkey_import_privkey");
#endif
}

value net_gnutls_privkey_import_x509(value key,value x509key,value flags) {
#ifdef HAVE_FUN_gnutls_privkey_import_x509
  gnutls_privkey_t key__c;
  gnutls_x509_privkey_t x509key__c;
  unsigned int flags__c;
  error_code result__c;
  CAMLparam3(key,x509key,flags);
  key__c = unwrap_gnutls_privkey_t(key);
  x509key__c = unwrap_gnutls_x509_privkey_t(x509key);
  flags__c = uint_val(flags);
  nettls_init();
  result__c = gnutls_privkey_import_x509(key__c,x509key__c,flags__c);
  net_gnutls_error_check(result__c);
  CAMLreturn(Val_unit);
#else
  invalid_argument("gnutls_privkey_import_x509");
#endif
}

value net_gnutls_pubkey_encrypt_data(value key,value flags,value plaintext) {
#ifdef HAVE_FUN_gnutls_pubkey_encrypt_data
  gnutls_pubkey_t key__c;
  unsigned int flags__c;
  const_str_datum_p plaintext__c;
  str_datum ciphertext__c;
  error_code result__c;
  CAMLparam3(key,flags,plaintext);
  CAMLlocal1(ciphertext);
  key__c = unwrap_gnutls_pubkey_t(key);
  flags__c = uint_val(flags);
  plaintext__c = unwrap_const_str_datum_p(plaintext);
  nettls_init();
  result__c = gnutls_pubkey_encrypt_data(key__c,flags__c,plaintext__c,&ciphertext__c);
  net_gnutls_error_check(result__c);
  ciphertext = wrap_str_datum(ciphertext__c);
  CAMLreturn(ciphertext);
#else
  invalid_argument("gnutls_pubkey_encrypt_data");
#endif
}

value net_gnutls_privkey_decrypt_data(value key,value flags,value ciphertext) {
#ifdef HAVE_FUN_gnutls_privkey_decrypt_data
  gnutls_privkey_t key__c;
  unsigned int flags__c;
  const_str_datum_p ciphertext__c;
  str_datum plaintext__c;
  error_code result__c;
  CAMLparam3(key,flags,ciphertext);
  CAMLlocal1(plaintext);
  key__c = unwrap_gnutls_privkey_t(key);
  flags__c = uint_val(flags);
  ciphertext__c = unwrap_const_str_datum_p(ciphertext);
  nettls_init();
  result__c = gnutls_privkey_decrypt_data(key__c,flags__c,ciphertext__c,&plaintext__c);
  net_gnutls_error_check(result__c);
  plaintext = wrap_str_datum(plaintext__c);
  CAMLreturn(plaintext);
#else
  invalid_argument("gnutls_privkey_decrypt_data");
#endif
}

value net_gnutls_privkey_sign_data(value key,value hash,value flags,value data) {
#ifdef HAVE_FUN_gnutls_privkey_sign_data
  gnutls_privkey_t key__c;
  gnutls_digest_algorithm_t hash__c;
  unsigned int flags__c;
  const_str_datum_p data__c;
  str_datum signature__c;
  error_code result__c;
  CAMLparam4(key,hash,flags,data);
  CAMLlocal1(signature);
  key__c = unwrap_gnutls_privkey_t(key);
  hash__c = unwrap_gnutls_digest_algorithm_t(hash);
  flags__c = uint_val(flags);
  data__c = unwrap_const_str_datum_p(data);
  nettls_init();
  result__c = gnutls_privkey_sign_data(key__c,hash__c,flags__c,data__c,&signature__c);
  net_gnutls_error_check(result__c);
  signature = wrap_str_datum(signature__c);
  CAMLreturn(signature);
#else
  invalid_argument("gnutls_privkey_sign_data");
#endif
}

value net_gnutls_pubkey_verify_data2(value key,value algo,value flags,value data,value signature) {
#ifdef HAVE_FUN_gnutls_pubkey_verify_data2
  gnutls_pubkey_t key__c;
  gnutls_sign_algorithm_t algo__c;
  unsigned int flags__c;
  const_str_datum_p data__c;
  const_str_datum_p signature__c;
  error_code result__c;
  CAMLparam5(key,algo,flags,data,signature);
  key__c = unwrap_gnutls_pubkey_t(key);
  algo__c = unwrap_gnutls_sign_algorithm_t(algo);
  flags__c = uint_val(flags);
  data__c = unwrap_const_str_datum_p(data);
  signature__c = unwrap_const_str_datum_p(signature);
  nettls_init();
  result__c = gnutls_pubkey_verify_data2(key__c,algo__c,flags__c,data__c,signature__c);
  net_gnutls_error_check(result__c);
  CAMLreturn(Val_unit);
#else
  invalid_argument("gnutls_pubkey_verify_data2");
#endif
}

